/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var kimchi = {

    url : "../../../",

    widget: {},

    trackingTasks: [],

    /**
     * A wrapper of jQuery.ajax function to allow custom bindings.
     *
     * @param settings an extended object to jQuery Ajax settings object
     *   with some extra properties (see below)
     *
     *   resend: if the XHR has failed due to 401, the XHR can be resent
     *     after user being authenticated successfully by setting resend
     *     to true: settings = {resend: true}. It's useful for switching
     *     pages (Guests, Templates, etc.).
     *       e.g., the user wants to list guests by clicking Guests tab,
     *     but he is told not authorized and a login window will pop up.
     *     After login, the Ajax request for /vms will be resent without
     *     user clicking the tab again.
     *       Default to false.
     */
    requestJSON : function(settings) {
        settings['originalError'] = settings['error'];
        settings['error'] = null;
        settings['kimchi'] = true;
        return $.ajax(settings);
    },

    /**
     *
     * Get host capabilities
     * suc: callback if succeed err: callback if failed
     */
    getCapabilities : function(suc, err, done) {
        done = typeof done !== 'undefined' ? done: function(){};
        kimchi.requestJSON({
            url : "config/capabilities",
            type : "GET",
            contentType : "application/json",
            dataType : "json",
            success: suc,
            error: err,
            complete: done
        });
    },

    /**
     * Get the i18 strings.
     */
    getI18n: function(suc, err, url, sync) {
        kimchi.requestJSON({
            url : url ? url : 'i18n.json',
            type : 'GET',
            resend: true,
            dataType : 'json',
            async : !sync,
            success : suc,
            error: err
        });
    },

    /**
     * Get the host static information.
     */
    getHost: function(suc, err) {
        kimchi.requestJSON({
            url : 'host',
            type : 'GET',
            resend: true,
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    /**
     * Get the dynamic host stats (usually used for monitoring).
     */
    getHostStats : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/stats',
            type : 'GET',
            contentType : 'application/json',
            headers: {'Kimchi-Robot': 'kimchi-robot'},
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    /**
     * Get the historic host stats.
     */
    getHostStatsHistory : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/stats/history',
            type : 'GET',
            resend: true,
            contentType : 'application/json',
            headers: {'Kimchi-Robot': 'kimchi-robot'},
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    /**
     *
     * Create a new Virtual Machine. Usage: kimchi.createVM({ name: 'MyUbuntu',
     * template: '/templates/ubuntu_base' }, creationSuc, creationErr);
     *
     * settings: name *(optional)*: The name of the VM. Used to identify the VM
     * in this API. If omitted, a name will be chosen based on the template
     * used. template: The URI of a Template to use when building the VM
     * storagepool *(optional)*: Assign a specific Storage Pool to the new VM
     * suc: callback if succeed err: callback if failed
     */
    createVM : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "vms",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json"
        }).done(suc).fail(err);
    },

    /**
     *
     * Create a new Template. settings name: The name of the Template. Used to
     * identify the Template in this API suc: callback if succeed err: callback
     * if failed
     */
    createTemplate : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "templates",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success: suc,
            error: err
        });
    },

    deleteTemplate : function(tem, suc, err) {
        kimchi.requestJSON({
            url : 'templates/' + encodeURIComponent(tem),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    cloneTemplate : function(tem, suc, err) {
        kimchi.requestJSON({
            url : 'templates/' + encodeURIComponent(tem) + "/clone",
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listTemplates : function(suc, err) {
        kimchi.requestJSON({
            url : 'templates',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    /**
     * Retrieve the information of a template by the given name.
     */
    retrieveTemplate : function(templateName, suc, err) {
        kimchi.requestJSON({
            url : "templates/" + encodeURIComponent(templateName),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json'
        }).done(suc);
    },

    /**
     * Update a template with new information. TODO: Update me when the RESTful
     * API is available. Now work it around by remove the template and then
     * recreate it with new information.
     */
    updateTemplate : function(name, settings, suc, err) {
        $.ajax({
            url : "templates/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json'
        }).done(suc).fail(err);
    },

    /**
     * Create a new Storage Pool. settings name: The name of the Storage Pool
     * path: The path of the defined Storage Pool type: The type of the defined
     * Storage Pool capacity: The total space which can be used to store volumes
     * The unit is MBytes suc: callback if succeed err: callback if failed
     */
    createStoragePool : function(settings, suc, err) {
        kimchi.requestJSON({
            url : 'storagepools',
            type : 'POST',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json'
        }).done(suc).fail(err);
    },

    updateStoragePool : function(name, content, suc, err) {
        $.ajax({
            url : "storagepools/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            dataType : 'json',
            data : JSON.stringify(content)
        }).done(suc).fail(err ? err : function(data) {
            kimchi.message.error(data.responseJSON.reason);
        });
    },

    startVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/start',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    poweroffVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/poweroff',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    shutdownVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/shutdown',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    resetVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/reset',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    suspendVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/suspend',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    resumeVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) + '/resume',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
     },

    /**
     * Retrieve the information of a given VM by its name.
     *
     * @param vm VM name
     * @param suc callback for success
     * @param err callback for error
     */
    retrieveVM : function(vm, suc, err) {
        $.ajax({
            url : 'vms/' + encodeURIComponent(vm),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    /**
     * Update a VM with new information.
     */
    updateVM : function(name, settings, suc, err) {
        $.ajax({
            url : "vms/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    deleteVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    vncToVM : function(vm) {
        kimchi.requestJSON({
            url : 'config',
            type : 'GET',
            dataType : 'json'
        }).done(function(data, textStatus, xhr) {
            proxy_port = data['display_proxy_port'];
            kimchi.requestJSON({
                url : "vms/" + encodeURIComponent(vm) + "/connect",
                type : "POST",
                dataType : "json"
            }).done(function() {
                url = 'https://' + location.hostname + ':' + proxy_port;
                url += "/console.html?url=" + encodeURIComponent("novnc/vnc_auto.html");
                url += "&port=" + proxy_port;
                /*
                 * From python documentation base64.urlsafe_b64encode(s)
                 * substitutes - instead of + and _ instead of / in the
                 * standard Base64 alphabet, BUT the result can still
                 * contain = which is not safe in a URL query component.
                 * So remove it when needed as base64 can work well without it.
                 * */
                url += "&path=?token=" + kimchi.urlSafeB64Encode(vm).replace(/=*$/g, "");
                url += "&kimchi=" + location.port;
                url += '&encrypt=1';
                window.open(url);
            });
        }).error(function() {
            kimchi.message.error.code('KCHAPI6002E');
        });
    },

    spiceToVM : function(vm) {
        kimchi.requestJSON({
            url : 'config',
            type : 'GET',
            dataType : 'json'
        }).done(function(data, textStatus, xhr) {
            proxy_port = data['display_proxy_port'];
            kimchi.requestJSON({
                url : "vms/" + encodeURIComponent(vm) + "/connect",
                type : "POST",
                dataType : "json"
            }).done(function(data, textStatus, xhr) {
                url = 'https://' + location.hostname + ':' + proxy_port;
                url += "/console.html?url=spice_auto.html&port=" + proxy_port;
                url += "&listen=" + location.hostname;
                /*
                 * From python documentation base64.urlsafe_b64encode(s)
                 * substitutes - instead of + and _ instead of / in the
                 * standard Base64 alphabet, BUT the result can still
                 * contain = which is not safe in a URL query component.
                 * So remove it when needed as base64 can work well without it.
                 * */
                url += "&token=" + kimchi.urlSafeB64Encode(vm).replace(/=*$/g, "");
                url += "&kimchi=" + location.port;
                url += '&encrypt=1';
                window.open(url);
            });
        }).error(function() {
            kimchi.message.error.code('KCHAPI6002E');
        });
    },

    listVMs : function(suc, err) {
        kimchi.requestJSON({
            url : 'vms',
            type : 'GET',
            contentType : 'application/json',
            headers: {'Kimchi-Robot': 'kimchi-robot'},
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listTemplates : function(suc, err) {
        kimchi.requestJSON({
            url : 'templates',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listStoragePools : function(suc, err) {
        kimchi.requestJSON({
            url : 'storagepools',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listStorageVolumes : function(poolName, suc, err) {
        $.ajax({
            url : 'storagepools/' + encodeURIComponent(poolName) + '/storagevolumes',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listIsos : function(suc, err) {
        kimchi.requestJSON({
            url : 'storagepools/kimchi_isos/storagevolumes',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listDistros : function(suc, err) {
        kimchi.requestJSON({
            url : 'config/distros',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    stepListDeepScanIsos : function(suc, err) {
        var deepScanHandler = {
            stop : false
        };
        var isoPool = 'iso' + new Date().getTime();
        kimchi.createStoragePool({
            name : isoPool,
            type : 'kimchi-iso',
            path : '/'
        }, function(result) {
            var taskId = result.task_id;
            function monitorTask() {
                if (deepScanHandler.stop) {
                    return;
                }
                kimchi.getTask(taskId, function(result) {
                    var status = result.status;
                    if (status === "finished") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        kimchi.listStorageVolumes(isoPool, function(isos) {
                            if (deepScanHandler.stop) {
                                return;
                            }
                            suc(isos, true);
                        }, err);
                    } else if (status === "running") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        kimchi.listStorageVolumes(isoPool, function(isos) {
                            if (deepScanHandler.stop) {
                                return;
                            }
                            suc(isos, false);
                            setTimeout(monitorTask, 2000);
                        }, err);
                    } else if (status === "failed") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        err(result.message);
                    }
                }, err);
            }
            setTimeout(monitorTask, 2000);
        }, err);
        return deepScanHandler;
    },

    getTask : function(taskId, suc, err) {
        kimchi.requestJSON({
            url : 'tasks/' + encodeURIComponent(taskId),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getTasksByFilter : function(filter, suc, err, sync) {
        kimchi.requestJSON({
            url : 'tasks?' + filter,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            async : !sync,
            success : suc,
            error : err
        });
    },

    login : function(settings, suc, err) {
        $.ajax({
            url : "login",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json"
        }).done(suc).fail(err);
    },

    logout : function(suc, err) {
        kimchi.requestJSON({
            url : 'logout',
            type : 'POST',
            contentType : "application/json",
            dataType : "json"
        }).done(suc).fail(err);
    },

    deleteStoragePool : function(poolName, suc, err) {
        $.ajax({
            url : 'storagepools/' + encodeURIComponent(poolName),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    changePoolState : function(poolName, state, suc, err) {
        if (state === 'activate' || state === 'deactivate')
            $.ajax({
                url : 'storagepools/' + encodeURIComponent(poolName) + '/' + state,
                type : 'POST',
                contentType : 'application/json',
                dataType : 'json',
                success : suc,
                error : err
            });
    },

    listPlugins : function(suc, err, sync) {
        kimchi.requestJSON({
            url : 'plugins',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            async : !sync,
            success : suc,
            error : err
        });
     },

    listNetworks : function(suc, err) {
        kimchi.requestJSON({
            url : 'networks',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    toggleNetwork : function(name, on, suc, err) {
        var action = on ? "activate" : "deactivate";
        kimchi.requestJSON({
            url : 'networks/' + encodeURIComponent(name) + '/' + action,
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    createNetwork : function(network, suc, err) {
        kimchi.requestJSON({
            url : 'networks',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            data : JSON.stringify(network),
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getInterfaces : function(suc, err) {
        kimchi.requestJSON({
            url : 'interfaces',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    deleteNetwork : function(name, suc, err) {
        kimchi.requestJSON({
            url : 'networks/' + encodeURIComponent(name),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    listReports : function(suc, err) {
        kimchi.requestJSON({
            url : 'debugreports',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    trackTask : function(taskID, suc, err, progress) {
        var onTaskResponse = function(result) {
            var taskStatus = result['status'];
            switch(taskStatus) {
            case 'running':
                progress && progress(result);
                setTimeout(function() {
                    kimchi.trackTask(taskID, suc, err, progress);
                }, 2000);
                break;
            case 'finished':
                suc && suc(result);
                break;
            case 'failed':
                err && err(result);
                break;
            default:
                break;
            }
        };

        kimchi.getTask(taskID, onTaskResponse, err);
        if(kimchi.trackingTasks.indexOf(taskID) < 0)
            kimchi.trackingTasks.push(taskID);
    },

    createReport: function(settings, suc, err, progress) {
        var onResponse = function(data) {
            taskID = data['id'];
            kimchi.trackTask(taskID, suc, err, progress);
        };

        kimchi.requestJSON({
            url : 'debugreports',
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success : onResponse,
            error : err
        });
    },

    renameReport : function(name, settings, suc, err) {
        $.ajax({
            url : "debugreports/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    deleteReport: function(settings, suc, err) {
        var reportName = encodeURIComponent(settings['name']);
        kimchi.requestJSON({
            url : 'debugreports/' + reportName,
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    downloadReport: function(settings, suc, err) {
        window.open(settings['file']);
    },

    shutdown: function(settings, suc, err) {
        var reboot = settings && settings['reboot'] === true;
        var url = 'host/' + (reboot ? 'reboot' : 'shutdown');
        kimchi.requestJSON({
            url : url,
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listHostPartitions : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/partitions',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getStorageServers: function(type, suc, err) {
        var url = 'storageservers?_target_type=' + type;
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getStorageTargets: function(server,type, suc, err) {
        var url = 'storageservers/' + server + '/storagetargets?_target_type=' + type;
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            timeout: 2000,
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getStoragePool: function(poolName, suc, err) {
        var url = kimchi.url + 'storagepools/' + encodeURIComponent(poolName);
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            timeout: 2000,
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getStoragePoolVolume: function(poolName, volumeName, suc, err) {
        var url = 'storagepools/' + encodeURIComponent(poolName) + '/storagevolumes/' + encodeURIComponent(volumeName);
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            timeout: 2000,
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    addVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        delete settings['vm'];
        kimchi.requestJSON({
            url : 'vms/' + vm + '/storages',
            type : 'POST',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    retrieveVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        var dev = encodeURIComponent(settings['dev']);
        kimchi.requestJSON({
            url : "vms/" + vm + '/storages/' + dev,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    replaceVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        var dev = encodeURIComponent(settings['dev']);
        kimchi.requestJSON({
            url : 'vms/' + vm + '/storages/' + dev,
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify({
                path: settings['path']
            }),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    deleteVMStorage : function(settings, suc, err) {
        var vm = settings['vm'];
        var dev = settings['dev'];
        kimchi.requestJSON({
            url : 'vms/' + encodeURIComponent(vm) +
                      '/storages/' + encodeURIComponent(dev),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listVMStorages : function(params, suc, err) {
        var vm = encodeURIComponent(params['vm']);
        var type = params['storageType'];
        var url = 'vms/' + vm + '/storages';
        if(type) {
            url += '?type=' + type;
        }
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listSoftwareUpdates : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/packagesupdate',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    updateSoftware : function(suc, err, progress) {
        var taskID = -1;
        var onResponse = function(data) {
            taskID = data['id'];
            trackTask();
        };

        var trackTask = function() {
            kimchi.getTask(taskID, onTaskResponse, err);
        };

        var onTaskResponse = function(result) {
            var taskStatus = result['status'];
            switch(taskStatus) {
            case 'running':
                progress && progress(result);
                setTimeout(function() {
                    trackTask();
                }, 200);
                break;
            case 'finished':
            case 'failed':
                suc(result);
                break;
            default:
                break;
            }
        };

        kimchi.requestJSON({
            url : 'host/swupdate',
            type : "POST",
            contentType : "application/json",
            dataType : "json",
            success : onResponse,
            error : err
        });
    },

    createRepository : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "host/repositories",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success: suc,
            error: err
        });
    },

    retrieveRepository : function(repository, suc, err) {
        var reposID = encodeURIComponent(repository);
        kimchi.requestJSON({
            url : "host/repositories/" + reposID,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    updateRepository : function(name, settings, suc, err) {
        var reposID = encodeURIComponent(name);
        $.ajax({
            url : "host/repositories/" + reposID,
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    enableRepository : function(name, enable, suc, err) {
        var reposID = encodeURIComponent(name);
        $.ajax({
            url : "host/repositories/" + reposID +
                '/' + (enable === true ? 'enable' : 'disable'),
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    deleteRepository : function(repository, suc, err) {
        var reposID = encodeURIComponent(repository);
        kimchi.requestJSON({
            url : 'host/repositories/' + reposID,
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listRepositories : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/repositories',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    getHostFCDevices: function(suc, err) {
        var url = 'host/devices?_cap=fc_host';
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getGuestInterfaces: function(name, suc, err) {
        var url = 'vms/'+encodeURIComponent(name)+'/ifaces';
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err || function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    createGuestInterface : function(name, interface, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(name)+'/ifaces',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            data : JSON.stringify(interface),
            success : suc,
            error : err || function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    deleteGuestInterface : function(vm, mac, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/ifaces/'+encodeURIComponent(mac),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    updateGuestInterface : function(vm, mac, interface, suc, err) {
        $.ajax({
            url : 'vms/'+encodeURIComponent(vm)+'/ifaces/'+encodeURIComponent(mac),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(interface),
            dataType : 'json',
            success: suc,
            error: err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getUserById : function(data, suc, err) {
        kimchi.requestJSON({
            url : 'users?_user_id=' + data.user_id,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            async : false,
            success : suc && suc(data),
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getUsers : function(suc, err) {
        kimchi.requestJSON({
            url : 'users',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getGroups : function(suc, err) {
        kimchi.requestJSON({
            url : 'groups',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getHostPCIDevices : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/devices?_passthrough=true&_cap=pci',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getPCIDeviceCompanions : function(pcidev, suc, err) {
        kimchi.requestJSON({
            url : 'host/devices?_passthrough_affected_by=' + pcidev,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getISCSITargets : function(server, port, suc, err) {
        server = encodeURIComponent(server);
        port = port ? '&_server_port='+encodeURIComponent(port) : '';
        kimchi.requestJSON({
            url : 'storageservers/'+server+'/storagetargets?_target_type=iscsi'+port,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getPeers : function(suc, err) {
        kimchi.requestJSON({
            url : 'peers',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getVMPCIDevices : function(id, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(id)+'/hostdevs',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    addVMPCIDevice : function(vm, device, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+ encodeURIComponent(vm) +'/hostdevs',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            data : JSON.stringify(device),
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    removeVMPCIDevice : function(vm, device, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+ encodeURIComponent(vm) +'/hostdevs/' + encodeURIComponent(device),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    /**
     * Create a new volume with capacity
     */
    createVolumeWithCapacity: function(poolName, settings, suc, err) {
        kimchi.requestJSON({
            url : 'storagepools/' + encodeURIComponent(poolName) + '/storagevolumes',
            type : 'POST',
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success : suc,
            error : err
        });
    },

    /**
     * Upload volume content
     */
    uploadVolumeToSP: function(poolName, volumeName, settings, suc, err) {
        var url = 'storagepools/' + encodeURIComponent(poolName) + '/storagevolumes/' + encodeURIComponent(volumeName);
        var fd = settings['formData'];
        kimchi.requestJSON({
            url : url,
            type : 'PUT',
            data : fd,
            processData : false,
            contentType : false,
            dataType: 'json',
            success : suc,
            error : err
        });
    },

    /**
     * Add a volume to a given storage pool by URL.
     */
    downloadVolumeToSP: function(settings, suc, err) {
        var sp = encodeURIComponent(settings['sp']);
        delete settings['sp'];
        kimchi.requestJSON({
            url : 'storagepools/' + sp + '/storagevolumes',
            type : 'POST',
            data : JSON.stringify(settings),
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    cloneGuest: function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+"/clone",
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    listSnapshots : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/snapshots',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getCurrentSnapshot : function(vm, suc, err, sync) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/snapshots/current',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            async : !sync,
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    revertSnapshot : function(vm, snapshot, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/snapshots/'+encodeURIComponent(snapshot)+'/revert',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    createSnapshot : function(vm, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/snapshots',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    deleteSnapshot : function(vm, snapshot, suc, err) {
        kimchi.requestJSON({
            url : 'vms/'+encodeURIComponent(vm)+'/snapshots/'+encodeURIComponent(snapshot),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getCPUInfo : function(suc, err) {
        kimchi.requestJSON({
            url : 'host/cpuinfo',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.cookie = {
    set: function(key, value, expireDays) {
        value = encodeURIComponent(value);
        value += '; secure'
        if (expireDays) {
            var expireDate = new Date();
            expireDate.setDate(expireDate.getDate() + expireDays);
            value += '; expires=' + expireDate.toUTCString();
        }
        document.cookie = key + '=' + value;
    },

    get: function(key) {
        var cookieRe = new RegExp(';?\\\s*(' + key + ')=(\s*[^;]*);?', 'g');
        var match = cookieRe.exec(document.cookie);
        return match ? decodeURIComponent(match[2]) : undefined;
    },

    remove: function(key) {
        var utcString = new Date().toUTCString();
        document.cookie = key + '=; expires=' + utcString;
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function($) {
    $.fn.serializeObject = function() {
        var formDataArray = $(this).serializeArray();
        var formData = new Object();
        $.each(formDataArray, function(index, data) {
            formData.setDeepValue(data.name, data.value);
        });
        return formData;
    };
}(jQuery));

(function($) {
    $.fn.fillWithObject = function(obj) {
        $(this).find("input").each(function(){
            switch($(this).attr('type')) {
                case 'text':
                    $(this).val(obj.getDeepValue($(this).attr("name")));
                    break;
                case 'radio':
                case 'checkbox':
                    var a=String($(this).val());
                    var b=String(obj.getDeepValue($(this).attr("name")));
                    $(this).prop("checked",(a==b));
                    break;
                default:
                    break;
                }
            });
     };
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.widget.Grid = function(opts) {
    this.opts = $.extend({}, this.opts, opts);
    this.createDOM();
    this.reload();
};

kimchi.widget.Grid.prototype = (function() {
    var htmlStr = [
        '<div id="{id}" class="grid">',
            '<div class="grid-content">',
                '<div class="grid-header">',
                    '<div class="grid-frozen-header-view">',
                        '<table class="grid-frozen-header-container">',
                        '</table>',
                    '</div>',
                    '<div class="grid-header-view">',
                        '<div class="grid-header-wrapper">',
                            '<table class="grid-header-container">',
                            '</table>',
                        '</div>',
                    '</div>',
                '</div>',
                '<div class="grid-body">',
                    '<div class="grid-frozen-body-view">',
                        '<div class="grid-frozen-body-wrapper">',
                            '<table class="grid-frozen-body-container">',
                            '</table>',
                        '</div>',
                    '</div>',
                    '<div class="grid-body-view">',
                        '<div class="grid-body-wrapper">',
                            '<table class="grid-body-container">',
                            '</table>',
                        '</div>',
                    '</div>',
                '</div>',
                '<div class="grid-resizer-leftmost hidden"></div>',
                '<div class="grid-resizer hidden"></div>',
            '</div>',
            '<div class="grid-footer"></div>',
            '<div class="grid-mask hidden">',
                '<div class="grid-loading">',
                    '<div class="grid-loading-icon"></div>',
                    '<div class="grid-loading-text">',
                        '{loading}',
                    '</div>',
                '</div>',
            '</div>',
            '<div class="grid-message hidden">',
                '<div class="grid-message-text">',
                    '{message}',
                    '<button class="retry-button btn-small">',
                        '{buttonLabel}',
                    '</button>',
                '</div>',
                '<div class="detailed-title">',
                    '{detailedLabel}',
                '</div>',
                '<div class="detailed-text"></div>',
            '</div>',
        '</div>'
    ].join('');

    var CONTAINER_NORMAL = 0, CONTAINER_FROZEN = 1;

    var setupHeaders = function(header, body, fields) {
        var colGroup = $('<colgroup></colgroup>').appendTo(header);
        var headerHeader = $('<thead></thead>');
        var headerRow = $('<tr></tr>').appendTo(headerHeader);
        $.each(fields || [], function(i, field) {
            $('<col class="' +
                field['class'] +
            '"/>')
                .appendTo(colGroup);
            $('<th><div class="cell-text-wrapper">' +
                field['label'] +
            '</div></th>').appendTo(headerRow);
        });
        headerHeader.appendTo(header);

        var totalWidth = 0;
        $('col', colGroup).each(function(index, col) {
            var width = $(col).width();
            totalWidth += width;
            $(col).css('width', width + 'px');
        });
        $(body).append(colGroup.clone());
        return totalWidth;
    };

    var getValue = function(name, obj) {
        var result=undefined;
        if(!Array.isArray(name)) {
            name=name.parseKey();
        }
        if(name.length!=0) {
            var tmpName=name.shift();
            if(obj[tmpName]!=undefined) {
                    result=obj[tmpName];
            }
            if(name.length!=0) {
                    result=getValue(name,obj[tmpName]);
            }
        }
        return(result);
    };

    var fillBody = function(container, fields) {
        var data = this.data;
        var tbody = ($('tbody', container).length && $('tbody', container))
            || $('<tbody></tbody>').appendTo(container);
        tbody.empty();
        $.each(data, function(i, row) {
            var rowNode = $('<tr></tr>').appendTo(tbody);
            $.each(fields, function(fi, field) {
                var value = getValue(field['name'], row);
                $('<td><div class="cell-text-wrapper"' +
                    (field['makeTitle'] === true
                        ? ' title="' + value + '"'
                        : ''
                    ) + '>' + value.toString() + '</div></td>'
                ).appendTo(rowNode);
            });
        });
    };

    var fixTableLayout = function(style) {
        $.each([
            this.frozenHeaderContainer,
            this.headerContainer,
            this.frozenBodyContainer,
            this.bodyContainer
        ], function(i, tableNode) {
            $(tableNode).css('table-layout', style || 'fixed');
        });
    };

    var initResizing = function(event) {
        var resizer = event.data.resizer;
        var pageX = event.pageX;
        var tailPos = $(this).width() + $(this).offset()['left'];
        var atResizer = Math.abs(pageX - tailPos) <= 2;
        var isResizing = !$(resizer).hasClass('hidden');
        $('body')[(atResizer || isResizing)
            ? 'addClass'
            : 'removeClass'
        ]('resizing');
    };

    var clearResizing = function(event) {
        $(event.data.resizer).hasClass('hidden') &&
            $('body').removeClass('resizing');
    };

    var stylingRow = function(row, className, add) {
        var index = $(row).index() + 1;
        $('tr', this.frozenBodyContainer)
            .removeClass(className);
        $('tr', this.bodyContainer)
            .removeClass(className);

        if(add === false) {
            return;
        }

        $('tr:nth-child(' + index + ')', this.frozenBodyContainer)
            .addClass(className);
        $('tr:nth-child(' + index + ')', this.bodyContainer)
            .addClass(className);
    };

    var setBodyListeners = function() {
        if(this['opts']['rowSelection'] != 'disabled') {
            $('tr', this.gridBody).on('mouseover', {
                grid: this
            }, function(event) {
                if (! $(this).hasClass('no-hover'))
                    stylingRow.call(event.data.grid, this, 'hover');
            });

            $('tr', this.gridBody).on('mouseout', {
                grid: this
            }, function(event) {
                stylingRow.call(event.data.grid, this, 'hover', false);
            });

            $('tr', this.gridBody).on('click', {
                grid: this
            }, function(event) {
                var grid = event.data.grid;
                grid.selectedIndex = $(this).index();
                stylingRow.call(grid, this, 'selected');
                grid['opts']['onRowSelected'] && grid['opts']['onRowSelected']();
            });
        }

        $('.grid-body-view', this.domNode).on('scroll', {
            grid: this
        }, function(event) {
            var grid = event.data.grid;
            $('.grid-header .grid-header-view', grid.domNode)
                .prop('scrollLeft', this.scrollLeft);
            $('.grid-body .grid-frozen-body-view', grid.domNode)
                .prop('scrollTop', this.scrollTop);
        });
    };

    var setData = function(data) {
        this.data = data;
        fillBody.call(this, this.frozenBodyContainer, this['opts']['frozenFields']);
        fillBody.call(this, this.bodyContainer, this['opts']['fields']);
        setBodyListeners.call(this);
    };

    var getSelected = function() {
        return this.selectedIndex >= 0
            ? this.data[this.selectedIndex]
            : null;
    };

    var startResizing = function(container, event) {
        var grid = event.data.grid;
        kimchi.widget.Grid.beingResized = grid;
        if(!($('body').hasClass('resizing')
                && $(grid.resizer).hasClass('hidden'))) {
            return;
        }

        grid.columnBeingResized = container;
        var pageX = event.pageX;
        var gridOffsetX = grid.domNode.offset()['left'];
        var leftmostOffsetX = $(container).offset()['left'] - gridOffsetX;
        var left = pageX - gridOffsetX;
        var contentHeight = $('.grid-content', grid.domNode).height();
        $(grid.resizerLeftmost).css({
            left: leftmostOffsetX + 'px',
            height: contentHeight + 'px'
        });
        $(grid.resizer).css({
            left: left + 'px',
            height: contentHeight + 'px'
        });
        $(grid.resizerLeftmost).removeClass('hidden');
        $(grid.resizer).removeClass('hidden');
        event.preventDefault();
    };

    var endResizing = function(event) {
        var grid = kimchi.widget.Grid.beingResized;
        if(!$('body').hasClass('resizing')) {
            return;
        }
        $(grid.resizerLeftmost).addClass('hidden');
        $(grid.resizer).addClass('hidden');
        $('body').removeClass('resizing');
        var leftmostOffset = $(grid.columnBeingResized).offset()['left'];
        var left = event.pageX;
        if(leftmostOffset > left) {
            return;
        }
        resizeColumnWidth.call(
            grid,
            $(grid.columnBeingResized).index(),
            left - leftmostOffset
        );
        fixTableLayout.call(grid);
        grid.columnBeingResized = null;
        kimchi.widget.Grid.beingResized = null;
    };

    var resizeColumnWidth = function(index, width) {
        var width = Math.ceil(width);
        var widthArray = [];
        var totalWidth = 0;
        var header = this.headerContainer;
        var body = this.bodyContainer;
        if(this.containerBeingResized === CONTAINER_FROZEN) {
            header = this.frozenHeaderContainer;
            body = this.frozenBodyContainer;
        }
        $('col', header).each(function(i, colNode) {
            var w = index === i ? width : $(colNode).width();
            widthArray.push(w);
            totalWidth += w;
        });
        $.each([header, body], function(i, container) {
            container.css({
                'table-layout': 'fixed',
                width: totalWidth + 'px'
            });
            $('col:nth-child(' + (index + 1) + ')', container).css({
                width: width + 'px'
            });
        });

        if(this.containerBeingResized === CONTAINER_FROZEN) {
            var headerView = $('.grid-header-view', this.domNode);
            var bodyView = $('.grid-body-view', this.domNode);
            $.each([headerView, bodyView], function(i, view) {
                view.css({
                    left: totalWidth + 'px'
                });
            });
        }
    };

    var positionResizer = function(event) {
        var grid = event.data.grid;
        if($(grid.resizer).hasClass('hidden')) {
            return;
        }

        var pageX = event.pageX;
        var gridOffsetX = $(grid.domNode).offset()['left'];
        var leftMost = $(grid.resizerLeftmost).position()['left'];
        var offsetX = pageX - gridOffsetX;
        offsetX = offsetX >= leftMost ? offsetX : leftMost;
        $(grid.resizer).css('left', offsetX + 'px');
    };

    var showMessage = function(msg) {
        $('.detailed-text', this.messageNode).text(msg);
        $(this.messageNode).removeClass('hidden');
    };

    var hideMessage = function() {
        $(this.messageNode).addClass('hidden');
    };

    var reload = function() {
        var data = this['opts']['data'];
        if(!data) {
            return;
        }

        $(this.messageNode).addClass('hidden');

        if($.isArray(data)) {
            return this.setData(data);
        }

        if($.isFunction(data)) {
            var loadData = data;
            $(this.maskNode).removeClass('hidden');
            loadData($.proxy(function(data) {
                this.setData(data);
                $(this.maskNode).addClass('hidden');
            }, this));
        }
    };

    var destroy = function() {
        $('body').off('mousemove.grid#' + this['opts']['id'], positionResizer);
        $('body').off('mouseup.grid#' + this['opts']['id'], endResizing);
    };

    var createDOM = function() {
        var containerID = this['opts']['container'];
        var container = $('#' + containerID);
        var gridID = this['opts']['id'];
        var rowSelection = this['opts']['rowSelection'] || 'single';
        var domNode = $(kimchi.substitute(htmlStr, {
            id: gridID,
            loading: i18n['KCHGRD6001M'],
            message: i18n['KCHGRD6002M'],
            buttonLabel: i18n['KCHGRD6003M'],
            detailedLabel: i18n['KCHGRD6004']
        })).appendTo(container);
        this.domNode = domNode;

        var height = domNode.height();
        var width = domNode.width();

        var title = this['opts']['title'];
        var titleNode = null;
        if(title) {
            titleNode = $('<div class="grid-caption">' + title + '</div>')
                .prependTo(domNode);
        }

        var toolbarButtons = this['opts']['toolbarButtons'];
        var toolbarNode = null;
        if(toolbarButtons) {
            toolbarNode = $('<div class="grid-toolbar"></div>');
            if(titleNode) {
                titleNode.after(toolbarNode);
            }
            else {
                toolbarNode.prependTo(domNode);
            }

            $.each(toolbarButtons, function(i, button) {
                var btnHTML = [
                    '<button',
                        button['id'] ? (' id="' + button['id'] + '"') : '',
                        ' class="grid-toolbar-button',
                            button['class'] ? (' ' + button['class']) : '',
                            '"',
                            button['disabled'] === true ? ' disabled' : '',
                            '>',
                            button['label'],
                    '</button>'
                ].join('');
                var btnNode = $(btnHTML).appendTo(toolbarNode);
                button['onClick'] &&
                    btnNode.on('click', button['onClick']);
            });
        }

        var frozenHeaderContainer = $('.grid-frozen-header-container', domNode);
        var frozenBodyContainer = $('.grid-frozen-body-container', domNode);
        var frozenWidth = setupHeaders(
                frozenHeaderContainer,
                frozenBodyContainer,
                this['opts']['frozenFields']
        );
        this.frozenHeaderContainer = frozenHeaderContainer;
        this.frozenBodyContainer = frozenBodyContainer;

        var headerContainer = $('.grid-header-container', domNode);
        var bodyContainer = $('.grid-body-container', domNode);
        setupHeaders(headerContainer, bodyContainer, this['opts']['fields']);
        this.headerContainer = headerContainer;
        this.bodyContainer = bodyContainer;

        fixTableLayout.call(this, 'auto');

        var gridContentNode = $('.grid-content', domNode);
        var captionHeight = titleNode && $(titleNode).height() || 0;
        var toolbarHeight = toolbarNode && $(toolbarNode).height() || 0;
        gridContentNode.css('top', (captionHeight + toolbarHeight) + 'px');

        var maskNode = $('.grid-mask', domNode);
        maskNode.css('top', captionHeight + 'px');
        this.maskNode = maskNode;

        var messageNode = $('.grid-message', domNode);
        messageNode.css('top', captionHeight + 'px');
        this.messageNode = messageNode;

        var headerView = $('.grid-header-view', domNode);
        var bodyView = $('.grid-body-view', domNode);
        headerView.css('left', (frozenWidth) + 'px');
        bodyView.css('left', (frozenWidth) + 'px');

        var bodyWidth = width - frozenWidth;
        headerContainer.css('width', bodyWidth + 'px');
        bodyContainer.css('width', bodyWidth + 'px');

        fixTableLayout.call(this);

        var gridBody = $('.grid-body', domNode);
        this.gridBody = gridBody;
        this.resizerLeftmost = $('.grid-resizer-leftmost', domNode);
        this.resizer = $('.grid-resizer', domNode);
        var gridHeader = $('.grid-header', domNode);
        $('th', gridHeader).on('mouseover mousemove', {
            resizer: this.resizer
        }, initResizing);

        $('th', gridHeader).on('mouseout', {
            resizer: this.resizer
        }, clearResizing);

        this.containerBeingResized = CONTAINER_NORMAL;
        $('th', frozenHeaderContainer).on('mousedown', {
            grid: this
        }, function(event) {
                event.data.grid.containerBeingResized = CONTAINER_FROZEN;
                startResizing(this, event);
        });
        $('th', headerContainer).on('mousedown', {
            grid: this
        }, function(event) {
                event.data.grid.containerBeingResized = CONTAINER_NORMAL;
                startResizing(this, event);
        });

        $('body').on('mousemove.grid#' + this['opts']['id'], {
            grid: this
        }, positionResizer);
        $('body').on('mouseup.grid#' + this['opts']['id'], endResizing);

        var data = this['opts']['data'];

        $('.retry-button', domNode).on('click', {
            grid: this
        }, function(event) {
            event.data.grid.reload();
        });
    };

    return {
        opts: {
            container: null,
            id: null,
            rowSelection: 'single',
            onRowSelected: null,
            title: null,
            toolbarButtons: null,
            frozenFields: null,
            fields: null
        },
        createDOM: createDOM,
        setData: setData,
        getSelected: getSelected,
        reload: reload,
        destroy: destroy,
        showMessage: showMessage
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_add_main = function() {
    var showTemplates = function() {
        kimchi.topic('templateCreated').unsubscribe(showTemplates);
        kimchi.listTemplates(function(result) {
            if (result && result.length) {
                $('#prompt-create-template').addClass('hidden');
                $('#prompt-choose-template').removeClass('hidden');
                var html = '';
                var tmpl = $('#tmpl-template').html();
                $.each(result, function(index, value) {
                    html += kimchi.substitute(tmpl, value);
                });
                $('#templateTile').html(html);
                return;
            }

            $('#btn-create-template').on('click', function(event) {
                kimchi.topic('templateCreated').subscribe(showTemplates);

                kimchi.window.open('template-add.html');

                event.preventDefault();
            });

            $('#prompt-choose-template').addClass('hidden');
            $('#prompt-create-template').removeClass('hidden');
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    };

    function validateForm() {
        if (!$('input[name=template]:checked', '#templateTile').val()) {
            return false;
        }
        return true;
    }

    $('#form-vm-add').change(function() {
        if (validateForm()) {
            $('#vm-doAdd').attr('disabled', false);
        }
    });

    var addGuest = function(event) {
        $('#vm-doAdd').attr('disabled', true);
        $('#vm-doAdd').attr('style', 'display:none');
        $('#vm-doAdding').attr('style', 'display');
        var formData = $('#form-vm-add').serializeObject();
        kimchi.createVM(formData, function() {
            kimchi.listVmsAuto();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            $('#vm-doAdd').attr('style', 'display');
            $('#vm-doAdding').attr('style', 'display:none');
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            kimchi.message.error(reason);
        });

        return false;
    };

    $('#form-vm-add').on('submit', addGuest);
    $('#vm-doAdd').on('click', addGuest);

    showTemplates();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_edit_main = function() {
    var buttonContainer = $('#action-button-container');
    $('#guest-edit-tabs').tabs({
        beforeActivate: function(event, ui) {
            var display_list = null;
            if(kimchi.thisVMState === "running") {
                display_list = ['form-guest-edit-permission'];
            } else {
                display_list = ['form-guest-edit-general', 'form-guest-edit-permission'];
            }
            $(buttonContainer).addClass('hidden');
            var deactivated = ui['newPanel'];
            if(display_list.indexOf($(deactivated).attr('id')) >= 0) {
                $(buttonContainer).removeClass('hidden');
            }
        }
    });

    var guestEditForm = $('#form-guest-edit-general');
    var saveButton = $('#guest-edit-button-save');
    var authType;

    var refreshCDROMs = function() {
        kimchi.listVMStorages({
            vm: kimchi.selectedGuest
        }, function(storages) {
            var container = $('#form-guest-edit-storage .body');
            $(container).empty();
            $.each(storages, function(index, storage) {
                storage['vm'] = kimchi.selectedGuest;
                rowHTML = $('#' + storage['type'] + '-row-tmpl').html();
                var templated = kimchi.substitute(rowHTML, storage);
                container.append(templated);
            });

            $('.replace', container).button({
                icons: {
                    primary: 'ui-icon-pencil'
                },
                text: false
            });

            $('.detach', container).button({
                icons: {
                    primary: 'ui-icon-trash'
                },
                text: false
            });
            if (kimchi.thisVMState === 'running') {
                $('.detach[data-type="cdrom"]', container).remove();
            }

            $('.save', container).button({
                icons: {
                    primary: 'ui-icon-disk'
                },
                text: false
            });

            $('.cancel', container).button({
                icons: {
                    primary: 'ui-icon-arrowreturnthick-1-w'
                },
                text: false
            });
        });
    };

    var initStorageListeners = function() {
        var container = $('#form-guest-edit-storage .body');
        var toggleCDROM = function(rowNode, toEdit) {
            $('button.replace,button.detach', rowNode)
                [(toEdit ? 'add' : 'remove') + 'Class']('hidden');
            $('button.save,button.cancel', rowNode)
                [(toEdit ? 'remove' : 'add') + 'Class']('hidden');
            var pathBox = $('.path input', rowNode)
                .prop('readonly', !toEdit);
            toEdit && pathBox.select();
            pathBox.val(pathBox.attr('value'));
        };

        var replaceCDROM = function(event) {
            event.preventDefault();
            kimchi.selectedGuestStorage = $(this).data('dev');
            $('.item', container).each(function(i, n) {
                toggleCDROM(n);
            });
            var rowNode = $('#cdrom-' + kimchi.selectedGuestStorage);
            toggleCDROM(rowNode, true);
        };

        $(container).on('click', 'button.replace', replaceCDROM);

        $(container).on('click', 'button.detach', function(e) {
            e.preventDefault();
            var settings = {
                title : i18n['KCHAPI6004M'],
                content : i18n['KCHVMCD6001M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            };
            if ($(this).data('type') == "disk")
                settings['content'] = i18n['KCHVMCD6009M'];

            var dev = $(this).data('dev');
            kimchi.confirm(settings, function() {
                kimchi.deleteVMStorage({
                    vm: kimchi.selectedGuest,
                    dev: dev
                }, function() {
                    kimchi.topic('kimchi/vmCDROMDetached').publish();
                });
            });
        });

        $(container).on('click', 'button.save', function(event) {
            event.preventDefault();
            var path = $('#cdrom-path-' + kimchi.selectedGuestStorage).val();
            var settings = {
                vm: kimchi.selectedGuest,
                dev: kimchi.selectedGuestStorage,
                path: path
            };

            kimchi.replaceVMStorage(settings, function(result) {
                kimchi.topic('kimchi/vmCDROMReplaced').publish({
                    result: result
                });
            }, function(result) {
                var errText = result['reason'] ||
                    result['responseJSON']['reason'];
                kimchi.message.error(errText);
            });
        });

        $(container).on('click', 'button.cancel', function(event) {
            event.preventDefault();
            var rowNode = $('#cdrom-' + kimchi.selectedGuestStorage);
            toggleCDROM(rowNode);
        });
    };

    var setupInterface = function() {
        $(".add", "#form-guest-edit-interface").button({
            icons: { primary: "ui-icon-plusthick" },
            text: false
        }).click(function(evt){
            evt.preventDefault();
            addItem({
                id: -1,
                mac: "",
                network: "",
                type: "network",
                viewMode: "hide",
                editMode: ""
            });
        });
        var toggleEdit = function(item, on, itemId){
            $("#label-mac-" + itemId, item).toggleClass("hide", on);
            $("#edit-mac-" + itemId, item).toggleClass("hide", !on);
            $("#label-network-" + itemId, item).toggleClass("hide", false);
            $("select", item).toggleClass("hide", true);
            $(".action-area", item).toggleClass("hide");
        };
        var addItem = function(data) {
            if (data.id == -1) {
                data.id = $('#form-guest-edit-interface > .body').children().size()
            }
            var itemNode = $.parseHTML(kimchi.substitute($('#interface-tmpl').html(),data));
            $(".body", "#form-guest-edit-interface").append(itemNode);
            $("select", itemNode).append(networkOptions);
            if(data.network!==""){
                $("select", itemNode).val(data.network);
            }
            $(".edit", itemNode).button({
                disabled: kimchi.thisVMState === "running",
                icons: { primary: "ui-icon-pencil" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                toggleEdit($(this).closest('div'), true, data.id);
            });
            $(".delete", itemNode).button({
                icons: { primary: "ui-icon-trash" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                kimchi.deleteGuestInterface(kimchi.selectedGuest, item.prop("id"), function(){
                    item.remove();
                });
            });
            $(".save", itemNode).button({
                icons: { primary: "ui-icon-disk" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                var interface = {
                    network: $("select", item).val(),
                    type: "network",
                    mac: $(":text", item).val()
                };
                var postUpdate = function(mac){
                    $("#label-network-" + data.id, item).text(interface.network);
                    $("#label-mac-" + data.id, item).text(mac);
                    $("#edit-mac-" + data.id, item).val(mac);
                    toggleEdit(item, false, data.id);
                };
                if(item.prop("id")==""){
                    kimchi.createGuestInterface(kimchi.selectedGuest, interface, function(data){
                        item.prop("id", data.mac);
                        postUpdate(data.mac);
                    });
                }else{
                    if (item.prop('id') == interface.mac) {
                        toggleEdit(item, false, data.id);
                    } else {
                        kimchi.updateGuestInterface(kimchi.selectedGuest, item.prop('id'),
                                interface, function(data){
                            item.prop("id", data.mac);
                            postUpdate(data.mac);
                        });
                    }
                }
            });
            $(".cancel", itemNode).button({
                icons: { primary: "ui-icon-arrowreturnthick-1-w" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                $("label", item).text()==="" ? item.remove() : toggleEdit(item, false, data.id);
            });
        };
        var networkOptions = "";
        kimchi.listNetworks(function(data){
            for(var i=0;i<data.length;i++){
                var isSlected = i==0 ? " selected" : "";
                networkOptions += "<option"+isSlected+">"+data[i].name+"</option>";
            }
            kimchi.getGuestInterfaces(kimchi.selectedGuest, function(data){
                for(var i=0;i<data.length;i++){
                    data[i].viewMode = "";
                    data[i].editMode = "hide";
                    data[i].id = i;
                    addItem(data[i]);
                }
            });
        });
    };

    var setupPermission = function() {
       //set up for LDAP
       $(".add", "#form-guest-edit-permission").button({
            icons: { primary: "ui-icon-plusthick" },
            text: false
        }).click(function(evt){
            evt.preventDefault();
            addItem({
                user: "",
                freeze: false,
                viewMode: "hide",
                editMode: "",
                checked: true
            });
        });
        var addItem = function(data) {
            var itemNode = $.parseHTML(kimchi.substitute($('#ldap-user-tmpl').html(),data));
            $(".body", "#form-guest-edit-permission .ldap").append(itemNode);
            $(".delete", itemNode).button({
                icons: { primary: "ui-icon-trash" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                item.remove();
            });
            $("input").focusout(function() {
                var item = $(this).parent().parent();
                var user= $(this).val();
                item.prop("id", user);
                $("label", item).text(user);
            });
            $("input").focusin(function() {
                $(this).removeClass("checked");
            });

            if (data.checked == true) {
                $(".checked", itemNode).addClass("hide");
            }
        };
        var toggleEdit = function(item, on){
            $("label", item).toggleClass("hide", on);
            $("input", item).toggleClass("hide", !on);
            $(".action-area", item).toggleClass("hide");
        };
        //set up for PAM
        var userNodes = {}, groupNodes = {};
        authType = kimchi.capabilities['auth']
        if (authType == 'pam') {
            $("#form-guest-edit-permission .ldap").hide();
            kimchi.retrieveVM(kimchi.selectedGuest, function(vm){
                kimchi.getUsers(function(users){
                    kimchi.getGroups(function(groups){
                        var subArray = function(a1, a2){ //a1-a2
                            for(var i=0; i<a2.length; i++){
                                for(var j=0; j<a1.length; j++){
                                    if(a2[i] == a1[j]){
                                        a1.splice(j, 1);
                                        break;
                                    }
                                }
                            }
                        };
                        subArray(users, vm.users); subArray(groups, vm.groups);
                        init(users, groups, vm.users, vm.groups);
                    });
                });
            });
        } else if (authType == 'ldap') {
            $("#form-guest-edit-permission .pam").hide();
            kimchi.retrieveVM(kimchi.selectedGuest, function(vm){
                for (var i=0; i<vm.users.length; i++) {
                    addItem({
                        user: vm.users[i],
                        viewMode: "",
                        freeze: true,
                        editMode: "hide",
                        checked: true});
                }
           });
        }
        var sortNodes = function(container, isUser){
            nodes = container.children();
            var keys = [];
            nodes.each(function(){
                keys.push($("label", this).text());
            });
            keys.sort();
            container.empty();
            for(var i=0; i<keys.length; i++){
                var itemNode = isUser ? userNodes[keys[i]] : groupNodes[keys[i]];
                $(itemNode).click(function(){
                    $(this).toggleClass("item-picked");
                });
                container.append(itemNode);
            }
        };
        var init = function(availUsers, availGroups, selUsers, selGroups){
            var initNode = function(key, isUserNode){
                var nodeGroups = isUserNode ? userNodes : groupNodes;
                nodeGroups[key] = $.parseHTML(kimchi.substitute($('#permission-item-pam').html(), {
                    val: key,
                    class: isUserNode? "user-icon" : "group-icon"
                }));
            };
            for(var i=0; i<availUsers.length; i++){
                initNode(availUsers[i], true);
                $("#permission-avail-users").append(userNodes[availUsers[i]]);
                sortNodes($("#permission-avail-users"), true);
            }
            for(var i=0; i<selUsers.length; i++){
                initNode(selUsers[i], true);
                $("#permission-sel-users").append(userNodes[selUsers[i]]);
                sortNodes($("#permission-sel-users"), true);
            }
            for(var i=0; i<availGroups.length; i++){
                initNode(availGroups[i], false);
                $("#permission-avail-groups").append(groupNodes[availGroups[i]]);
                sortNodes($("#permission-avail-groups"), false);
            }
            for(var i=0; i<selGroups.length; i++){
                initNode(selGroups[i], false);
                $("#permission-sel-groups").append(groupNodes[selGroups[i]]);
                sortNodes($("#permission-sel-groups"), false);
            }
        };
        var filterNodes = function(key, container){
            container.children().each(function(){
                $(this).css("display", $("label", this).text().indexOf(key)==-1 ? "none" : "");
            });
        }
        $("#permission-avail-searchBox").on("keyup", function() {
            var key = $(this).val();
            filterNodes(key, $("#permission-avail-users"));
            filterNodes(key, $("#permission-avail-groups"));
        });
        $("#permission-sel-searchBox").on("keyup", function() {
            var key = $(this).val();
            filterNodes(key, $("#permission-sel-users"));
            filterNodes(key, $("#permission-sel-groups"));
        });
        $('#permissionGo').button().click(function(evt) {
            evt.preventDefault();
            $("#permission-avail-users").children(".item-picked").appendTo("#permission-sel-users").removeClass("item-picked");
            sortNodes($("#permission-sel-users"), true);
            $("#permission-avail-groups").children(".item-picked").appendTo("#permission-sel-groups").removeClass("item-picked");
            sortNodes($("#permission-sel-groups"), false);
            $("#permission-sel-searchBox").val("");
            filterNodes("", $("#permission-sel-users"));
            filterNodes("", $("#permission-sel-groups"));
        });
        $('#permissionBack').button().click(function(evt) {
            evt.preventDefault();
            $("#permission-sel-users").children(".item-picked").appendTo("#permission-avail-users").removeClass("item-picked");
            sortNodes($("#permission-avail-users"), true);
            $("#permission-sel-groups").children(".item-picked").appendTo("#permission-avail-groups").removeClass("item-picked");
            sortNodes($("#permission-avail-groups"), false);
            $("#permission-avail-searchBox").val("");
            filterNodes("", $("#permission-avail-users"));
            filterNodes("", $("#permission-avail-groups"));
        });
    }
    var setupPCIDevice = function(){
        kimchi.getHostPCIDevices(function(hostPCIs){
            kimchi.getVMPCIDevices(kimchi.selectedGuest, function(vmPCIs){
                var pciEnabled = kimchi.capabilities.kernel_vfio;
                for(var i=0; i<hostPCIs.length; i++){
                    var itemNode = $.parseHTML(kimchi.substitute($('#pci-tmpl').html(),{
                        name: hostPCIs[i].name,
                        product: hostPCIs[i].product.description,
                        vendor: hostPCIs[i].vendor.description
                    }));
                    $(".body", "#form-guest-edit-pci").append(itemNode);
                    var iconClass = "ui-icon-plus";
                    for(var j=0; j<vmPCIs.length; j++){
                        if(hostPCIs[i].name==vmPCIs[j].name){
                            iconClass = "ui-icon-minus";
                            break;
                        }
                    }
                    pciEnabled || $("button", itemNode).remove();
                    $("button", itemNode).button({
                        icons: { primary: iconClass },
                        text: false
                    }).click(function(){
                        var obj = $(this);
                        if(obj.button("option", "icons").primary == "ui-icon-minus"){
                            kimchi.removeVMPCIDevice(kimchi.selectedGuest, obj.parent().prop("id"), function(){
                                kimchi.getVMPCIDevices(kimchi.selectedGuest, function(vmPCIs1){
                                    for(var k=0; k<hostPCIs.length; k++) {
                                        $("button", "#" + hostPCIs[k].name).button("option", "icons", {primary: "ui-icon-plus"});
                                    }
                                    for(var k=0; k<vmPCIs1.length; k++) {
                                        $("button", "#" + vmPCIs1[k].name).button("option", "icons", {primary: "ui-icon-minus"});
                                    }
                                });
                                filterNodes($("select", "#form-guest-edit-pci").val(), $("input", "#form-guest-edit-pci").val());
                            });
                        } else {
                            kimchi.addVMPCIDevice(kimchi.selectedGuest, { name: obj.parent().prop("id") }, function(){
                                kimchi.getVMPCIDevices(kimchi.selectedGuest, function(vmPCIs1){
                                    for(var k=0; k<vmPCIs1.length; k++) {
                                        $("button", "#" + vmPCIs1[k].name).button("option", "icons", {primary: "ui-icon-minus"});
                                    }
                                });
                                filterNodes($("select", "#form-guest-edit-pci").val(), $("input", "#form-guest-edit-pci").val());
                            });
                        }
                    });
                    kimchi.getPCIDeviceCompanions(hostPCIs[i].name, function(infoData) {
                        var pciTitle = i18n["KCHVMED6007M"] + "\n";
                        var haveCompanions = false;
                        for(var p=0; p<infoData.length; p++) {
                            if(infoData[p].device_type === "net") {
                                haveCompanions = true;
                                pciTitle += "   " + infoData[p].name + "\n";
                                pciTitle += "      " + i18n["KCHVMED6001M"] + " " + infoData[p].interface;
                                pciTitle += ", " + i18n["KCHVMED6002M"] + " " + infoData[p].address;
                                pciTitle += ", " + i18n["KCHVMED6003M"] + " " + infoData[p].link_type + "\n";
                            } else if(infoData[p].device_type === "storage") {
                                haveCompanions = true;
                                pciTitle += "   " + infoData[p].name + "\n";
                                pciTitle += "      " + i18n["KCHVMED6004M"] + " " + infoData[p].block;
                                pciTitle += ", " + i18n["KCHVMED6005M"] + " " + infoData[p].drive_type;
                                pciTitle += ", " + i18n["KCHVMED6006M"] + " " + infoData[p].model + "\n";
                            }
                        }
                        for(var q=0; q<infoData.length; q++) {
                            haveCompanions && $(".name", "#" + infoData[q].parent).attr("title", pciTitle);
                            haveCompanions && $(".product", "#" + infoData[q].parent).attr("title", pciTitle);
                            haveCompanions && $(".vendor", "#" + infoData[q].parent).attr("title", pciTitle);
                        }
                    });
                }
            });
        });
        var filterNodes = function(group, text){
            text = text.toLowerCase();
            $(".body", "#form-guest-edit-pci").children().each(function(){
                var textFilter = $(".name", this).text().toLowerCase().indexOf(text)!=-1;
                textFilter = textFilter || $(".product", this).text().toLowerCase().indexOf(text)!=-1;
                textFilter = textFilter || $(".vendor", this).text().toLowerCase().indexOf(text)!=-1;
                var display = "none";
                var itemGroup = $("button", this).button("option", "icons").primary;
                if(textFilter){
                    if(group == "all"){
                        display = "";
                    }else if(group=="toAdd" && itemGroup=="ui-icon-plus"){
                        display = ""
                    }else if(group == "added" && itemGroup=="ui-icon-minus"){
                        display = ""
                    }
                }
                $(this).css("display", display);
            });
        };
        $("select", "#form-guest-edit-pci").change(function(){
            filterNodes($(this).val(), $("input", "#form-guest-edit-pci").val());
        });
        $("input", "#form-guest-edit-pci").on("keyup", function() {
            filterNodes($("select", "#form-guest-edit-pci").val(), $(this).val());
        });
    };

    var setupSnapshot = function() {
        var currentSnapshot;
        var setCurrentSnapshot = function(aSnapshot){
            if(!aSnapshot)
                kimchi.getCurrentSnapshot(kimchi.selectedGuest, function(snapshot){
                    if(snapshot&&snapshot.name) aSnapshot = snapshot.name;
                }, null, true);
            if(aSnapshot){
                if(currentSnapshot) $(".ui-icon-check", "#"+currentSnapshot).addClass("hide");
                $(".ui-icon-check", "#"+aSnapshot).removeClass("hide");
                currentSnapshot = aSnapshot;
            }
        };
        var addItem = function(data, container) {
            var itemNode = $.parseHTML(kimchi.substitute($('#snapshot-tmpl').html(),data));
            $("."+container, "#form-guest-edit-snapshot").append(itemNode);
            $(".delete", itemNode).button({
                icons: { primary: "ui-icon-trash" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                $("button", "#form-guest-edit-snapshot").button("disable");
                kimchi.deleteSnapshot(kimchi.selectedGuest, item.prop("id"), function(){
                    item.remove();
                    setCurrentSnapshot();
                    $("button", "#form-guest-edit-snapshot").button("enable");
                }, function(data){
                    kimchi.message.error(data.responseJSON.reason);
                    $("button", "#form-guest-edit-snapshot").button("enable");
                });
            });
            $(".revert", itemNode).button({
                icons: { primary: "ui-icon-arrowthick-1-ne" },
                text: false
            }).click(function(evt){
                evt.preventDefault();
                var item = $(this).parent().parent();
                $(".ui-icon-check", item).addClass("hide");
                $(".icon", item).removeClass("hide");
                $("button", "#form-guest-edit-snapshot").button("disable");
                kimchi.revertSnapshot(kimchi.selectedGuest, item.prop("id"), function(){
                    $(".icon", item).addClass("hide");
                    $("button", "#form-guest-edit-snapshot").button("enable");
                    setCurrentSnapshot(item.prop("id"));
                    kimchi.listVmsAuto();
                    kimchi.window.close();
                }, function(data){
                    kimchi.message.error(data.responseJSON.reason);
                    $(".icon", item).addClass("hide");
                    $("button", "#form-guest-edit-snapshot").button("enable");
                });
            });
        };
        var addOngoingItem = function(task){
            var uri = task.target_uri;
            addItem({
                name: uri.substring(uri.lastIndexOf('/')+1, uri.length),
                created: "",
                listMode: "hide",
                createMode: ""
            }, 'task');
            if(kimchi.trackingTasks.indexOf(task.id)==-1)
                kimchi.trackTask(task.id, function(task){
                    listGeneratingSnapshots();
                    $("button", "#form-guest-edit-snapshot").button("enable");
                }, function(err){
                    kimchi.message.error(err.message);
                    listGeneratingSnapshots();
                    $("button", "#form-guest-edit-snapshot").button("enable");
                });
        };
        var listGeneratingSnapshots = function(){
            kimchi.getTasksByFilter('status=running&target_uri='+encodeURIComponent('^/snapshots/*'), function(tasks) {
                $(".task", "#form-guest-edit-snapshot").empty();
                for(var i=0;i<tasks.length;i++){
                    addOngoingItem(tasks[i]);
                }
                if(tasks.length==0) listSnapshots();
            });
        };
        var listSnapshots = function(){
            kimchi.listSnapshots(kimchi.selectedGuest, function(data){
                $(".body", "#form-guest-edit-snapshot").empty();
                for(var i=0;i<data.length;i++){
                    data[i].created = new Date(data[i].created*1000).toLocaleString();
                    data[i].createMode = "hide";
                    data[i].listMode = "";
                    addItem(data[i], 'body');
                }
                setCurrentSnapshot();
            });
        };
        listGeneratingSnapshots();
        $(".add", "#form-guest-edit-snapshot").button({
            icons: { primary: "ui-icon-plusthick" },
            text: false
        }).click(function(evt){
            evt.preventDefault();
            kimchi.createSnapshot(kimchi.selectedGuest, function(task){
                $("button", "#form-guest-edit-snapshot").button("disable");
                addOngoingItem(task);
            });
        });
        if(kimchi.thisVMState=="running") $("button", "#form-guest-edit-snapshot").remove();
    };

    var initContent = function(guest) {
        guest['icon'] = guest['icon'] || 'images/icon-vm.png';
        $('#form-guest-edit-general').fillWithObject(guest);
        kimchi.thisVMState = guest['state'];
        refreshCDROMs();
        $('#guest-edit-attach-cdrom-button').button({
                icons: {
                    primary: "ui-icon-plusthick"
                },
                text: false
            }).click(function(event) {
                event.preventDefault();
                kimchi.window.open("guest-storage-add.html");
            });
        if(kimchi.thisVMState === "running") {
            $("#form-guest-edit-general input").prop("disabled", true);
        } else {
            $("#action-button-container").removeClass("hidden");
        }

        var onAttached = function(params) {
            refreshCDROMs();
        };
        var onReplaced = function(params) {
            refreshCDROMs();
        };
        var onDetached = function(params) {
            refreshCDROMs();
        };

        initStorageListeners();
        setupInterface();
        setupPermission();
        setupPCIDevice();
        setupSnapshot();

        kimchi.topic('kimchi/vmCDROMAttached').subscribe(onAttached);
        kimchi.topic('kimchi/vmCDROMReplaced').subscribe(onReplaced);
        kimchi.topic('kimchi/vmCDROMDetached').subscribe(onDetached);

        kimchi.clearGuestEdit = function() {
            kimchi.topic('kimchi/vmCDROMAttached').unsubscribe(onAttached);
            kimchi.topic('kimchi/vmCDROMReplaced').unsubscribe(onReplaced);
            kimchi.topic('kimchi/vmCDROMDetached').unsubscribe(onDetached);
        };
    };

    kimchi.retrieveVM(kimchi.selectedGuest, initContent);

    var generalSubmit = function(event) {
        $(saveButton).prop('disabled', true);
        var data=$('#form-guest-edit-general').serializeObject();
        if(data['memory']!=undefined) {
            data['memory'] = Number(data['memory']);
        }
        if(data['cpus']!=undefined) {
            data['cpus']   = Number(data['cpus']);
        }

        kimchi.updateVM(kimchi.selectedGuest, data, function() {
            kimchi.listVmsAuto();
            kimchi.window.close();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
            $(saveButton).prop('disabled', false);
        });
    }

    var permissionSubmit = function(event) {
        var content = { users: [], groups: [] };
        authType = kimchi.capabilities['auth']
        if (authType == 'pam') {
            $("#permission-sel-users").children().each(function(){
                content.users.push($("label", this).text());
            });
            $("#permission-sel-groups").children().each(function(){
                content.groups.push($("label", this).text());
            });
            kimchi.updateVM(kimchi.selectedGuest, content, function(){
                kimchi.window.close();
            });
        } else if (authType == 'ldap') {
            $(saveButton).prop('disabled', true);
            var errors = 0;

            $(".body", "#form-guest-edit-permission .ldap").children().each(function () {
                var elem = $(this);
                content.users.push(elem.attr("id"));

                if (!$('input', elem).hasClass('hide')) {
                    var user = {'user_id': $(this).attr("id")};
                    kimchi.getUserById(user, null, function (data) {
                        errors += 1;
                        $("input", elem).addClass("checked");
                    });
                }
            });
            if (errors == 0) {
                kimchi.updateVM(kimchi.selectedGuest, content, function(){
                   kimchi.window.close();
                });
            } else {
                $(saveButton).prop('disabled', false);
            }
        }
    }

    // tap map, "general": 0, "storage": 1, "interface": 2, "permission": 3, "password": 4
    var submit_map = {0: generalSubmit, 3:permissionSubmit};
    var submitForm = function(event) {
        var current = $('#guest-edit-tabs').tabs( "option", "active" );
        var submitFun = submit_map[current];
        submitFun && submitFun(event);
        event.preventDefault();
    };

    $(guestEditForm).on('submit', submitForm);
    $(saveButton).on('click', submitForm);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.sampleGuestObject = {
    "name": "",
    "uuid": "",
    "state": "shutoff",
    "persistent": true,
    "icon": null,
    "cpus": 0,
    "memory": 0,
    "stats": {
        "net_throughput": 0,
        "io_throughput_peak": 100,
        "cpu_utilization": 0,
        "io_throughput": 0,
        "net_throughput_peak": 100
    },
    "screenshot": null,
    "graphics": {
        "passwd": null,
        "passwdValidTo": null,
        "type": "vnc",
        "port": null,
        "listen": "127.0.0.1"
    },
    "users": [],
    "groups": [],
    "access": "full"
};

kimchi.vmstart = function(event) {
    var button=$(this);
    if (!button.hasClass('loading')) {
        button.addClass('loading');
        var vm=$(this).closest('li[name=guest]');
        var vm_id=vm.attr("id");
        kimchi.startVM(vm_id, function(result) {
            button.removeClass('loading');
            kimchi.listVmsAuto();
            }, function(err) {
                button.removeClass('loading');
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    } else {
        event.preventDefault();
        event.stopPropagation();
        return;
    }
};

kimchi.vmsuspend = function(event) {
    var button=$(this);
    if (!button.hasClass('pause-gray')) {
        button.addClass('pause-gray');
        var vm=$(this).closest('li[name=guest]');
        var vm_id=vm.attr("id");
        kimchi.suspendVM(vm_id, function(result) {
            button.removeClass('pause-gray');
            kimchi.listVmsAuto();
            }, function(err) {
                button.removeClass('pause-gray');
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    } else {
        event.preventDefault();
        event.stopPropagation();
        return;
    }
};

kimchi.vmresume = function(event) {
    var button=$(this);
    if (!button.hasClass('resume-gray')) {
        button.addClass('resume-gray');
        var vm=$(this).closest('li[name=guest]');
        var vm_id=vm.attr("id");
        kimchi.resumeVM(vm_id, function(result) {
            button.removeClass('resume-gray');
            kimchi.listVmsAuto();
            }, function(err) {
                button.removeClass('resume-gray');
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    } else {
        event.preventDefault();
        event.stopPropagation();
        return;
    }
};

kimchi.vmpoweroff = function(event) {
    var button=$(this);
    if (!button.hasClass('loading')) {
        button.addClass('loading');
        var vm=button.closest('li[name=guest]');
        var vm_id=vm.attr("id");
        var vmObject=vm.data();
        var vm_persistent=vmObject.persistent == true;
        var content_msg = vm_persistent ? i18n['KCHVM6003M'] :
            i18n['KCHVM6009M'];
        var settings = {
            title : i18n['KCHVM6002M'],
            content : content_msg,
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };
        kimchi.confirm(settings, function() {
            kimchi.poweroffVM(vm_id, function(result) {
                button.removeClass('loading');
                kimchi.listVmsAuto();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        }, function() {
        });
    } else {
        event.preventDefault();
        event.stopPropagation();
    }
};

kimchi.vmshutdown = function(event){
    var vm=$(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    var settings = {
        title : i18n['KCHVM6006M'],
        content : i18n['KCHVM6007M'],
        confirm : i18n['KCHAPI6002M'],
        cancel : i18n['KCHAPI6003M']
    };
    kimchi.confirm(settings, function() {
        kimchi.shutdownVM(vm_id, function(result) {
                kimchi.listVmsAuto();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    }, function() {
    });
};

kimchi.vmreset = function(event){
    var vm=$(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    var settings = {
        title : i18n['KCHVM6004M'],
        content : i18n['KCHVM6005M'],
        confirm : i18n['KCHAPI6002M'],
        cancel : i18n['KCHAPI6003M']
    };
    kimchi.confirm(settings, function() {
        kimchi.resetVM(vm_id, function(result) {
                kimchi.listVmsAuto();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    }, function() {
    });
};

kimchi.vmdelete = function(event) {
    var vm = $(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    var settings = {
        title : i18n['KCHVM6008M'],
        content : i18n['KCHVM6001M'],
        confirm : i18n['KCHAPI6002M'],
        cancel : i18n['KCHAPI6003M']
    };
    kimchi.confirm(settings, function() {
        kimchi.deleteVM(vm_id, function(result) {
            kimchi.listVmsAuto();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    }, function() {
    });
};

kimchi.vmedit = function(event) {
    var vm = $(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    kimchi.selectedGuest = vm_id;
    kimchi.window.open({
        url: 'guest-edit.html',
        close: function() {
            kimchi.clearGuestEdit();
        }
    });
};

kimchi.openVmConsole = function(event) {
    var vm=$(this).closest('li[name=guest]');
    var vmObject=vm.data();
    if (vmObject.graphics['type'] == 'vnc') {
        kimchi.vncToVM(vm.attr('id'));
    }
    else if (vmObject.graphics['type'] == 'spice') {
        kimchi.spiceToVM(vm.attr('id'));
    }

};

kimchi.getVmsCurrentConsoleImgs = function() {
    var res = new Object();
    $('#guestList').children().each(function() {
        res[$(this).attr('id')] = $(this).find('img.imgactive').attr('src');
    })
    return res;
};

kimchi.getOpenMenuVmId = function() {
    var result;
    var openMenu = $('#guestList div[name="actionmenu"] .popover:visible');
    if(openMenu) {
        var li_element=openMenu.closest('li');
        result=li_element.attr('id');
    }
    return result;
};

kimchi.listVmsAuto = function() {
    if (kimchi.vmTimeout) {
        clearTimeout(kimchi.vmTimeout);
    }
    var getCreatingGuests = function(){
        var guests = [];
        kimchi.getTasksByFilter('status=running&target_uri='+encodeURIComponent('^/vms/[^/]+$'), function(tasks) {
            for(var i=0;i<tasks.length;i++){
                var guestUri = tasks[i].target_uri;
                var guestName = guestUri.split('/')[2]
                guests.push($.extend({}, kimchi.sampleGuestObject, {name: guestName, isCreating: true}));
                if(kimchi.trackingTasks.indexOf(tasks[i].id)==-1)
                    kimchi.trackTask(tasks[i].id, null, function(err){
                        kimchi.message.error(err.message);
                    }, null);
            }
        }, null, true);
        return guests;
    };
    var getCloningGuests = function(){
        var guests = [];
        kimchi.getTasksByFilter('status=running&target_uri='+encodeURIComponent('^/vms/.+/clone'), function(tasks) {
            for(var i=0;i<tasks.length;i++){
                var guestUri = tasks[i].target_uri;
                var guestName = guestUri.split('/')[2]
                guests.push($.extend({}, kimchi.sampleGuestObject, {name: guestName, isCloning: true}));
                if(kimchi.trackingTasks.indexOf(tasks[i].id)==-1)
                    kimchi.trackTask(tasks[i].id, null, function(err){
                        kimchi.message.error(err.message);
                    }, null);
            }
        }, null, true);
        return guests;
    };
    kimchi.listVMs(function(result, textStatus, jqXHR) {
        if (result && textStatus=="success") {
            result = getCloningGuests().concat(result);
            result = getCreatingGuests().concat(result);
            if(result.length) {
                var listHtml = '';
                var guestTemplate = kimchi.guestTemplate;
                var currentConsoleImages = kimchi.getVmsCurrentConsoleImgs();
                var openMenuGuest = kimchi.getOpenMenuVmId();
                $('#guestList').empty();
                $('#guestListField').show();
                $('#noGuests').hide();

                $.each(result, function(index, vm) {
                    var guestLI = kimchi.createGuestLi(vm, currentConsoleImages[vm.name], vm.name==openMenuGuest);
                    $('#guestList').append(guestLI);
                });
            } else {
                $('#guestListField').hide();
                $('#noGuests').show();
            }
        }

        kimchi.vmTimeout = window.setTimeout("kimchi.listVmsAuto();", 5000);
    }, function(errorResponse, textStatus, errorThrown) {
        if(errorResponse.responseJSON && errorResponse.responseJSON.reason) {
            kimchi.message.error(errorResponse.responseJSON.reason);
        }
        kimchi.vmTimeout = window.setTimeout("kimchi.listVmsAuto();", 5000);
    });
};

kimchi.createGuestLi = function(vmObject, prevScreenImage, openMenu) {
    var result=kimchi.guestElem.clone();
    
    //Setup the VM list entry
    var vmRunningBool=(vmObject.state=="running");
    var vmSuspendedBool = (vmObject.state=="paused");
    var vmPoweredOffBool = (vmObject.state=="shutoff");
    var vmPersistent = (vmObject.persistent == true);
    result.attr('id',vmObject.name);
    result.data(vmObject);

    //Add the Name
    var guestTitle=result.find('.title').attr('title',vmObject.name);
    guestTitle.html(vmObject.name);

    //Setup the VM console thumbnail display
    var curImg = vmObject.icon;
    if (vmObject.screenshot) {
        curImg = vmObject.screenshot.replace(/^\//,'');
    }
    var load_src = curImg || 'images/icon-vm.png';
    var tile_src = prevScreenImage || vmObject['load-src'];
    var liveTile=result.find('div[name=guest-tile] > .tile');
    liveTile.addClass(vmObject.state);
    liveTile.find('.imgactive').attr('src',tile_src);
    var imgLoad=liveTile.find('.imgload');
    imgLoad.on('load', function() {
                                     var oldImg=$(this).parent().find('.imgactive');
                                     oldImg.removeClass("imgactive").addClass("imgload");
                                     oldImg.attr("src","");
                                     $(this).addClass("imgactive").removeClass("imgload");
                                     $(this).off('load');
                                   });
    imgLoad.attr('src',load_src);

    //Link the stopped tile to the start action, the running tile to open the console, and the paused tile to resume
    if(!(vmObject.isCloning || vmObject.isCreating)){
        if (vmPoweredOffBool) {
            liveTile.off("click", kimchi.openVmConsole);
 	    liveTile.off("click", kimchi.vmresume);
            liveTile.on("click", kimchi.vmstart);
            liveTile.hover(function(event){$(this).find('.overlay').show()}, function(event){$(this).find('.overlay').hide()});
        } else if (vmSuspendedBool) {
	    liveTile.off("click", kimchi.vmstart);
	    liveTile.off("click", kimchi.openVmConsole);
            liveTile.on("click", kimchi.vmresume);
	    if(vmObject.state="paused") {
	        liveTile.find('.overlay').attr('src',"/images/theme-default/ac24_resume.png");
	        liveTile.find('.overlay').attr('alt',"Resume");
	    }
            liveTile.hover(function(event){$(this).find('.overlay').show()}, function(event){$(this).find('.overlay').hide()});
        } else {
            liveTile.off("click", kimchi.vmstart);
 	    liveTile.off("click", kimchi.vmresume);
            liveTile.on("click", kimchi.openVmConsole);
        }
    }

    //Setup the gauges
    var stats=vmObject.stats;
    var gaugeValue=0;
    gaugeValue=parseInt(stats.net_throughput);
    kimchi.circleGaugeInit(result, "net_throughput",gaugeValue,(gaugeValue*100/stats.net_throughput_peak));
    gaugeValue=parseInt(stats.io_throughput);
    kimchi.circleGaugeInit(result, "io_throughput",gaugeValue,(gaugeValue*100/stats.io_throughput_peak));
    gaugeValue=parseInt(stats.cpu_utilization);
    kimchi.circleGaugeInit(result, "cpu_utilization",gaugeValue+"%",gaugeValue);

    //Setup the VM Actions
    var guestActions=result.find("div[name=guest-actions]");
    guestActions.find(".shutoff-disabled").prop("disabled", !vmRunningBool);
    guestActions.find(".running-disabled").prop("disabled", vmRunningBool);
    guestActions.find(".non-persistent-disabled").prop("disabled", !vmPersistent);
    guestActions.find(".reset-disabled").prop("disabled", vmPoweredOffBool || !vmPersistent);
    guestActions.find(".pause-disabled").prop("disabled", vmPoweredOffBool || !vmPersistent);

    if (vmSuspendedBool) { //VM is paused
        //Hide Start
        guestActions.find(".running-hidden").hide();
        //Hide Pause button and menu
        guestActions.find(".pause-disabled").hide();
        guestActions.find(".pause-hidden").hide();
    }	

    if (vmRunningBool) { //VM IS running
        //Hide Start
        guestActions.find(".running-hidden").hide();
        //Hide Resume
        guestActions.find(".resume-hidden").hide();
    }

    if (vmPoweredOffBool) { //VM is powered off
        //Hide PowerOff
        guestActions.find(".shutoff-hidden").hide();
        //Hide Pause
        guestActions.find(".pause-hidden").hide();
        //Hide Resume
        guestActions.find(".resume-hidden").hide();
    }

    var consoleActions=guestActions.find("[name=vm-console]");

    if ((vmObject.graphics['type'] == 'vnc') || (vmObject.graphics['type'] == 'spice')) {
        consoleActions.on("click", kimchi.openVmConsole);
        consoleActions.show();
    } else {         //we don't recognize the VMs supported graphics, so hide the menu choice
        consoleActions.hide();
        consoleActions.off("click",kimchi.openVmConsole);
    }

    //Setup action event handlers
    if(!(vmObject.isCloning || vmObject.isCreating)){
        guestActions.find("[name=vm-start]").on({click : kimchi.vmstart});
        guestActions.find("[name=vm-poweroff]").on({click : kimchi.vmpoweroff});
        if ((vmRunningBool) || (vmSuspendedBool)) {  
            //If the guest is not running, do not enable reset; otherwise, reset is enabled (when running or paused)
            guestActions.find("[name=vm-reset]").on({click : kimchi.vmreset});

	    //If the guest is not running, do not enable shutdown;otherwise, shutdown is enabled (when running or paused)
            guestActions.find("[name=vm-shutdown]").on({click : kimchi.vmshutdown});
        }

        if (vmSuspendedBool) {
            guestActions.find("[name=vm-resume]").on({click : kimchi.vmresume});
        }

        if (vmRunningBool) {
            guestActions.find("[name=vm-pause]").on({click : kimchi.vmsuspend});
        }

        guestActions.find("[name=vm-edit]").on({click : kimchi.vmedit});
        guestActions.find("[name=vm-delete]").on({click : kimchi.vmdelete});
        guestActions.find("[name=vm-clone]").click(function(){
            var guest = $(this).closest('li[name=guest]').attr("id");
            kimchi.confirm({
                title : i18n['KCHAPI6006M'],
                content : i18n['KCHVM6010M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            }, function() {
                kimchi.cloneGuest(guest, function(data){
                    kimchi.listVmsAuto();
                });
            }, null);
        });

        //Maintain menu open state
        var actionMenu=guestActions.find("div[name=actionmenu]");
        if (openMenu) {
            $('.popover', actionMenu).toggle();
        }

    }else{
        guestActions.find('.btn').attr('disabled', true);
        $('.popover', guestActions.find("div[name=actionmenu]")).remove();

        result.find('.guest-pending').removeClass('hide-content');
        pendingText = result.find('.guest-pending .text')
        if(vmObject.isCloning)
            pendingText.text(i18n['KCHAPI6009M']);
        else
            pendingText.text(i18n['KCHAPI6008M']);
    }

    return result;
};

kimchi.circleGaugeInit = function(topElement, divName, display, percentage){
    var gauge=topElement.find('div[name="' + divName + '"] .circleGauge');
    if(gauge) {
        var data=Object();
        data.percentage = percentage;
        data.display = display;
        gauge.data(data);
    }
    gauge.circleGauge();
    return(gauge);
};

kimchi.guestSetRequestHeader = function(xhr) {
    xhr.setRequestHeader('Accept', 'text/html');
};

kimchi.guest_main = function() {
    if(kimchi.tabMode['guests'] === 'admin') {
        $('.tools').attr('style','display');
        $("#vm-add").on("click", function(event) {
            kimchi.window.open('guest-add.html');
        });
    }
    kimchi.guestTemplate = $('#guest-tmpl').html();
    kimchi.guestElem=$('<div/>').html(kimchi.guestTemplate).find('li');
    $('#guests-root-container').on('remove', function() {
        kimchi.vmTimeout && clearTimeout(kimchi.vmTimeout);
    });
    kimchi.listVmsAuto()
};

kimchi.editTemplate = function(guestTemplate, oldPopStat) {
    if (oldPopStat) {
        return guestTemplate.replace("vm-action", "vm-action open");
    }
    return guestTemplate;
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_media_main = function() {

    var refreshCDROMs = function() {
        kimchi.listVMStorages({
            vm: kimchi.selectedGuest,
            storageType: 'cdrom'
        }, function(storages) {
            var rowHTML = $('#cdrom-row-tmpl').html();
            var container = $('#guest-edit-cdrom-row-container');
            $(container).empty();

            $.each(storages, function(index, storage) {
                storage['vm'] = kimchi.selectedGuest;
                var templated = kimchi.substitute(rowHTML, storage);
                container.append(templated);
            });

            var replaceCDROM = function(event) {
                event.preventDefault();
                kimchi.selectedGuestStorage = $(this).data('dev');
                kimchi.window.open("guest-cdrom-edit.html");
            };

            $('input[type="text"][name="cdrom"]', container).on('click', replaceCDROM);
            $('.replace', container).on('click', replaceCDROM);
        });
    };

    refreshCDROMs();

    var onReplaced = function(params) {
        refreshCDROMs();
    };
    kimchi.topic('kimchi/vmCDROMReplaced').subscribe(onReplaced);

    kimchi.clearGuestMedia = function() {
        kimchi.topic('kimchi/vmCDROMReplaced').unsubscribe(onReplaced);
    };
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_storage_add_main = function() {
    var types = [{
        label: 'cdrom',
        value: 'cdrom',
    },
    {
        label: 'disk',
        value: 'disk',
    }];
    var typesRunning = [{
        label: 'disk',
        value: 'disk'
    }];

    var storageAddForm = $('#form-guest-storage-add');
    var submitButton = $('#guest-storage-button-add');
    var typeTextbox = $('input[name="type"]', storageAddForm);
    var pathTextbox = $('input[name="path"]', storageAddForm);
    var poolTextbox = $('input[name="pool"]', storageAddForm);
    var volTextbox = $('input[name="vol"]', storageAddForm);

    typeTextbox.change(function() {
        var pathObject = {'cdrom': ".path-section", 'disk': '.volume-section'}
        selectType = $(this).val();
        $.each(pathObject, function(type, value) {
            if(selectType == type){
                $(value).removeClass('hidden');
            } else {
                $(value).addClass('hidden');
            }
        });

        if ($(".path-section").hasClass('hidden')) {
            $(poolTextbox).val('default');
            $(poolTextbox).change();
            $(pathTextbox).val("");
        }
        else {
            $(poolTextbox).val("");
            $(volTextbox).val("");
        }
    });

    kimchi.listStoragePools(function(result) {
        var options = [];
        if (result && result.length) {
            $.each(result, function(index, storagePool) {
                if ((storagePool.state=="active") && (storagePool.type !== 'kimchi-iso')) {
                    options.push({
                        label: storagePool.name,
                        value: storagePool.name
                        });
                    }
                });
                kimchi.select('guest-add-storage-pool-list', options);
        }
    });

    poolTextbox.change(function() {
        var options = [];
        kimchi.listStorageVolumes($(this).val(), function(result) {
            var validVolType = { cdrom: /iso/, disk: /^(raw|qcow|qcow2|bochs|qed|vmdk)$/};
            $('#guest-disk').selectMenu();
            if (result.length) {
                $.each(result, function(index, value) {
                    // Only unused volume can be attached
                    if (value.used_by.length == 0 && (value.type != 'file' || validVolType[selectType].test(value.format))) {
                        options.push({
                            label: value.name,
                            value: value.name
                        });
                    }
                });
                if (options.length) {
                    $(volTextbox).val(options[0].value);
                    $(volTextbox).change();
                }
            }
            $('#guest-disk').selectMenu("setData", options);
        });
    });


    typeTextbox.change(function() {
        var pathObject = {'cdrom': ".path-section", 'disk': '.volume-section'}
        var selectType = $(this).val();
        $.each(pathObject, function(type, value) {
            if(selectType == type){
                $(value).removeClass('hidden');
            } else {
                $(value).addClass('hidden');
            }
        });
    });

    if (kimchi.thisVMState === 'running') {
        types =typesRunning;
        $(typeTextbox).val('disk');
        typeTextbox.change();
        poolTextbox.change();
    }
    var selectType = $(typeTextbox).val();
    kimchi.select('guest-storage-type-list', types);

    var validateCDROM = function(settings) {
        if (/^((https|http|ftp|ftps|tftp|\/).*)+$/.test(settings['path']))
            return true;
        else {
            kimchi.message.error.code('KCHVMSTOR0001E');
            return false;
        }
    }

    var validateDisk = function(settings) {
        if (settings['pool'] && settings['vol'])
            return true;
        else {
            kimchi.message.error.code('KCHVMSTOR0002E');
            return false;
        }
    }

    validator = {cdrom: validateCDROM, disk: validateDisk};
    var submitForm = function(event) {
        if (submitButton.prop('disabled')) {
            return false;
        }

        var formData = storageAddForm.serializeObject();
        var settings = {
            vm: kimchi.selectedGuest,
            type: typeTextbox.val(),
        };

        $(submitButton).prop('disabled', true);
        $.each([pathTextbox, poolTextbox, volTextbox], function(i, c) {
            $(c).prop('disabled', true);
            val = $(c).val()
            if (val && val != '') {
                settings[$(c).attr('name')] = $(c).val();
            }
        });
        // Validate form for cdrom and disk
        validateSpecifiedForm = validator[settings['type']];
        if (!validateSpecifiedForm(settings)) {
            $(submitButton).prop('disabled', false);
            $.each([submitButton, pathTextbox, poolTextbox, volTextbox], function(i, c) {
                $(c).prop('disabled', false);
            });
            return false;
        }
        $(submitButton).addClass('loading').text(i18n['KCHVMCD6003M']);

        kimchi.addVMStorage(settings, function(result) {
            kimchi.window.close();
            kimchi.topic('kimchi/vmCDROMAttached').publish({
                result: result
            });
        }, function(result) {
            var errText = result['reason'] ||
                result['responseJSON']['reason'];
            kimchi.message.error(errText);

            $.each([submitButton, pathTextbox, poolTextbox, volTextbox], function(i, c) {
                $(c).prop('disabled', false);
            });
            $(submitButton).removeClass('loading').text(i18n['KCHVMCD6002M']);
        });

        event.preventDefault();
    };

    storageAddForm.on('submit', submitForm);
    submitButton.on('click', submitForm);
    pathTextbox.on('change input propertychange', function(event) {
        $(submitButton).prop('disabled', $(this).val() === '');
    });
    volTextbox.on('change propertychange', function (event) {
        $(submitButton).prop('disabled', $(this).val() === '');
    });

};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.host={};

kimchi.host_main = function() {
    var expand = function(header, toExpand) {
        var controlledNode = $(header).attr('aria-controls');
        $('#' + controlledNode)[toExpand ? 'removeClass' : 'addClass']('hidden');
        $(header).attr('aria-expanded', toExpand ? 'true' : 'false');
    };

    var repositoriesGrid = null;
    var initRepositoriesGrid = function(repo_type) {
        var gridFields=[];
        if (repo_type == "yum") {
                gridFields=[{
                    name: 'repo_id',
                    label: i18n['KCHREPO6004M'],
                    'class': 'repository-id'
                }, {
                    name: 'config[repo_name]',
                    label: i18n['KCHREPO6005M'],
                    'class': 'repository-name'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled'
                }];
        }
        else if (repo_type == "deb") {
                gridFields=[{
                    name: 'baseurl',
                    label: i18n['KCHREPO6006M'],
                    makeTitle: true,
                    'class': 'repository-baseurl deb'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled deb'
                }, {
                    name: 'config[dist]',
                    label: "dist",
                    'class': 'repository-gpgcheck deb'
                }, {
                    name: 'config[comps]',
                    label: "comps",
                    'class': 'repository-gpgcheck deb'
                }];
        }
        else {
            gridFields=[{
                name: 'repo_id',
                label: i18n['KCHREPO6004M'],
                'class': 'repository-id'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled'
                }, {
                    name: 'baseurl',
                    label: i18n['KCHREPO6006M'],
                    makeTitle: true,
                    'class': 'repository-baseurl'
                }];
        }
        repositoriesGrid = new kimchi.widget.Grid({
            container: 'repositories-grid-container',
            id: 'repositories-grid',
            title: i18n['KCHREPO6003M'],
            toolbarButtons: [{
                id: 'repositories-grid-add-button',
                label: i18n['KCHREPO6012M'],
                onClick: function(event) {
                    kimchi.window.open({url:'repository-add.html', class: repo_type});
                }
            }, {
                id: 'repositories-grid-enable-button',
                label: i18n['KCHREPO6016M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }
                    var name = repository['repo_id'];
                    var enable = !repository['enabled'];
                    $(this).prop('disabled', true);
                    kimchi.enableRepository(name, enable, function() {
                        kimchi.topic('kimchi/repositoryUpdated').publish();
                    });
                }
            }, {
                id: 'repositories-grid-edit-button',
                label: i18n['KCHREPO6013M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }
                    kimchi.selectedRepository = repository['repo_id'];
                    kimchi.window.open({url:'repository-edit.html', class: repo_type});
                }
            }, {
                id: 'repositories-grid-remove-button',
                label: i18n['KCHREPO6014M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }

                    var settings = {
                        title : i18n['KCHREPO6001M'],
                        content : i18n['KCHREPO6002M'],
                        confirm : i18n['KCHAPI6004M'],
                        cancel : i18n['KCHAPI6003M']
                    };

                    kimchi.confirm(settings, function() {
                        kimchi.deleteRepository(
                            repository['repo_id'],
                            function(result) {
                                kimchi.topic('kimchi/repositoryDeleted').publish(result);
                            }, function(error) {
                            }
                        );
                    });
                }
            }],
            onRowSelected: function(row) {
                var repository = repositoriesGrid.getSelected();
                if(!repository) {
                    return;
                }
                $('#repositories-grid-remove-button').prop('disabled', false);
                $('#repositories-grid-edit-button').prop('disabled', false);
                var enabled = repository['enabled'];
                $('#repositories-grid-enable-button')
                    .text(i18n[enabled ? 'KCHREPO6017M' : 'KCHREPO6016M'])
                    .prop('disabled', false);
            },
            frozenFields: [],
            fields: gridFields,
            data: listRepositories
        });
    };

    var listRepositories = function(gridCallback) {
        kimchi.listRepositories(function(repositories) {
            if($.isFunction(gridCallback)) {
                gridCallback(repositories);
            }
            else {
                if(repositoriesGrid) {
                    repositoriesGrid.setData(repositories);
                }
                else {
                    initRepositoriesGrid();
                    repositoriesGrid.setData(repositories);
                }
            }
        },
        function(error) {
            var message = error && error['responseJSON'] && error['responseJSON']['reason'];

            if($.isFunction(gridCallback)) {
                gridCallback([]);
            }
            repositoriesGrid &&
                repositoriesGrid.showMessage(message || i18n['KCHUPD6008M']);
        });

        $('#repositories-grid-remove-button').prop('disabled', true);
        $('#repositories-grid-edit-button').prop('disabled', true);
        $('#repositories-grid-enable-button').prop('disabled', true);
    };

    var softwareUpdatesGridID = 'software-updates-grid';
    var softwareUpdatesGrid = null;
    var progressAreaID = 'software-updates-progress-textarea';
    var reloadProgressArea = function(result) {
        var progressArea = $('#' + progressAreaID)[0];
        $(progressArea).text(result['message']);
        var scrollTop = $(progressArea).prop('scrollHeight');
        $(progressArea).prop('scrollTop', scrollTop);
    };

    var initSoftwareUpdatesGrid = function(softwareUpdates) {
        softwareUpdatesGrid = new kimchi.widget.Grid({
            container: 'software-updates-grid-container',
            id: softwareUpdatesGridID,
            title: i18n['KCHUPD6001M'],
            rowSelection: 'disabled',
            toolbarButtons: [{
                id: softwareUpdatesGridID + '-update-button',
                label: i18n['KCHUPD6006M'],
                disabled: true,
                onClick: function(event) {
                    var updateButton = $(this);
                    var progressArea = $('#' + progressAreaID)[0];
                    $('#software-updates-progress-container').removeClass('hidden');
                    $(progressArea).text('');
                    !kimchi.isElementInViewport(progressArea) &&
                        progressArea.scrollIntoView();
                    $(updateButton).text(i18n['KCHUPD6007M']).prop('disabled', true);

                    kimchi.updateSoftware(function(result) {
                        reloadProgressArea(result);
                        $(updateButton).text(i18n['KCHUPD6006M']).prop('disabled', false);
                        kimchi.topic('kimchi/softwareUpdated').publish({
                            result: result
                        });
                    }, function(error) {
                        var message = error && error['responseJSON'] && error['responseJSON']['reason'];
                        kimchi.message.error(message || i18n['KCHUPD6009M']);
                        $(updateButton).text(i18n['KCHUPD6006M']).prop('disabled', false);
                    }, reloadProgressArea);
                }
            }],
            frozenFields: [],
            fields: [{
                name: 'package_name',
                label: i18n['KCHUPD6002M'],
                'class': 'software-update-name'
            }, {
                name: 'version',
                label: i18n['KCHUPD6003M'],
                'class': 'software-update-version'
            }, {
                name: 'arch',
                label: i18n['KCHUPD6004M'],
                'class': 'software-update-arch'
            }, {
                name: 'repository',
                label: i18n['KCHUPD6005M'],
                'class': 'software-update-repos'
            }],
            data: listSoftwareUpdates
        });
    };

    var listSoftwareUpdates = function(gridCallback) {
        kimchi.listSoftwareUpdates(function(softwareUpdates) {
            if($.isFunction(gridCallback)) {
                gridCallback(softwareUpdates);
            }
            else {
                if(softwareUpdatesGrid) {
                    softwareUpdatesGrid.setData(softwareUpdates);
                }
                else {
                    initSoftwareUpdatesGrid(softwareUpdates);
                }
            }

            var updateButton = $('#' + softwareUpdatesGridID + '-update-button');
            $(updateButton).prop('disabled', softwareUpdates.length === 0);
        }, function(error) {
            var message = error && error['responseJSON'] && error['responseJSON']['reason'];
            if($.isFunction(gridCallback)) {
                gridCallback([]);
            }
            softwareUpdatesGrid &&
                softwareUpdatesGrid.showMessage(message || i18n['KCHUPD6008M']);
        });
    };

    var reportGridID = 'available-reports-grid';
    var reportGrid = null;
    var enableReportButtons = function(toEnable) {
        var buttonID = '#{grid}-{btn}-button';
        $.each(['rename', 'remove', 'download'], function(i, n) {
            $(kimchi.substitute(buttonID, {
                grid: reportGridID,
                btn: n
            })).prop('disabled', !toEnable);
        });
    };
    var initReportGrid = function(reports) {
        reportGrid = new kimchi.widget.Grid({
            container: 'available-reports-grid-container',
            id: reportGridID,
            title: i18n['KCHDR6002M'],
            toolbarButtons: [{
                id: reportGridID + '-generate-button',
                label: i18n['KCHDR6006M'],
                onClick: function(event) {
                    kimchi.window.open('report-add.html');
                }
            }, {
                id: reportGridID + '-rename-button',
                label: i18n['KCHDR6008M'],
                disabled: true,
                onClick: function(event) {
                    var report = reportGrid.getSelected();
                    if(!report) {
                        return;
                    }

                    kimchi.selectedReport = report['name'];
                    kimchi.window.open('report-rename.html');
                }
            }, {
                id: reportGridID + '-remove-button',
                label: i18n['KCHDR6009M'],
                disabled: true,
                onClick: function(event) {
                    var report = reportGrid.getSelected();
                    if(!report) {
                        return;
                    }

                    var settings = {
                        title : i18n['KCHAPI6004M'],
                        content : i18n['KCHDR6001M'],
                        confirm : i18n['KCHAPI6002M'],
                        cancel : i18n['KCHAPI6003M']
                    };

                    kimchi.confirm(settings, function() {
                        kimchi.deleteReport({
                            name: report['name']
                        }, function(result) {
                            listDebugReports();
                        }, function(error) {
                           kimchi.message.error(error.responseJSON.reason);
                        });
                    });
                }
            }, {
                id: reportGridID + '-download-button',
                label: i18n['KCHDR6010M'],
                disabled: true,
                onClick: function(event) {
                    var report = reportGrid.getSelected();
                    if(!report) {
                        return;
                    }

                    kimchi.downloadReport({
                        file: report['uri']
                    });
                }
            }],
            onRowSelected: function(row) {
                var report = reportGrid.getSelected();
                // Only enable report buttons if the selected line is not a
                // pending report
                if (report['time'] == i18n['KCHDR6007M']) {
                    var gridElement = $('#'+ reportGridID);
                    var row = $('tr:contains(' + report['name'] + ')', gridElement);
                    enableReportButtons(false);
                    row.attr('class', '');
                }
                else {
                    enableReportButtons(true);
                }
            },
            frozenFields: [],
            fields: [{
                name: 'name',
                label: i18n['KCHDR6003M'],
                'class': 'debug-report-name'
            }, {
                name: 'time',
                label: i18n['KCHDR6005M'],
                'class': 'debug-report-time'
            }],
            data: reports
        });
    };

    var getPendingReports = function() {
        var reports = []
        var filter = 'status=running&target_uri=' + encodeURIComponent('^/debugreports/*')

        kimchi.getTasksByFilter(filter, function(tasks) {
            for(var i = 0; i < tasks.length; i++) {
                reportName = tasks[i].target_uri.replace(/^\/debugreports\//, '') || i18n['KCHDR6012M'];
                reports.push({'name': reportName, 'time': i18n['KCHDR6007M']})

                if(kimchi.trackingTasks.indexOf(tasks[i].id) >= 0) {
                    continue;
                }

                kimchi.trackTask(tasks[i].id, function(result) {
                    kimchi.topic('kimchi/debugReportAdded').publish();
                }, function(result) {
                    // Error message from Async Task status
                    if (result['message']) {
                        var errText = result['message'];
                    }
                    // Error message from standard kimchi exception
                    else {
                        var errText = result['responseJSON']['reason'];
                    }
                    result && kimchi.message.error(errText);
                    kimchi.topic('kimchi/debugReportAdded').publish();
                }, null);
            }
        }, null, true);

        return reports;
    };

    var listDebugReports = function() {
        kimchi.listReports(function(reports) {
            pendingReports = getPendingReports();
            allReports = pendingReports.concat(reports);
            $('#debug-report-section').removeClass('hidden');

            // Row selection will be cleared so disable buttons here
            enableReportButtons(false);

            if(reportGrid) {
                reportGrid.setData(allReports);
            }
            else {
                initReportGrid(allReports);
            }

            // Set id-debug-img to pending reports
            // It will display a loading icon
            var gridElement = $('#' + reportGridID);
                $.each($('td:contains(' + i18n['KCHDR6007M']  + ')', gridElement), function(index, row) {
                $(row).parent().addClass('no-hover');
                $(row).attr('id', 'id-debug-img');
            });
        }, function(error) {
            if(error['status'] == 403) {
                $('#debug-report-section').addClass('hidden');
                return;
            }
            $('#debug-report-section').removeClass('hidden');
        });
    };

    var shutdownButtonID = '#host-button-shutdown';
    var restartButtonID = '#host-button-restart';
    var shutdownHost = function(params) {
        var settings = {
            title : i18n['KCHAPI6004M'],
            content : i18n['KCHHOST6008M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };

        kimchi.confirm(settings, function() {
            kimchi.shutdown(params);
            $(shutdownButtonID).prop('disabled', true);
            $(restartButtonID).prop('disabled', true);
            // Check if there is any VM is running.
            kimchi.listVMs(function(vms) {
                for(var i = 0; i < vms.length; i++) {
                    if(vms[i]['state'] === 'running') {
                        kimchi.message.error.code('KCHHOST6001E');
                        $(shutdownButtonID).prop('disabled', false);
                        $(restartButtonID).prop('disabled', false);
                        return;
                    }
                }

            });
        }, function() {
        });
    };

    var initPage = function() {
        $('#host-info-container .section-header').each(function(i, header) {
            $('<span class="arrow"></span>').prependTo(header);
            var toExpand = $(header).attr('aria-expanded') !== 'false';
            expand(header, toExpand);
        });

        $('#host-info-container').on('click', '.section-header', function(event) {
            var toExpand = $(this).attr('aria-expanded') === 'false';
            expand(this, toExpand);
        });

        $('#host-button-shutdown').on('click', function(event) {
            shutdownHost(null);
        });

        $('#host-button-restart').on('click', function(event) {
            shutdownHost({
                reboot: true
            });
        });

        var setupUI = function() {
            if (kimchi.capabilities == undefined) {
                setTimeout(setupUI, 2000);
                return;
            }

            if((kimchi.capabilities['repo_mngt_tool']) && (kimchi.capabilities['repo_mngt_tool']!="None")) {
                initRepositoriesGrid(kimchi.capabilities['repo_mngt_tool']);
                $('#repositories-section').switchClass('hidden', kimchi.capabilities['repo_mngt_tool']);
                kimchi.topic('kimchi/repositoryAdded')
                    .subscribe(listRepositories);
                kimchi.topic('kimchi/repositoryUpdated')
                    .subscribe(listRepositories);
                kimchi.topic('kimchi/repositoryDeleted')
                    .subscribe(listRepositories);
            }

            if(kimchi.capabilities['update_tool']) {
                $('#software-update-section').removeClass('hidden');
                initSoftwareUpdatesGrid();
                kimchi.topic('kimchi/softwareUpdated')
                    .subscribe(listSoftwareUpdates);
                $('#software-updates-progress-container').accordion({
                    collapsible: true
                });
            }

            if(kimchi.capabilities['system_report_tool']) {
                listDebugReports();
                kimchi.topic('kimchi/debugReportAdded')
                    .subscribe(listDebugReports);
                kimchi.topic('kimchi/debugReportRenamed')
                    .subscribe(listDebugReports);
            }
        };
        setupUI();
    };

    kimchi.getHost(function(data) {
        var htmlTmpl = $('#host-tmpl').html();
        data['logo'] = data['logo'] || '';
        data['memory'] = kimchi.formatMeasurement(data['memory'], {
            fixed: 2
        });
        var templated = kimchi.substitute(htmlTmpl, data);
        $('#host-content-container').html(templated);

        initPage();
        initTracker();
    });

    var StatsMgr = function() {
        var statsArray = {
            cpu: {
                u: {
                    type: 'percent',
                    legend: i18n['KCHHOST6002M'],
                    points: []
                }
            },
            memory: {
                u: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    legend: i18n['KCHHOST6003M'],
                    points: []
                }
            },
            diskIO: {
                r: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6004M'],
                    points: []
                },
                w: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6005M'],
                    'class': 'disk-write',
                    points: []
                }
            },
            networkIO: {
                r: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6006M'],
                    points: []
                },
                s: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6007M'],
                    'class': 'network-sent',
                    points: []
                }
            }
        };
        var SIZE = 20;
        var cursor = SIZE;

        var add = function(stats) {
            for(var key in stats) {
                var item = stats[key];
                for(var metrics in item) {
                    var value = item[metrics]['v'];
                    var max = item[metrics]['max'];
                    var unifiedMetrics = statsArray[key][metrics];
                    var ps = unifiedMetrics['points'];
                    if(!Array.isArray(value)){
                        ps.push(value);
                        if(ps.length > SIZE + 1) {
                            ps.shift();
                        }
                    }
                    else{
                        ps=ps.concat(value);
                        ps.splice(0, ps.length-SIZE-1);
                        unifiedMetrics['points']=ps;
                    }
                    if(max !== undefined) {
                        unifiedMetrics['max'] = max;
                    }
                    else {
                        if(unifiedMetrics['type'] !== 'value') {
                            continue;
                        }
                        max = -Infinity;
                        $.each(ps, function(i, value) {
                            if(value > max) {
                                max = value;
                            }
                        });
                        if(max === 0) {
                            ++max;
                        }
                        max *= 1.1;
                        unifiedMetrics['max'] = max;
                    }
                }
            }
            cursor++;
        };

        var get = function(which) {
            var stats = statsArray[which];
            var lines = [];
            for(var k in stats) {
                var obj = stats[k];
                var line = {
                    type: obj['type'],
                    base: obj['base'],
                    unit: obj['unit'],
                    fixed: obj['fixed'],
                    legend: obj['legend']
                };
                if(obj['max']) {
                    line['max'] = obj['max'];
                }
                if(obj['class']) {
                    line['class'] = obj['class'];
                }
                var ps = obj['points'];
                var numStats = ps.length;
                var unifiedPoints = [];
                $.each(ps, function(i, value) {
                    unifiedPoints.push({
                        x: cursor - numStats + i,
                        y: value
                    });
                });
                line['points'] = unifiedPoints;
                lines.push(line);
            }
            return lines;
        };

        return {
            add: add,
            get: get
        };
    };

    var Tracker = function(charts) {
      var charts = charts;
      var timer = null;
      var statsPool = new StatsMgr();
      var setCharts = function(newCharts) {
          charts = newCharts;
          for(var key in charts) {
              var chart = charts[key];
              chart.updateUI(statsPool.get(key));
          }
      };

      var self = this;

      var UnifyStats = function(stats) {
          var result= {
              cpu: {
                  u: {
                      v: stats['cpu_utilization']
                  }
              },
              memory: {
                  u: {
                  }
              },
              diskIO: {
                  r: {
                      v: stats['disk_read_rate']
                  },
                  w: {
                      v: stats['disk_write_rate']
                  }
              },
              networkIO: {
                  r: {
                      v: stats['net_recv_rate']
                  },
                  s: {
                      v: stats['net_sent_rate']
                  }
              }
          };
          if(Array.isArray(stats['memory'])){
              result.memory.u['v']=[];
              result.memory.u['max']=-Infinity;
              for(var i=0;i<stats['memory'].length;i++){
                  result.memory.u['v'].push(stats['memory'][i]['avail']);
                  result.memory.u['max']=Math.max(result.memory.u['max'],stats['memory'][i]['total']);
              }
          }
          else {
              result.memory.u['v']=stats['memory']['avail'],
              result.memory.u['max']=stats['memory']['total']
          }
          return(result);
      };


      var statsCallback = function(stats) {
              var unifiedStats = UnifyStats(stats);
              statsPool.add(unifiedStats);
              for(var key in charts) {
                  var chart = charts[key];
                  chart.updateUI(statsPool.get(key));
              }
              timer = setTimeout(function() {
                  continueTrack();
              }, 1000);
          };

      var track = function() {
          kimchi.getHostStatsHistory(statsCallback,
            function() {
                continueTrack();
            });
      };

      var continueTrack = function() {
          kimchi.getHostStats(statsCallback,
            function() {
                continueTrack();
            });
      };

      var destroy = function() {
          timer && clearTimeout(timer);
          timer = null;
      };

      return {
        setCharts: setCharts,
        start: track,
        stop: destroy
      };
    };

    var initTracker = function() {
        // TODO: Extend tabs with onUnload event to unregister timers.
        if(kimchi.hostTimer) {
            kimchi.hostTimer.stop();
            delete kimchi.hostTimer;
        }

        var trackedCharts = {
            cpu: new kimchi.widget.LineChart({
                id: 'chart-cpu',
                node: 'container-chart-cpu',
                type: 'percent'
            }),
            memory: new kimchi.widget.LineChart({
                id: 'chart-memory',
                node: 'container-chart-memory',
                type: 'value'
            }),
            diskIO: new kimchi.widget.LineChart({
                id: 'chart-disk-io',
                node: 'container-chart-disk-io',
                type: 'value'
            }),
            networkIO: new kimchi.widget.LineChart({
                id: 'chart-network-io',
                node: 'container-chart-network-io',
                type: 'value'
            })
        };

        if(kimchi.hostTimer) {
            kimchi.hostTimer.setCharts(trackedCharts);
        }
        else {
            kimchi.hostTimer = new Tracker(trackedCharts);
            kimchi.hostTimer.start();
        }
    };

    $('#host-root-container').on('remove', function() {
        if(kimchi.hostTimer) {
            kimchi.hostTimer.stop();
            delete kimchi.hostTimer;
            }

        repositoriesGrid && repositoriesGrid.destroy();
        kimchi.topic('kimchi/repositoryAdded')
            .unsubscribe(listRepositories);
        kimchi.topic('kimchi/repositoryUpdated')
            .unsubscribe(listRepositories);
        kimchi.topic('kimchi/repositoryDeleted')
            .unsubscribe(listRepositories);

        softwareUpdatesGrid && softwareUpdatesGrid.destroy();
        kimchi.topic('kimchi/softwareUpdated').unsubscribe(listSoftwareUpdates);

        reportGrid && reportGrid.destroy();
        kimchi.topic('kimchi/debugReportAdded').unsubscribe(listDebugReports);
        kimchi.topic('kimchi/debugReportRenamed').unsubscribe(listDebugReports);
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.lang = {
    all: function() {
        return {
          'en_US': 'English (US)',
          'zh_CN': '中文（简体）',
          'pt_BR': 'Português (Brasil)',
          'de_DE': 'Deutsch (Deutschland)',
          'es_ES': 'Español (España)',
          'fr_FR': 'Français (France)',
          'it_IT': 'Italiano (Italia)',
          'ja_JP': '日本語 (日本)',
          'ko_KR': '한국어 (대한민국)',
          'ru_RU': 'Русский (Россия)',
          'zh_TW': '中文（繁體）'
        };
    },

    /**
     * Language is determined by the following sequence:
     * 1) Cookie setting; or if not set ->
     * 2) HTML DOM lang attribute; or if not set ->
     * 3) DEFAULT (en_US).
     */
    get: function() {
        return kimchi.cookie.get('kimchiLang') ||
            $('html').prop('lang') ||
            'en_US';
    },

    set: function(lang) {
        kimchi.cookie.set('kimchiLang', lang, 365);
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * new kimchi.widget.LineChart({
 *   node: 'line-chart-cpu',
 *   id: 'line-chart',
 *   type: 'value'
 * });
 */
kimchi.widget.LineChart = function(params) {
    var container = $('#' + params['node']);
    container.addClass('chart-container');
    var height = container.height();
    var width = container.width();
    var numHLines = 4;
    var linesSpace = height / numHLines;
    var period = params['period'] || 20;
    var xFactor = width / period;
    var yFactor = height / 100;
    var xStart = params['xStart'] || 0;
    var linesOffset = 0;
    var canvasID = params['id'];
    var maxValue = params['maxValue'] || -Infinity;
    var type = params['type'];
    var chartVAxis = null;
    var chartTitle = null;
    var chartLegend = null;
    var seriesMap = {};
    var formatSettings = {};

    var setMaxValue = function(newValue) {
        maxValue = newValue;
    };

    /**
     *
     * settings: {
     *   'class': 'disk-read-rate'
     * }
     */
    var updateUI = function(data) {
        var container = $('#' + params['node']);
        if(!container.length) {
            return;
        }

        if(!$.isArray(data)) {
            data = [data];
        }
        var seriesCount = 0;
        var singleSeries = data.length === 1;
        var firstSeries = data[0];

        // TODO: Multiple axes support.
        if(type === 'value') {
            $.each(data, function(i, series) {
                if(series['max'] > maxValue) {
                    maxValue = series['max'];
                    formatSettings = {
                        base: series['base'],
                        unit: series['unit'],
                        fixed: series['fixed']
                    };
                }
            });
        }

        var canvasNode = $('#' + canvasID);
        canvasNode.length && canvasNode.remove();
        var htmlStr = [
          '<svg id="', canvasID, '" class="line-chart"',
              ' height="', height, '" width="', width, '"',
          '>',
            '<rect height="', height, '" width="', width, '" class="background" />'
        ];

        for(var x = linesOffset; x < width; x += linesSpace) {
            htmlStr.push(
                '<line x1="', x, '" y1="', 0, '" x2="', x, '" y2="', height, '" />'
            );
        }

        linesOffset -= xFactor;
        while(linesOffset < 0) {
            linesOffset = linesSpace + linesOffset;
        }

        for(var y = height - linesSpace; y > 0; y -= linesSpace) {
            htmlStr.push(
                '<line x1="', 0, '" y1="', y, '" x2="', width, '" y2="', y, '" />'
            );
        }

        var maxValueLabel = i18n['KCHHOST6001M'] + ' ' +
            (type === 'value'
                ? kimchi.formatMeasurement(maxValue, formatSettings)
                : '100%');
        if(!chartVAxis) {
            chartVAxis = $('<div class="chart-vaxis-container">' +
                maxValueLabel +
                '</div>'
            );
            container.before(chartVAxis);
        }
        else {
            chartVAxis.text(maxValueLabel);
        }

        seriesNames = [];
        $.each(data, function(i, series) {
            var points = series['points'];
            var className = series['class'];
            var latestPoint = points.slice(-1).pop();
            xStart = latestPoint['x'] - period;

            htmlStr.push('<polyline',
                ' class="series', className ? ' ' + className : '', '"',
                ' points="'
            );
            var first = true;
            $.each(points, function(i, point) {
                if(first) {
                    first = false;
                }
                else {
                    htmlStr.push(' ');
                }

                var x = xFactor * (point['x'] - xStart);
                var y = height - yFactor * (type === 'value' ?
                    point['y'] * 100 / maxValue :
                    point['y']
                );
                htmlStr.push(x, ',', y);
            });
            htmlStr.push('" />');
        });

        htmlStr.push('</svg>');

        var canvasNode = $(htmlStr.join('')).appendTo(container);

        if(!chartLegend) {
            chartLegend = $('<div class="chart-legend-container"></div>');
            container.after(chartLegend);
        }
        else {
            chartLegend.empty();
        }
        $('polyline.series', canvasNode).each(function(i, polyline) {
            var wrapper = $('<div class="legend-wrapper"></div>')
                .appendTo(chartLegend);
            $([
                '<svg class="legend-icon" width="20" height="10">',
                    '<line x1="0" y1="5" x2="20" y2="5"/>',
                '</svg>'
            ].join('')).appendTo(wrapper);
            $('line', wrapper).css({
                stroke: $(polyline).css('stroke'),
                'stroke-width': $(polyline).css('stroke-width')
            });
            var label = data[i]['legend'];
            var base = data[i]['base'];
            $('<label class="legend-label">' + label + '</label>')
                .appendTo(wrapper);
            var latestPoint = data[i]['points'].slice(-1).pop();
            var latestValue = latestPoint['y'];
            if(type === 'value') {
                latestValue = kimchi.formatMeasurement(
                    latestValue,
                    formatSettings
                );
            }
            else {
                latestValue += '%';
            }
            $('<div class="latest-value">' + latestValue + '</div>')
                .appendTo(wrapper);
        });
    };

    return {
        setMaxValue: setMaxValue,
        updateUI: updateUI
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.login_main = function() {

    var selectedLanguage = kimchi.lang.get();
    $('#userLang').val(selectedLanguage);

    $('#userLang').on('change', function() {
        kimchi.lang.set($(this).val());
        location.reload();
    });

    var query = window.location.search;
    var error = /.*error=(.*?)(&|$)/g.exec(query);
    if (error && error[1] == "sessionTimeout") {
        $("#messSession").show();
    }

    var userNameBox = $('#username');
    var passwordBox = $('#password');
    var loginButton = $('#btn-login');

    var login = function(event) {
        $("#login").hide();
        $("#logging").show();

        var userName = userNameBox.val();
        userName && kimchi.user.setUserName(userName);
        var settings = {
            username: userName,
            password: passwordBox.val()
        };

        kimchi.login(settings, function(data) {
            var query = window.location.search;
            var next  = /.*next=(.*?)(&|$)/g.exec(query);
            if (next) {
                var next_url = decodeURIComponent(next[1]);
            }
            else {
                var lastPage = kimchi.cookie.get('lastPage');
                var next_url = lastPage ? lastPage.replace(/\"/g,'') : "/";
            }
            kimchi.cookie.set('roles',JSON.stringify(data.roles));
            window.location.replace(window.location.pathname.replace(/\/+login.html/, '') + next_url)
        }, function() {
            $("#messUserPass").show();
            $("#messSession").hide();
            $("#logging").hide();
            $("#login").show();
        });

        return false;
    };

    $('#btn-login').bind('click', login);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.tabMode = {};

kimchi.capabilities = undefined;
kimchi.getCapabilities(function(result) {
    kimchi.capabilities = result;

    if(kimchi.capabilities.federation=="on")
        $('#peers').removeClass('hide-content');
}, function() {
    kimchi.capabilities = {};
});

kimchi.main = function() {
    kimchi.isLoggingOut = false;
    kimchi.popable();

    var genTabs = function(tabs) {
        var tabsHtml = [];
        $(tabs).each(function(i, tab) {
            var title = tab['title'];
            var path = tab['path'];
            var mode = tab['mode'];
            if (mode != 'none') {
                var helpPath = kimchi.checkHelpFile(path);
                var disableHelp = (helpPath.length == 0 ? "disableHelp" : helpPath);
                tabsHtml.push(
                    '<li>',
                        '<a class="item ', disableHelp,'" href="', path, '">',
                            title,
                        '</a>',
                        '<input id="helpPathId" name="helpPath" value="' + helpPath + '" type="hidden"/>',
                    '</li>'
                );
            }
        });
        return tabsHtml.join('');
    };

    var parseTabs = function(xmlData) {
        var tabs = [];
        $(xmlData).find('tab').each(function() {
            var $tab = $(this);
            var titleKey = $tab.find('title').text();
            var title = i18n[titleKey] ? i18n[titleKey] : titleKey;
            var path = $tab.find('path').text();
            var roles = kimchi.cookie.get('roles');
            if (roles) {
                var role = JSON.parse(roles)[titleKey.toLowerCase()];
                var mode = $tab.find('[role="' + role + '"]').attr('mode');
                kimchi.tabMode[titleKey.toLowerCase()] = mode;
                tabs.push({
                    title: title,
                    path: path,
                    mode: mode
                });
            } else {
                document.location.href = 'login.html';
            }
        });

        return tabs;
    };

    var retrieveTabs = function(url) {
        var tabs;
        $.ajax({
            url : url,
            async : false,
            success : function(xmlData) {
                tabs = parseTabs(xmlData);
            }
        });
        return tabs;
    };

    var tabConfigUrl = 'config/ui/tabs.xml';
    var pluginConfigUrl = 'plugins/{plugin}/ui/config/tab-ext.xml';
    var pluginI18nUrl = 'plugins/{plugin}/i18n.json';
    var DEFAULT_HASH;
    var buildTabs = function(callback) {
        var tabs = retrieveTabs(tabConfigUrl);
        kimchi.listPlugins(function(plugins) {
            $(plugins).each(function(i, p) {
                var url = kimchi.substitute(pluginConfigUrl, {
                    plugin: p
                });
                var i18nUrl = kimchi.substitute(pluginI18nUrl, {
                    plugin: p
                });
                kimchi.getI18n(function(i18nObj){ $.extend(i18n, i18nObj)},
                               function(i18nObj){ //i18n is not define by plugin
                               }, i18nUrl, true);
                tabs.push.apply(tabs, retrieveTabs(url));
            });

            var defaultTab = tabs[1]

            var defaultTabPath = defaultTab && defaultTab['path']
            // Remove file extension from 'defaultTabPath'
            DEFAULT_HASH = defaultTabPath &&
                defaultTabPath.substring(0, defaultTabPath.lastIndexOf('.'))

            $('#nav-menu').append(genTabs(tabs));

            callback && callback();
        }, function(data) {
           kimchi.message.error(data.responseJSON.reason);
        }, true);
    };

    var onLanguageChanged = function(lang) {
        kimchi.lang.set(lang);
        location.reload();
    };

    /**
     * Do the following setup:
     *   1) Clear any timing events.
     *   2) If the given URL is invalid (i.e., no corresponding href value in
     *      page tab list.), then clear location.href and inform the user;
     *
     *      Or else:
     *      Move the page tab indicator to the right position;
     *      Load the page content via Ajax.
     */
    var onKimchiRedirect = function(url) {
        /*
         * Find the corresponding tab node and animate the arrow indicator to
         * point to the tab. If nothing found, inform user the URL is invalid
         * and clear location.hash to jump to home page.
         */
        var tab = $('#nav-menu a[href="' + url + '"]');
        if (tab.length === 0) {
            location.hash = '';
            return;
        }

        // Animate arrow indicator.
        var left = $(tab).parent().position().left;
        var width = $(tab).parent().width();
        $('.menu-arrow').stop().animate({
            left : left + width / 2 - 10
        });

        // Update the visual style of tabs; focus the selected one.
        $('#nav-menu a').removeClass('current');
        $(tab).addClass('current');
        $(tab).focus();
        // Disable Help button according to selected tab
        if ($(tab).hasClass("disableHelp")) {
            $('#btn-help').css('cursor', "not-allowed");
            $('#btn-help').off("click");
        }
        else {
            $('#btn-help').css('cursor', "pointer");
            $('#btn-help').on("click", kimchi.openHelp);
        }
        // Load page content.
        loadPage(url);
    };

    /**
     * Use Ajax to dynamically load a page without a page refreshing. Handle
     * arrow cursor animation, DOM node focus, and page content rendering.
     */
    var loadPage = function(url) {
        // Get the page content through Ajax and render it.
        url && $('#main').load(url, function(responseText, textStatus, jqXHR) {
            if (jqXHR['status'] === 401 || jqXHR['status'] === 303) {
                var isSessionTimeout = jqXHR['responseText'].indexOf("sessionTimeout")!=-1;
                document.location.href= isSessionTimeout ? 'login.html?error=sessionTimeout' : 'login.html';
                return;
            }
        });
    };

    /*
     * Update page content.
     * 1) If user types in the main page URL without hash, then we apply the
     *    default hash. e.g., http://kimchi.company.com:8000;
     * 2) If user types a URL with hash, then we publish an "redirect" event
     *    to load the page, e.g., http://kimchi.company.com:8000/#templates.
     */
    var updatePage = function() {
        // Parse hash string.
        var hashString = (location.hash && location.hash.substr(1));
        /*
         * If hash string is empty, then apply the default one;
         * or else, publish an "redirect" event to load the page.
         */
        if (!hashString) {
            location.hash = DEFAULT_HASH;
        }
        else {
            kimchi.topic('redirect').publish(hashString + '.html');
        }
    };

    /**
     * Register listeners including:
     * 1) Kimchi redirect event
     * 2) hashchange event
     * 3) Tab list click event
     * 4) Log-out button click event
     * 5) About button click event
     * 6) Help button click event
     * 7) Peers button click event
     */
    var searchingPeers = false;
    var initListeners = function() {
        kimchi.topic('languageChanged').subscribe(onLanguageChanged);
        kimchi.topic('redirect').subscribe(onKimchiRedirect);

        /*
         * If hash value is changed, then we know the user is intended to load
         * another page.
         */
        window.onhashchange = updatePage;

        /*
         * Register click listener of tabs. Replace the default reloading page
         * behavior of <a> with Ajax loading.
         */
        $('#nav-menu').on('click', 'a.item', function(event) {
            var href = $(this).attr('href');
            // Remove file extension from 'href'
            location.hash = href.substring(0,href.lastIndexOf('.'))
            /*
             * We use the HTML file name for hash, like: guests for guests.html
             * and templates for templates.html.
             *     Retrieve hash value from the given URL and update location's
             * hash part. It has 2 effects: one is to publish Kimchi "redirect"
             * event to trigger listener, the other is to put an entry into the
             * browser's address history to make pages be bookmark-able.
             */
            // Prevent <a> causing browser redirecting to other page.
            event.preventDefault();
        });

        // Perform logging out via Ajax request.
        $('#btn-logout').on('click', function() {
            kimchi.logout(function() {
                kimchi.isLoggingOut = true;
                document.location.href = "login.html";
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        });

        // Set handler for about button
        $('#btn-about').on('click', function(event) {
            kimchi.window.open({"content": $('#about-tmpl').html()});
            event.preventDefault();
            });

        // Set handler for help button
        $('#btn-help').on('click', kimchi.openHelp);

        // Set handler to peers drop down
        $('#peers').on('click', function() {

            // Check if any request is in progress
            if ($('.popover', '#peers').is(':visible') || searchingPeers == true)
                return

            $('#search-peers').show();
            $('#no-peers').addClass('hide-content');
            $('a', '#peers').remove();

            searchingPeers = true;

            kimchi.getPeers(function(data){
                $('#search-peers').hide();
                if (data.length == 0)
                    $('#no-peers').removeClass('hide-content');

                for(var i=0; i<data.length; i++){
                    $('.dropdown', '#peers').append("<a href='"+data[i]+"' target='_blank'>"+data[i]+"</a>");
                }
                searchingPeers = false;
            });
        });
    };

    var initUI = function() {
        $(document).bind('ajaxError', function(event, jqXHR, ajaxSettings, errorThrown) {
            if (!ajaxSettings['kimchi']) {
                return;
            }

            if (jqXHR['status'] === 401) {
                var isSessionTimeout = jqXHR['responseText'].indexOf("sessionTimeout")!=-1;
                kimchi.user.showUser(false);
                kimchi.previousAjax = ajaxSettings;
                $(".empty-when-logged-off").empty();
                $(".remove-when-logged-off").remove();
                document.location.href= isSessionTimeout ? 'login.html?error=sessionTimeout' : 'login.html';
                return;
            }
            else if((jqXHR['status'] == 0) && ("error"==jqXHR.statusText) && !kimchi.isLoggingOut) {
                kimchi.message.error(i18n['KCHAPI6007E'].replace("%1", jqXHR.state()));
            }
            if(ajaxSettings['originalError']) {
                ajaxSettings['originalError'](jqXHR, jqXHR.statusText, errorThrown);
            }
        });

        kimchi.user.showUser(true);
        initListeners();
        updatePage();
    };

    // Load i18n translation strings first and then render the page.
    kimchi.getI18n(
        function(i18nStrings){ //success
            i18n = i18nStrings;
            buildTabs(initUI);
            },
        function(data){ //error
            kimchi.message.error(data.responseJSON.reason);
            });
};


kimchi.checkHelpFile = function(path) {
    var lang = kimchi.lang.get();
    var url = ""
    // Find help page path according to tab name
    if (/^tabs/.test(path))
        url = path.replace("tabs", "help/" + lang);
    else if (/^plugins/.test(path))
        url = path.slice(0, path.lastIndexOf('/')) + "help/" + lang + path.slice(path.lastIndexOf('/'));
    // Checking if help page exist.
    $.ajax({
        url: url,
        async: false,
        error: function() { url = ""; },
        success: function() { }
    });
    return url;
};


kimchi.openHelp = function(e) {
    var tab = $('#nav-menu a.current');
    var url = $(tab).parent().find("input[name='helpPath']").val();
    window.open(url, "Kimchi Help");
    e.preventDefault();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.message = function(msg, level, node) {
    var container = node || $('#messageField');
    if ($(container).size() < 1) {
        container = $('<div id="messageField"/>').appendTo(document.body);
    }
    var message = '<div class="message ' + (level || '') + '" style="display: none;">';
    if(!node) {
        message += '<div class="close">X</div>';
    }
    message += '<div class="content">' + msg + '</div>';
    message += '</div>';
    var $message = $(message);
    $(container).append($message);
    $message.fadeIn(100);

    setTimeout(function() {
        $message.fadeOut(2000, function() {
            $(this).remove();
        });
    }, 4000);

    $(container).on("click", ".close", function(e) {
        $(this).parent().fadeOut(200, function() {
            $(this).remove();
        });
    });
};

/**
 * A public function of confirm box.
 *
 * @param msg
 *            type:[object]
 * @param msg.title
 *            The title of the confirm box.
 * @param msg.content
 *            The main text of the confirm box.
 * @param msg.confirm
 *            The text of the confirm button.
 * @param msg.cancel
 *            the text of the cancel button.
 * @param confirmCallback
 *            the callback function of click the confirm button.
 * @param cancelCallback
 *            The callback function of click the cancel and X button.
 */
kimchi.confirm = function(settings, confirmCallback, cancelCallback) {
    if ($('#confirmbox-container ').size() < 1) {
        $(document.body).append('<div id="confirmbox-container" class="bgmask"></div>');
    }
    var confirmboxHtml = '<div class="confirmbox">';
    confirmboxHtml += '<header>';
    confirmboxHtml += '<h4 class="title">' + (settings.title || '') + '</h4>';
    confirmboxHtml += '<div class="close cancel">X</div>';
    confirmboxHtml += '</header>';
    confirmboxHtml += '<div class="content">';
    confirmboxHtml += settings.content + '</div>';
    confirmboxHtml += '<footer>';
    confirmboxHtml += '<div class="btn-group">';
    confirmboxHtml += '<button id="button-confirm" class="btn-small"><span class="text">' + (settings.confirm || i18n['KCHAPI6004M']) + '</span></button>';
    confirmboxHtml += '<button id="button-cancel" class="btn-small cancel"><span class="text">' + (settings.cancel || i18n['KCHAPI6003M']) + '</span></button>';
    confirmboxHtml += '</div>';
    confirmboxHtml += '</footer>';
    confirmboxHtml += '</div>';
    var confirmboxNode = $(confirmboxHtml);
    $('#confirmbox-container').append(confirmboxNode);
    confirmboxNode.fadeIn();

    $('#confirmbox-container').on("click", "#button-confirm", function(e) {
        if (confirmCallback) {
            confirmCallback();
        }
        confirmboxNode.fadeOut(1, function() {
            $('#confirmbox-container').remove();
        });
    });
    $('#confirmbox-container').on("click", ".cancel", function(e) {
        if (cancelCallback) {
            cancelCallback();
        }
        confirmboxNode.fadeOut(1, function() {
            $('#confirmbox-container').remove();
        });
    });
};

kimchi.message.warn = function(msg, node) {
    kimchi.message(msg, 'warn', node);
};
kimchi.message.error = function(msg, node) {
    kimchi.message(msg, 'error', node);
};
kimchi.message.error.code = function(code) {
    msg = code + ": " + i18n[code]
    kimchi.message(msg, 'error');
};
kimchi.message.success = function(msg, node) {
    kimchi.message(msg, 'success', node);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.NETWORK_TYPE_BRIDGE = "bridged";

kimchi.initNetwork = function() {
    if(kimchi.tabMode['network'] === 'admin') {
        $('.tools').attr('style','display');
        $('#network-content .header span:last-child').attr('style','display');
        kimchi.initNetworkCreation();
    }
    kimchi.initNetworkListView();
    kimchi.initNetworkDialog();
    kimchi.initNetworkCleanup();
};

kimchi.initNetworkListView = function() {
    kimchi.listNetworks(function(data) {
        for (var i = 0; i < data.length; i++) {
            var network = {
                name : data[i].name,
                in_use : data[i].in_use,
                state : data[i].state === "active" ? "up" : "down"
            };
            if (data[i].connection === "bridge") {
                network.type = kimchi.NETWORK_TYPE_BRIDGE;
            } else {
                network.type = data[i].connection;
            }
            network.interface = data[i].interface ? data[i].interface : null;
            network.addrSpace = data[i].subnet ? data[i].subnet : null;
            network.persistent = data[i].persistent;
            kimchi.addNetworkItem(network);
        }
    });
};

kimchi.addNetworkItem = function(network) {
    $("#networkBody").append(kimchi.getNetworkItemHtml(network));
    if(kimchi.tabMode["network"] === "admin") {
        $(".column-action").attr("style","display");
    } else {
        $(".column-space").addClass('column-space-no-border-right');
    }
    kimchi.addNetworkActions(network);
};

kimchi.getNetworkItemHtml = function(network) {
    if(!network.interface) {
        network.interface = i18n["KCHNET6001M"];
    }
    if(!network.addrSpace) {
        network.addrSpace = i18n["KCHNET6001M"];
    }
    if(i18n["network_type_" + network.type]) {
        network.type = i18n["network_type_" + network.type];
    }

    var disable_in_use = network.in_use ? "ui-state-disabled" : "";
    var networkItem = kimchi.substitute($('#networkItem').html(), {
        name : network.name,
        state : network.state,
        type : network.type,
        interface: network.interface,
        addrSpace : network.addrSpace,
        startClass : network.state === "up" ? "hide-action-item" : "",
        stopClass : network.state === "down" ? "hide-action-item" : disable_in_use,
        stopDisabled : network.in_use ? "disabled" : "",
        deleteClass : network.state === "up" || network.in_use ? "ui-state-disabled" : "",
        deleteDisabled: network.state === "up" || network.in_use ? "disabled" : ""
    });
    return networkItem;
};

kimchi.stopNetwork = function(network,menu) {
    $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("up", "nw-loading");
    $("[nwAct='stop']", menu).addClass("ui-state-disabled");
    kimchi.toggleNetwork(network.name, false, function() {
        $("[nwAct='start']", menu).removeClass("hide-action-item");
        $("[nwAct='stop']", menu).addClass("hide-action-item");
        $("[nwAct='stop']", menu).removeClass("ui-state-disabled");
        if (!network.in_use) {
            $("[nwAct='delete']", menu).removeClass("ui-state-disabled");
            $(":first-child", $("[nwAct='delete']", menu)).removeAttr("disabled");
        }
        $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("nw-loading", "down");
    }, function(err) {
        $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("nw-loading", "up");
        if (!network.in_use) {
            $("[nwAct='stop']", menu).removeClass("ui-state-disabled");
        }
        kimchi.message.error(err.responseJSON.reason);
    });
}

kimchi.addNetworkActions = function(network) {
    $(".menu-container", "#" + kimchi.escapeStr(network.name)).menu();

    $('#' + kimchi.escapeStr(network.name)).on('click', '.menu-container li', function(evt) {
        var menu = $(evt.currentTarget).parent();
        if ($(evt.currentTarget).attr("nwAct") === "start") {
            $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("down", "nw-loading");
            $("[nwAct='start']", menu).addClass("ui-state-disabled");
            $("[nwAct='delete']", menu).addClass("ui-state-disabled");
            $(":first-child", $("[nwAct='delete']", menu)).attr("disabled", true);
            kimchi.toggleNetwork(network.name, true, function() {
                $("[nwAct='start']", menu).addClass("hide-action-item");
                $("[nwAct='start']", menu).removeClass("ui-state-disabled");
                $("[nwAct='stop']", menu).removeClass("hide-action-item");
                network.state = "up";
                if (network.in_use) {
                    $("[nwAct='stop']", menu).addClass("ui-state-disabled");
                }
                $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("nw-loading", "up");
            }, function(err) {
                $(".network-state", $("#" + kimchi.escapeStr(network.name))).switchClass("nw-loading","down");
                $("[nwAct='start']", menu).removeClass("ui-state-disabled");
                if (!network.in_use) {
                    $("[nwAct='delete']", menu).removeClass("ui-state-disabled");
                }
                $(":first-child", $("[nwAct='delete']", menu)).removeAttr("disabled");
                kimchi.message.error(err.responseJSON.reason);
            });
        } else if ($(evt.currentTarget).attr("nwAct") === "stop") {
            if (network.in_use) {
                return false;
            }
            if (!network.persistent) {
                var settings = {
                    title : i18n['KCHAPI6001M'],
                    content : i18n['KCHNET6004M'],
                    confirm : i18n['KCHAPI6002M'],
                    cancel : i18n['KCHAPI6003M']
                };
                kimchi.confirm(settings, function() {
                    kimchi.stopNetwork(network, menu);
                    $(evt.currentTarget).parents(".item").remove();
                }, null);
            }
            else {
                kimchi.stopNetwork(network, menu);
                network.state = "down";
            }
        } else if ($(evt.currentTarget).attr("nwAct") === "delete") {
            if (network.state === "up" || network.in_use) {
                return false;
            }
            kimchi.confirm({
                title : i18n['KCHAPI6006M'],
                content : i18n['KCHNET6002M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            }, function() {
                kimchi.deleteNetwork(network.name, function() {
                    $(evt.currentTarget).parents(".item").remove();
                });
            }, null);
        }
    });

    $("#networkBody .column-action .popable").button({
        icons : {
            secondary : "action-button-icon"
        }
    });

};

kimchi.initNetworkCreation = function() {
    $("#networkAdd").on("click", function() {
        kimchi.openNetworkDialog(function() {
            var errorCallback = function(){
                $("#networkFormOk").button("enable");
                $("#networkName").removeAttr("readonly");
                $("#networkFormOk span").text(i18n.KCHAPI6005M);
            };
            var network = kimchi.getNetworkDialogValues();
            var data = {
                name : network.name,
                connection: network.type
            };
            if (network.type === kimchi.NETWORK_TYPE_BRIDGE) {
                data.connection = "bridge";
                data.interface = network.interface;
                if ($("#enableVlan").prop("checked")) {
                    data.vlan_id = network.vlan_id;
                    if (!(data.vlan_id >=1 && data.vlan_id <= 4094)) {
                        kimchi.message.error.code('KCHNET6001E');
                        errorCallback();
                        return;
                    }
                }
            }
            kimchi.createNetwork(data, function(result) {
                network.state = result.state === "active" ? "up" : "down";
                network.interface = result.interface ? result.interface : i18n["KCHNET6001M"];
                network.addrSpace = result.subnet ? result.subnet : i18n["KCHNET6001M"];
                network.persistent = result.persistent;
                kimchi.addNetworkItem(network);
                $("#networkConfig").dialog("close");
            }, function(data) {
                kimchi.message.error(data.responseJSON.reason);
                errorCallback();
            });
        });
    });
};

kimchi.initNetworkDialog = function() {
    buttonsObj= {};
    buttonsObj['id'] = "networkFormOk";
    buttonsObj['text'] = i18n.KCHAPI6005M;
    buttonsObj['class'] = "ui-button-primary";
    buttonsObj['disabled'] = true;
    buttonsObj['click'] = function() { };
    $("#networkConfig").dialog({
        autoOpen : false,
        modal : true,
        width : 600,
        draggable : false,
        resizable : false,
        closeText: "X",
        dialogClass : "network-ui-dialog remove-when-logged-off",
        open : function(){
            $(".ui-dialog-titlebar-close", $("#networkConfig").parent()).removeAttr("title");
        },
        beforeClose : function() {
            kimchi.cleanNetworkDialog();
        },
        buttons : [buttonsObj]
    });
    kimchi.setupNetworkFormEvent();
};

kimchi.openNetworkDialog = function(okCallback) {
    kimchi.getInterfaces(function(result) {
        var options = [];
        $('#networkDestinationID').selectMenu();
        var nics = {};
        for (var i = 0; i < result.length; i++) {
            options.push({label:result[i].name,value:result[i].name});
            nics[result[i].name] = result[i];
        }
        result.length>0 && $("#networkDestinationID").selectMenu("setData", options);
        onChange = function() {
            $("#networkDestinationLabel").text($("#networkDestinationID li:first-child").text());
            $("#networkDestinationID li:first-child").addClass("active");
            if (result.length>0 && nics[$("#networkDestinationLabel").text()].type === "bridge") {
                $("#enableVlan").prop("checked", false);
                $("#enableVlan").prop("disabled", true);
                $("#networkVlanID").val("");
                $("#networkVlanID").toggle(false);
                $("#labelNetworkVlanID").toggle(false);
            } else {
                $("#enableVlan").prop("disabled",false);
            }
        };
        $("#networkDestinationLabel").on("change", onChange);
        kimchi.setDefaultNetworkType(result.length!==0);
        onChange();
    });
    $("#networkConfig").dialog({
        title : i18n.KCHNET6003M
    });
    $("#networkFormOk").on("click", function() {
        $("#networkFormOk").button("disable");
        $("#networkName").prop("readonly", "readonly");
        $("#networkFormOk span").text(i18n.KCHAPI6008M);
        okCallback();
    });
    $("#enableVlan").on("click", function() {
        $("#networkVlanID").prop("disabled", !this.checked);
        if (!this.checked) {
            $("#networkVlanID").slideUp(100);
            $("#labelNetworkVlanID").slideUp(100);
            $("#networkVlanID").val("");
        }
        else {
            $("#networkVlanID").slideDown(100);
            $("#labelNetworkVlanID").slideDown(100);
        }
    });
    $("#networkConfig").dialog("open");
};

kimchi.enableBridgeOptions = function(enable) {
    if (!enable) {
        $("#enableVlan").prop("checked", false);
        $("#networkVlanID").toggle(false);
        $("#labelNetworkVlanID").toggle(false);
        $("#networkVlanID").val("");
        $("#networkDestinationLabel").text("");
        $("#bridgeOptions").slideUp(100);
    } else if (!$("#networkDestinationLabel").text()){
        $("#networkDestinationLabel").text($("#networkDestinationID li:first-child").text());
        $("#bridgeOptions").slideDown(100);
        $("#networkVlanID").toggle(false);
        $("#labelNetworkVlanID").toggle(false);
    }
};


kimchi.setDefaultNetworkType = function(isInterfaceAvail) {
    $("#networkTypeBri").prop("checked", isInterfaceAvail);
    $("#networkTypeBri").prop("disabled", !isInterfaceAvail);
    $("#networkTypeNat").prop("checked", !isInterfaceAvail);
    if (!isInterfaceAvail) {
        kimchi.enableBridgeOptions(false);
        $("#networkBriDisabledLabel").show();
    } else {
        if (kimchi.capabilities && kimchi.capabilities.nm_running) {
            kimchi.message.warn(i18n['KCHNET6001W']);
        }
        $("#bridgeOptions").slideDown(100);
        $("#networkVlanID").toggle(false);
        $("#labelNetworkVlanID").toggle(false);
        $("#networkBriDisabledLabel").hide();
    }
};

kimchi.getNetworkDialogValues = function() {
    var network = {
        name : $("#networkName").val(),
        type : $("input:radio[name=networkType]:checked").val()
    };
    if (network.type === kimchi.NETWORK_TYPE_BRIDGE) {
        network.interface = $("#networkDestinationLabel").text();
        network.vlan_id = parseInt($("#networkVlanID").val());
    }
    return network;
};

kimchi.cleanNetworkDialog = function() {
    $("input:text", "#networkConfig").val(null).removeClass("invalid-field");
    $("#networkTypeIso").prop("checked", false);
    $("#networkTypeNat").prop("checked", false);
    $("#networkTypeBri").prop("checked", false);
    $("#networkDestinationLabel").text($("#networkDestinationID li:first-child").text());
    $("#networkFormOk").off("click");
    $("#networkFormOk").button("disable");
    $("#networkFormOk span").text(i18n.KCHAPI6005M);
    $("#networkName").removeAttr("readonly");
    $("#networkVlanID").toggle(false);
    $("#labelNetworkVlanID").toggle(false);
    $("#enableVlan").prop("checked", false);

};
kimchi.setupNetworkFormEvent = function() {
    $("#networkName").on("keyup", function(event) {
        $("#networkName").toggleClass("invalid-field", !$("#networkName").val().match(/^[^\"\/]+$/));
        kimchi.updateNetworkFormButton();
    });
    $("#networkTypeIso").on("click", function(event) {
        kimchi.enableBridgeOptions(false);
    });
    $("#networkTypeNat").on("click", function(event) {
        kimchi.enableBridgeOptions(false);
    });
    $("#networkTypeBri").on("click", function(event) {
        kimchi.enableBridgeOptions(true);
    });
};

kimchi.updateNetworkFormButton = function() {
    if($("#networkName").hasClass("invalid-field")){
        $("#networkFormOk").button("disable");
    }else{
        $("#networkFormOk").button("enable");
    }
};

kimchi.initNetworkCleanup = function() {
    $("#network-content").on("remove", function() {
        $("#networkConfig").dialog("destroy");
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Object.defineProperty(Object.prototype, "getDeepValue", {
    value: function(key) {
        var result=undefined;
        try {
            if(!Array.isArray(key)) {
                key=key.parseKey();
            }
            if(key.length!=0) {
                var tmpName=key.shift();
                if(this[tmpName]!=undefined) {
                    result=this[tmpName];
                }
                if(key.length!=0) {
                    result=result.getDeepValue(key);
                }
            }
        }
        catch (err) {
            //do nothing
        }
        return(result);
        }
});

Object.defineProperty(Object.prototype, "setDeepValue", {
    value: function(key, val) {
        var keys;
        if(Array.isArray(key)) {
            keys=key;
        }
        else {
            keys=key.parseKey();
        }
        if(keys.length!=0) {
            var key=keys.shift();
            if(keys.length==0) {
                if(this[key]==undefined) {
                    this[key]=val;
                }
                else if(Array.isArray(this[key])){
                    this[key].push(val);
                }
                else {
                    var tmpArray=[]
                    tmpArray.push(this[key]);
                    tmpArray.push(val);
                    this[key]=tmpArray;
                }
            }
            else {
                if(this[key]==undefined) {
                    this[key]=new Object();
                    this[key].setDeepValue(keys,val);
                }
                else if(Array.isArray(this[key])){
                    var tmpO=new Object();
                    this[key].push(tmpO);
                    tmpO.setDeepValue(keys,val);
                }
                else {
                   this[key].setDeepValue(keys,val);
                }
            }
        }
        return(this);
    }
});
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.popable = function() {
    $(document).click(function(e) {
        $('.popable').each(function(i, n) {
            n === e.target || $.contains(n, e.target) ||
                $('.popover', n).toggle(false);
        });
     });
     $(document).on('click', '.popable', function(e) {
        var popup = $('.popover', this)[0];
        $(popup).toggle();

        // Scroll the popup menu into viewport if invisible.
        !$(popup).is(':visible') ||
            kimchi.isElementInViewport(popup) ||
            popup.scrollIntoView();
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.report_add_main = function() {
    var reportGridID = 'available-reports-grid';
    var addReportForm = $('#form-report-add');
    var submitButton = $('#button-report-add');
    var nameTextbox = $('input[name="name"]', addReportForm);
    nameTextbox.select();

    var submitForm = function(event) {
        if(submitButton.prop('disabled')) {
            return false;
        }
        var reportName = nameTextbox.val();
        var validator = RegExp("^[_A-Za-z0-9-]*$");
        if (!validator.test(reportName)) {
            kimchi.message.error.code('KCHDR6011M');
            return false;
        }
        var formData = addReportForm.serializeObject();
        var taskAccepted = false;
        var onTaskAccepted = function() {
            if(taskAccepted) {
                return;
            }
            taskAccepted = true;
            kimchi.window.close();
            kimchi.topic('kimchi/debugReportAdded').publish();
        };

        kimchi.createReport(formData, function(result) {
            onTaskAccepted();
            kimchi.topic('kimchi/debugReportAdded').publish();
        }, function(result) {
            // Error message from Async Task status
            if (result['message']) {
                var errText = result['message'];
            }
            // Error message from standard kimchi exception
            else {
                var errText = result['responseJSON']['reason'];
            }
            result && kimchi.message.error(errText);

            taskAccepted &&
                $('.grid-body-view table tr:first-child',
                    '#' + reportGridID).remove();
            submitButton.prop('disabled', false);
            nameTextbox.select();
        }, onTaskAccepted);

        event.preventDefault();
    };

    addReportForm.on('submit', submitForm);
    submitButton.on('click', submitForm);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.report_rename_main = function() {
    var renameReportForm = $('#form-report-rename');
    var submitButton = $('#button-report-rename');
    var nameTextbox = $('input[name="name"]', renameReportForm);
    var submitForm = function(event) {
        if(submitButton.prop('disabled')) {
            return false;
        }
        var reportName = nameTextbox.val();

        // if the user hasn't changed the report's name,
        // nothing should be done.
        if (reportName == kimchi.selectedReport) {
            kimchi.message.error.code('KCHDR6013M');
            return false;
        }

        var validator = RegExp("^[A-Za-z0-9-]*$");
        if (!validator.test(reportName)) {
            kimchi.message.error.code('KCHDR6011M');
            return false;
        }
        var formData = renameReportForm.serializeObject();
        submitButton.prop('disabled', true);
        nameTextbox.prop('disabled', true);
        kimchi.renameReport(kimchi.selectedReport, formData, function(result) {
            submitButton.prop('disabled', false);
            nameTextbox.prop('disabled', false);
            kimchi.window.close();
            kimchi.topic('kimchi/debugReportRenamed').publish({
                result: result
            });
        }, function(result) {
            var errText = result &&
                result['responseJSON'] &&
                result['responseJSON']['reason'];
            kimchi.message.error(errText);
            submitButton.prop('disabled', false);
            nameTextbox.prop('disabled', false).focus();
        });

        event.preventDefault();
    };

    renameReportForm.on('submit', submitForm);
    submitButton.on('click', submitForm);

    nameTextbox.val(kimchi.selectedReport).select();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.repository_add_main = function() {

    var addForm = $('#form-repository-add');
    var addButton = $('#button-repository-add');

    var validateField = function(event) {
        var valid=($(this).val()!=='');
        $(addButton).prop('disabled', !valid);
        return(valid);
    };

    var validateForm = function(event) {
        var valid=false;
        addForm.find('input.required').each( function() {
            valid=($(this).val()!=='');
            return(!valid);
        });
        return(valid);
    }

    addForm.find('input.required').on('input propertychange', validateField);

    var weedObject = function(obj) {
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) {
                if((typeof(obj[key])==="object") && !Array.isArray(obj[key])) {
                    weedObject(obj[key]);
                }
                else if(obj[key] == '') {
                    delete obj[key];
                }
            }
        }
    }

    var addRepository = function(event) {
        var valid = validateForm();
        if(!valid) {
            return false;
        }

        var formData = $(addForm).serializeObject();

        if (formData && formData.isMirror!=undefined) {
            formData.isMirror=(String(formData.isMirror).toLowerCase() === 'true');
        }
        if(formData.isMirror) {
            if(formData.config==undefined) {
                formData.config=new Object();
            }
            formData.config.mirrorlist=formData.baseurl;
            delete formData.baseurl;
            delete formData.isMirror;
        }
        weedObject(formData);
        if(formData.config && formData.config.comps) {
            formData.config.comps=formData.config.comps.split(/[,\s]/);
            for(var i=0; i>formData.config.comps.length; i++) {
                formData.config.comps[i]=formData.config.comps[i].trim();
            }
            for (var j=formData.config.comps.indexOf(""); j!=-1; j=formData.config.comps.indexOf("")) {
                formData.config.comps.splice(j, 1);
            }
        }

        kimchi.createRepository(formData, function() {
            kimchi.topic('kimchi/repositoryAdded').publish();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            kimchi.message.error(reason);
        });
        return false;
    };

    $(addForm).on('submit', addRepository);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.repository_edit_main = function() {

    var editForm = $('#form-repository-edit');

    var saveButton = $('#repository-edit-button-save');

    if(kimchi.capabilities['repo_mngt_tool']=="yum") {
        editForm.find('input.deb').prop('disabled', true);
    }
    else if(kimchi.capabilities['repo_mngt_tool']=="deb") {
        editForm.find('input.yum').prop('disabled', true);
    }

    kimchi.retrieveRepository(kimchi.selectedRepository, function(repository) {
        editForm.fillWithObject(repository);

        $('input', editForm).on('input propertychange', function(event) {
            if($(this).val() !== '') {
                $(saveButton).prop('disabled', false);
            }
        });
    });


    var editRepository = function(event) {
        var formData = $(editForm).serializeObject();

        if (formData && formData.config) {
            formData.config.gpgcheck=(String(formData.config.gpgcheck).toLowerCase() === 'true');
        }

        if(formData.config && formData.config.comps) {
            formData.config.comps=formData.config.comps.split(/[,\s]/);
            for(var i=0; i>formData.config.comps.length; i++) {
                formData.config.comps[i]=formData.config.comps[i].trim();
            }
            for (var j=formData.config.comps.indexOf(""); j!=-1; j=formData.config.comps.indexOf("")) {
                formData.config.comps.splice(j, 1);
            }
        }

        kimchi.updateRepository(kimchi.selectedRepository, formData, function() {
            kimchi.topic('kimchi/repositoryUpdated').publish();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            kimchi.message.error(reason);
        });

        return false;
    };

    $(editForm).on('submit', editRepository);
    $(saveButton).on('click', editRepository);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.select = function(id, options) {
    var listControl = $('#'+ id);
    var targetId = listControl.data('target');
    var labelId = listControl.data('label');
    var value = $('#' + targetId).val();
    var item;
    var itemTag = 'li';
    var selectedClass = 'active';
    $.each(options, function(index, option) {
        item = $('<' + itemTag + '></' + itemTag + '>');
        item.text(option.label);
        item.data('value', option.value);
        if(option.value === value) {
            item.addClass(selectedClass);
            $('#' + labelId).text(option.label);
        }
        listControl.append(item);
    });

    listControl.on('click', itemTag, function() {
        listControl.children().removeClass(selectedClass);
        $(this).addClass(selectedClass);
        $('#' + labelId).text($(this).text());
        var target = $('#' + targetId);
        var oldValue = target.val();
        var newValue = $(this).data('value');
        target.val(newValue);
        if(oldValue !== newValue) {
            target.change();
        }
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.doListStoragePools = function() {
    kimchi.listStoragePools(function(result) {
        var storageHtml = $('#storageTmpl').html();
        if (result && result.length) {
            var listHtml = '';
            $.each(result, function(index, value) {
                value.usage = Math.round(value.allocated / value.capacity * 100) || 0;
                value.capacity = kimchi.changetoProperUnit(value.capacity,1);
                value.allocated = kimchi.changetoProperUnit(value.allocated,1);
                value.enableExt = value.type==="logical" ? "" : "hide-content";
                if ('kimchi-iso' !== value.type) {
                    listHtml += kimchi.substitute(storageHtml, value);
                }
            });
            $('#storagepoolsList').html(listHtml);
            if(kimchi.tabMode['storage'] === 'admin') {
                $('.storage-button').attr('style','display');
            } else {
                $('.storage-allocate').addClass('storage-allocate-padding-user');
            }
            kimchi.storageBindClick();
        } else {
            $('#storagepoolsList').html('');
        }
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
    });
}

kimchi.storageBindClick = function() {

    $('.inactive').each(function(index) {
        if ('active' === $(this).data('state')) {
            $(this).hide();
        } else {
            $(this).show();
        }
    });

    $('.list-storage .storage-state .active').each(function(index) {
        if ('active' === $(this).data('state')) {
            $(this).show();
        } else {
            $(this).hide();
        }
    });

    $('.pool-activate').each(function(index) {
        if ('active' === $(this).data('stat')) {
            $(this).hide();
        } else {
            $(this).show();
        }
    });

    $('.pool-deactivate').each(function(index) {
        if ('active' === $(this).data('stat')) {
            $(this).show();
        } else {
            $(this).hide();
        }
    });

    $('.pool-add-volume').each(function(index) {
        var canAddVolume =
            $(this).data('stat') === 'active' &&
            $(this).data('type') !== 'iscsi' &&
            $(this).data('type') !== 'scsi';
        if(canAddVolume) {
            $(this).show();
        }
        else {
            $(this).hide();
        }
    });

    if(kimchi.tabMode['storage'] === 'admin') {
        $('.pool-delete').on('click', function(event) {
            var $pool = $(this);
            var settings = {
                title : i18n['KCHAPI6001M'],
                content : i18n['KCHPOOL6001M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            };
            kimchi.confirm(settings, function() {
                var poolName = $pool.data('name');
                kimchi.deleteStoragePool(poolName, function() {
                    kimchi.doListStoragePools();
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            });
        });

        $('.pool-activate').on('click', function(event) {
            var poolName = $(this).data('name');
            kimchi.changePoolState(poolName, 'activate', function() {
                kimchi.doListStoragePools();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        });

        $('.pool-deactivate').on('click', function(event) {
            var poolName = $(this).data('name');
            var settings = {
                title : i18n['KCHAPI6001M'],
                content : i18n['KCHPOOL6012M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            };
            if (!$(this).data('persistent')) {
                kimchi.confirm(settings, function() {
                    kimchi.changePoolState(poolName, 'deactivate', function() {
                        kimchi.doListStoragePools();
                    }, function(err) {
                        kimchi.message.error(err.responseJSON.reason);
                    });
                }, function() {
                    return false;
                });
            }
            else {
                kimchi.changePoolState(poolName, 'deactivate', function() {
                    kimchi.doListStoragePools();
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            }
        });

        $('.pool-add-volume').on('click', function(event) {
            var poolName = $(this).data('name');
            kimchi.selectedSP = poolName;
            kimchi.window.open('storagepool-add-volume.html');
        });

        $('.storage-action').on('click', function() {
            var storage_action = $(this);
            var deleteButton = storage_action.find('.pool-delete');
            if ('active' === deleteButton.data('stat')) {
                deleteButton.attr('disabled', 'disabled');
            } else {
                deleteButton.removeAttr('disabled');
            }
        });

        $('.pool-extend').on('click', function() {
            $("#logicalPoolExtend").dialog("option", "poolName", $(this).data('name'));
            $("#logicalPoolExtend").dialog("open");
        });
    }

    $('.storage-li').on('click', function(event) {
        if (!$(event.target).parents().hasClass('bottom')) {
            if ($(this).data('stat') === 'active') {
                var that = $(this);
                var volumeDiv = $('#volume' + that.data('name'));
                var slide = $(this).next('.volumes');
                if (that.hasClass('in')) {
                    kimchi.doListVolumes(that);
                } else {
                    slide.slideUp('slow');
                    that.addClass('in');
                    kimchi.changeArrow(that.children().last().children());
                }
            }
        }
    });
}

kimchi._generateVolumeHTML = function(volume) {
    if(volume['type'] === 'kimchi-iso') {
        return '';
    }
    var volumeHtml = $('#volumeTmpl').html();
    volume.capacity = kimchi.changetoProperUnit(volume.capacity,1);
    volume.allocation = kimchi.changetoProperUnit(volume.allocation,1);
    return kimchi.substitute(volumeHtml, volume);
};

kimchi.doListVolumes = function(poolObj) {
    var poolName = poolObj.data('name')

    var getOngoingVolumes = function() {
        var result = {}
        var filter = 'status=running&target_uri=' + encodeURIComponent('^/storagepools/' + poolName + '/*')
        kimchi.getTasksByFilter(filter, function(tasks) {
            for(var i = 0; i < tasks.length; i++) {
                var volumeName = tasks[i].target_uri.split('/').pop();
                result[volumeName] = tasks[i];

                if(kimchi.trackingTasks.indexOf(tasks[i].id) >= 0) {
                    continue;
                }

                kimchi.trackTask(tasks[i].id, function(result) {
                    kimchi.topic('kimchi/volumeTransferFinished').publish(result);
                }, function(result) {
                    kimchi.topic('kimchi/volumeTransferError').publish(result);
                }, function(result) {
                    kimchi.topic('kimchi/volumeTransferProgress').publish(result);
                });
            }
        }, null, true);
        return result;
    };

    var volumeDiv = $('#volume' + poolName);
    $(volumeDiv).empty();
    var slide = poolObj.next('.volumes');
    var handleArrow = poolObj.children().last().children();

    kimchi.listStorageVolumes(poolName, function(result) {
        var listHtml = '';
        var ongoingVolumes = [];
        var ongoingVolumesMap = getOngoingVolumes();
        $.each(ongoingVolumesMap, function(volumeName, task) {
            ongoingVolumes.push(volumeName)
            var volume = {
                poolName: poolName,
                used_by: [],
                capacity: 0,
                name: volumeName,
                format: '',
                bootable: null,
                os_distro: '',
                allocation: 0,
                os_version: '',
                path: '',
                type: 'file'
            };
            listHtml += kimchi._generateVolumeHTML(volume);
        });

        $.each(result, function(index, value) {
            if (ongoingVolumes.indexOf(value.name) == -1) {
                value.poolname = poolName;
                listHtml += kimchi._generateVolumeHTML(value);
            }
        });

        if (listHtml.length > 0) {
            volumeDiv.html(listHtml);
        } else {
            volumeDiv.html("<div class='pool-empty'>" + i18n['KCHPOOL6002M'] + "</div>");
        }

        $.each(ongoingVolumesMap, function(volumeName, task) {
            kimchi.topic('kimchi/volumeTransferProgress').publish(task);
        });

        poolObj.removeClass('in');
        kimchi.changeArrow(handleArrow);
        slide.slideDown('slow');
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
    });
}

kimchi.initLogicalPoolExtend = function() {
    $("#logicalPoolExtend").dialog({
        autoOpen : false,
        modal : true,
        width : 600,
        resizable : false,
        closeText: "X",
        open : function(){
            $('#loading-info', '#logicalPoolExtend').removeClass('hidden');
            $(".ui-dialog-titlebar-close", $("#logicalPoolExtend").parent()).removeAttr("title");
            kimchi.listHostPartitions(function(data) {
                $('#loading-info', '#logicalPoolExtend').addClass('hidden');
                if (data.length > 0) {
                    for(var i=0;i<data.length;i++){
                        if (data[i].type === 'part' || data[i].type === 'disk') {
                            $('.host-partition', '#logicalPoolExtend').append(kimchi.substitute($('#logicalPoolExtendTmpl').html(), data[i]));
                        }
                    }
                } else {
                    $('.host-partition').html(i18n['KCHPOOL6011M']);
                    $('.host-partition').addClass('text-help');
                }
            }, function(err) {
                $('#loading-info', '#logicalPoolExtend').addClass('hidden');
                $('.host-partition').html(i18n['KCHPOOL6013M'] + '<br/>(' + err.responseJSON.reason + ')');
                $('.host-partition').addClass('text-help');
            });
        },
        beforeClose : function() { $('.host-partition', '#logicalPoolExtend').empty(); },
        buttons : [{
            class: "ui-button-primary",
            text: i18n.KCHAPI6007M,
            click: function(){
                var devicePaths = [];
                $("input[type='checkbox']:checked", "#logicalPoolExtend").each(function(){
                    devicePaths.push($(this).prop('value'));
                })
                kimchi.updateStoragePool($("#logicalPoolExtend").dialog("option", "poolName"),{disks: devicePaths},function(data){
                    var item = $("#"+$("#logicalPoolExtend").dialog("option", "poolName"));
                    $(".usage", $(".storage-name", item)).text((Math.round(data.allocated/data.capacity*100)||0)+"%");
                    $(".storage-text", $(".storage-capacity", item)).text(kimchi.changetoProperUnit(data.capacity,1));
                    $(".storage-text", $(".storage-allocate", item)).text(kimchi.changetoProperUnit(data.allocated,1));
                });
                $(this).dialog("close");
            }
        }]
    });
}

kimchi.storage_main = function() {
    if(kimchi.tabMode['storage'] === 'admin') {
        $('.tools').attr('style','display');
        $('#storage-pool-add').on('click', function() {
            kimchi.window.open('storagepool-add.html');
        });
        $('.list-title .title-actions').attr('style','display');
    }
    kimchi.doListStoragePools();
    kimchi.initLogicalPoolExtend();

    kimchi.topic('kimchi/storageVolumeAdded').subscribe(function() {
        pool = kimchi.selectedSP;
        var poolNode = $('.storage-li[data-name="' + pool + '"]');
        kimchi.doListVolumes(poolNode);
    });

    kimchi.topic('kimchi/volumeTransferProgress').subscribe(function(result) {
        var extractProgressData = function(data) {
            var sizeArray = /(\d+)\/(\d+)/g.exec(data) || [0, 0, 0];
            var downloaded = sizeArray[1];
            var percent = 0;
            if(downloaded) {
                var total = sizeArray[2];
                if(!isNaN(total)) {
                    percent = downloaded / total * 100;
                }
            }
            var formatted = kimchi.formatMeasurement(downloaded);
            var size = (1.0 * formatted['v']).toFixed(1) + formatted['s'];
            return {
                size: size,
                percent: percent
            };
        };

        var uriElements = result.target_uri.split('/');
        var poolName = uriElements[2];
        var volumeName = uriElements.pop();
        var progress = extractProgressData(result['message']);
        var size = progress['size'];
        var percent = progress['percent'];

        volumeBox = $('#volume' + poolName + ' [data-volume-name="' + volumeName + '"]');
        $('.progress-bar-inner', volumeBox).css({
            width: percent + '%'
        });
        $('.progress-transferred', volumeBox).text(size);
        $('.volume-progress', volumeBox).removeClass('hidden');
        $('.progress-status', volumeBox).text(i18n['KCHPOOL6014M']);
    });

    kimchi.topic('kimchi/volumeTransferFinished').subscribe(function(result) {
        var uriElements = result.target_uri.split('/');
        var poolName = uriElements[2];
        var volumeName = uriElements.pop();
        var volumeBox = $('#volume' + poolName + ' [data-volume-name="' + volumeName + '"]');
        $('.volume-progress', volumeBox).addClass('hidden');
        kimchi.getStoragePoolVolume(poolName, volumeName, function(volume) {
            var html = kimchi._generateVolumeHTML(volume);
            $(volumeBox).replaceWith(html);
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });

    kimchi.topic('kimchi/volumeTransferError').subscribe(function(result) {
        // Error message from Async Task status
        if (result['message']) {
            var errText = result['message'];
        }
        // Error message from standard kimchi exception
        else {
            var errText = result['responseJSON']['reason'];
        }
        result && kimchi.message.error(errText);

        var uriElements = result.target_uri.split('/');
        var poolName = uriElements[2];
        var volumeName = uriElements.pop();
        volumeBox = $('#volume' + poolName + ' [data-volume-name="' + volumeName + '"]');
        $('.progress-status', volumeBox).text(i18n['KCHPOOL6015M']);
    });
};

kimchi.changeArrow = function(obj) {
    if ($(obj).hasClass('arrow-down')) {
        $(obj).removeClass('arrow-down').addClass('arrow-up');
    } else {
        $(obj).removeClass('arrow-up').addClass('arrow-down');
    }
}
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.storagepool_add_main = function() {
    kimchi.initStorageAddPage();
    $('#form-pool-add').on('submit', kimchi.addPool);
    $('#pool-doAdd').on('click', kimchi.addPool);
    // 'pool-doAdd' button starts as disabled.
    $("#pool-doAdd").button();
    $("#pool-doAdd").button("disable");
    // Make any change in the form fields enables the
    // 'pool-doAdd' button if all the visible form
    // fields are filled, disables it otherwise.
    $('#form-pool-add').on('input change propertychange', function() {
        if (!kimchi.inputsNotBlank())
            $("#pool-doAdd").button("disable");
        else
           $("#pool-doAdd").button("enable");
    });
};

kimchi.storageFilterSelect = function(id, isUpdate) {
    var input = $('input', '#'+id);
    var options = $(".option", '#'+id);
    var filter = function(container, key){
        container.children().each(function(){
            $(this).css("display", $(this).text().indexOf(key)==-1 ? "none" : "");
        });
    };
    if(!isUpdate){
        input.on("keyup", function(){
            filter(options, input.val());
        });
    }
    options.children().each(function(){
        $(this).click(function(){
            options.children().removeClass("active");
            input.val($(this).text());
            input.trigger("change");
            $(this).addClass("active");
            filter(options, "");
        });
    });
};

kimchi.setupISCSI = function(){
    var loadTargets = function(server, port, callback){
        var isUpdate = $(".option", "#iSCSITarget").children().length > 0;
        $(".option", "#iSCSITarget").empty();
        $('input', "#iSCSITarget").attr("placeholder", i18n['KCHPOOL6006M']);
        kimchi.getISCSITargets(server, port, function(data){
            if(data.length==0){
                $('input', "#iSCSITarget").attr("placeholder", i18n['KCHPOOL6007M']);
            }else{
                for(var i=0; i<data.length; i++){
                    var itemNode = $.parseHTML("<li>"+data[i].target+"</li>");
                    $(".option", "#iSCSITarget").append(itemNode);
                }
                $('input', "#iSCSITarget").attr("placeholder", "");
                $(".popover", "#iSCSITarget").css("display", "block");
            }
            kimchi.storageFilterSelect('iSCSITarget', isUpdate);
            $('input', "#iSCSITarget").trigger("focus");
            callback();
        }, function(data){
            $('input', "#iSCSITarget").attr("placeholder", i18n['KCHPOOL6008M']);
            callback();
            kimchi.message.error(data.responseJSON.reason);
        });
    };
    var triggerLoadTarget = function(){
        $('input', "#iSCSITarget").val("");
        var server = $("#iscsiserverId").val().trim();
        var port = $("#iscsiportId").val().trim();
        if(server!="" && !$("#iscsiserverId").hasClass("invalid-field") && !$("#iscsiportId").hasClass("invalid-field")){
            $("#iscsiserverId").attr("disabled", true);
            $("#iscsiportId").attr("disabled", true);
            loadTargets(server, port, function(){
                $("#iscsiserverId").attr("disabled", false);
                $("#iscsiportId").attr("disabled", false);
            });
        }
    };
    $("#iscsiserverId").change(function(){
        $('input', "#iSCSITarget").off('focus', triggerLoadTarget);
        $('input', "#iSCSITarget").one('focus', triggerLoadTarget);
    });
    $("#iscsiportId").change(function(){
        $('input', "#iSCSITarget").off('focus', triggerLoadTarget);
        $('input', "#iSCSITarget").one('focus', triggerLoadTarget);
    });
    var initISCSIServers = function(){
        kimchi.getStorageServers("iscsi", function(data){
            for(var i=0;i<data.length;i++){
                var itemNode = $.parseHTML("<li>"+data[i].host+"</li>");
                $(".option", "#iSCSIServer").append(itemNode);
                $(itemNode).click(function(){
                    $("#iscsiportId").val($(this).prop("port"));
                    $("#iscsiserverId").val($(this).text());
                    triggerLoadTarget();
                }).prop("port", data[i].port);
            }
            kimchi.storageFilterSelect('iSCSIServer', false);
         });
    };
    initISCSIServers();
};

kimchi.initStorageAddPage = function() {
    kimchi.listHostPartitions(function(data) {
        if (data.length > 0) {
            var deviceHtml = $('#partitionTmpl').html();
            var listHtml = '';
            valid_types = ['part', 'disk', 'mpath'];
            $.each(data, function(index, value) {
                if (valid_types.indexOf(value.type) != -1) {
                    listHtml += kimchi.substitute(deviceHtml, value);
                }
            });
            $('.host-partition', '#form-pool-add').html(listHtml);
        } else {
            $('.host-partition').html(i18n['KCHPOOL6011M']);
            $('.host-partition').addClass('text-help');
        }
    }, function(err) {
        $('.host-partition').html(i18n['KCHPOOL6013M'] + '<br/>(' + err.responseJSON.reason + ')');
        $('.host-partition').addClass('text-help');
    });

    kimchi.getHostFCDevices(function(data){
        if(data.length>0){
            for(var i=0;i<data.length;i++){
                data[i].label = data[i].name;
                data[i].value = data[i].name;
            }
            $('#scsiAdapter').selectMenu();
            $("input", "#scsiAdapter").val(data[0].name);
            $('#scsiAdapter').selectMenu("setData", data);
        } else {
            $('#scsiAdapter').html(i18n['KCHPOOL6005M']);
            $('#scsiAdapter').addClass('text-help');
        }
    });

    $('#poolTypeId').selectMenu();
    $('#serverComboboxId').combobox();
    $('#targetFilterSelectId').filterselect();
    var options = [ {
        label : "DIR",
        value : "dir"
    }, {
        label : "NFS",
        value : "netfs"
    }, {
        label : "iSCSI",
        value : "iscsi"
    }, {
        label : "LOGICAL",
        value : "logical"
    }, {
        label : i18n.KCHPOOL6004M,
        value : "scsi"
    } ];
    $('#poolTypeId').selectMenu("setData", options);

    kimchi.getStorageServers('netfs', function(data) {
        var serverContent = [];
        if (data.length > 0) {
            $.each(data, function(index, value) {
                serverContent.push({
                    label : value.host,
                    value : value.host
                });
            });
        }
        $('#serverComboboxId').combobox("setData", serverContent);
        $('input[name=nfsServerType]').change(function() {
            if ($(this).val() === 'input') {
                $('#nfsServerInputDiv').removeClass('tmpl-html');
                $('#nfsServerChooseDiv').addClass('tmpl-html');
            } else {
                $('#nfsServerInputDiv').addClass('tmpl-html');
                $('#nfsServerChooseDiv').removeClass('tmpl-html');
            }
        });
        $('#nfsserverId').on("change keyup",function() {
            if ($(this).val() !== '' && kimchi.isServer($(this).val())) {
                $('#nfspathId').prop('disabled',false);
                $(this).removeClass("invalid-field");
            } else {
                $(this).addClass("invalid-field");
                $('#nfspathId').prop( "disabled",true);
            }
            $('#targetFilterSelectId').filterselect('clear');
        });
        $('#nfspathId').focus(function() {
            var targetContent = [];
            kimchi.getStorageTargets($('#nfsserverId').val(), 'netfs', function(data) {
                if (data.length > 0) {
                    $.each(data, function(index, value) {
                        targetContent.push({
                            label : value.target,
                            value : value.target
                        });
                    });
                }
                $('#targetFilterSelectId').filterselect("setData", targetContent);
            });
        });
    });

    $('#poolTypeInputId').change(function() {
        var poolObject = {'dir': ".path-section", 'netfs': '.nfs-section',
                          'iscsi': '.iscsi-section', 'scsi': '.scsi-section',
                          'logical': '.logical-section'}
        var selectType = $(this).val();
        $.each(poolObject, function(type, value) {
            if(selectType == type){
                $(value).removeClass('tmpl-html');
            } else {
                $(value).addClass('tmpl-html');
            }
        });
    });
    $('#authId').click(function() {
        if ($(this).prop("checked")) {
            $('.authenticationfield').removeClass('tmpl-html');
        } else {
            $('.authenticationfield').addClass('tmpl-html');
        }
    });
    $('#iscsiportId').keyup(function(event) {
        $(this).toggleClass("invalid-field",!/^[0-9]*$/.test($(this).val()));
    });
    $('#iscsiserverId').keyup(function(event) {
        $(this).toggleClass("invalid-field",!kimchi.isServer($(this).val().trim()));
    }).change(function(event) {
        $(this).toggleClass("invalid-field",!kimchi.isServer($(this).val().trim()));
    });
    kimchi.setupISCSI();
};

/* Returns 'true' if all form fields were filled, 'false' if
 * any field is left blank. The function takes into account
 * the current poolType selected.
 *
 * Any 'field is blank' verification that were done in other
 * validate functions were deleted, since we're doing it here
 * already.
 */
kimchi.inputsNotBlank = function() {
    if (!$('#poolId').val()) return false;
    var poolType = $("#poolTypeInputId").val();
    if (poolType === "dir") {
        if (!$('#pathId').val()) return false;
    } else if (poolType === "netfs") {
        if (!$('#nfspathId').val()) return false;
        if (!$('#nfsserverId').val()) return false;
    } else if (poolType === "iscsi") {
        if (!$('#iscsiserverId').val()) return false;
        if (!$('#iscsiTargetId').val()) return false;
    } else if (poolType === "logical") {
        if ($("input[name=devices]:checked").length === 0)
            return false;
    }
    return true;
};

kimchi.validateForm = function() {
    var poolType = $("#poolTypeInputId").val();
    if (poolType === "dir") {
        return kimchi.validateDirForm();
    } else if (poolType === "netfs") {
        return kimchi.validateNfsForm();
    } else if (poolType === "iscsi") {
        return kimchi.validateIscsiForm();
    } else if (poolType === "logical") {
        return kimchi.validateLogicalForm();
    } else {
        return true;
    }
};

kimchi.validateDirForm = function () {
    var path = $('#pathId').val();
    if (!/(^\/.*)$/.test(path)) {
        kimchi.message.error.code('KCHAPI6003E');
        return false;
    }
    return true;
};

kimchi.validateNfsForm = function () {
    var nfspath = $('#nfspathId').val();
    var nfsserver = $('#nfsserverId').val();
    if (!kimchi.validateServer(nfsserver)) {
        return false;
    }
    if (!/((\/([0-9a-zA-Z-_\.]+)))$/.test(nfspath)) {
        kimchi.message.error.code('KCHPOOL6005E');
        return false;
    }
    $('#nfs-mount-loading').removeClass('hidden');
    return true;
};

kimchi.validateIscsiForm = function() {
    var iscsiServer = $('#iscsiserverId').val();
    var iscsiTarget = $('#iscsiTargetId').val();
    if (!kimchi.validateServer(iscsiServer)) {
        return false;
    }
    return true;
};

kimchi.validateServer = function(serverField) {
    if(!kimchi.isServer(serverField)) {
        kimchi.message.error.code('KCHPOOL6009E');
        return false;
    }
    return true;
};

kimchi.validateLogicalForm = function () {
    if ($("input[name=devices]:checked").length === 0) {
        kimchi.message.error.code('KCHPOOL6006E');
        return false;
    } else {
        return true;
    }
};

kimchi.addPool = function(event) {
    if (kimchi.validateForm()) {
        var formData = $('#form-pool-add').serializeObject();
        delete formData.authname;
        var poolType = $('#poolTypeId').selectMenu('value');
        if (poolType === 'dir') {
            formData.path = $('#pathId').val();
        } else if (poolType === 'logical') {
            var source = {};
            if (!$.isArray(formData.devices)) {
                var deviceObj = [];
                deviceObj[0] =  formData.devices;
                source.devices = deviceObj;
            } else {
                source.devices = formData.devices;
            }
            delete formData.devices;
            formData.source = source;
        } else if (poolType === 'netfs'){
            var source = {};
            source.path = $('#nfspathId').val();
            source.host = $('#nfsserverId').val();
            formData.source = source;
        } else if (poolType === 'iscsi') {
            var source = {};
            source.target = $('#iscsiTargetId').val();
            source.host = $('#iscsiserverId').val();
            $('#iscsiportId').val() !== '' ? source.port = parseInt($('#iscsiportId').val()): null;
            if ($('#authId').prop("checked")) {
                source.auth = {
                    "username" : $('#usernameId').val(),
                    "password" : $('#passwordId').val()
                };
            }
            formData.source = source;
        } else if (poolType === 'scsi'){
            formData.source = { adapter_name: $('#scsiAdapter').selectMenu('value') };
        }
        var storagePoolAddingFunc = function() {
            $('input', '#form-pool-add').attr('disabled','disabled');
            $('#pool-doAdd').hide();
            $('#pool-loading').show();
            kimchi.createStoragePool(formData, function() {
                    kimchi.doListStoragePools();
                    kimchi.window.close();
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                    $('input', '#form-pool-add').removeAttr('disabled');
                    $('#pool-loading').hide();
                    $('#pool-doAdd').show();
                });
        };
        if (poolType === 'logical') {
            var settings = {
                title : i18n['KCHAPI6001M'],
                content : i18n['KCHPOOL6003M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            };
            kimchi.confirm(settings, function() {
                storagePoolAddingFunc();
            }, function() {
            });
        } else {
            storagePoolAddingFunc();
        }
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014-2015
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.sp_add_volume_main = function() {
    // download from remote server or upload from local file
    var type = 'download';

    var addButton = $('#sp-add-volume-button');
    var remoteURLBox = $('#volume-remote-url');
    var localFileBox = $('#volume-input-file');
    var typeRadios = $('input.volume-type');

    var isValidURL = function() {
        var url = $(remoteURLBox).val();
        return kimchi.template_check_url(url);
    };

    var isValidFile = function() {
        var fileName = $(localFileBox).val();
        return fileName.length > 0;
    };

    $(typeRadios).change(function(event) {
        $('.volume-input').prop('disabled', true);
        $('.volume-input.' + this.value).prop('disabled', false);
        type = this.value;
        if(type == 'download') {
            $(addButton).prop('disabled', !isValidURL());
        }
        else {
            $(addButton).prop('disabled', !isValidFile());
        }
    });

    $(remoteURLBox).on('input propertychange', function(event) {
        $(addButton).prop('disabled', !isValidURL());
    });

    $(localFileBox).on('change', function(event) {
        $(addButton).prop('disabled', !isValidFile());
    });

    var onError = function(result) {
        $(this).prop('disabled', false);
        $(typeRadios).prop('disabled', false);
        if(!result) {
            return;
        }
        var msg = result['message'] || (
            result['responseJSON'] && result['responseJSON']['reason']
        );
        kimchi.message.error(msg);
    };

    var fetchRemoteFile = function() {
        var volumeURL = remoteURLBox.val();
        var volumeName = volumeURL.split(/(\\|\/)/g).pop();
        kimchi.downloadVolumeToSP({
            sp: kimchi.selectedSP,
            url: volumeURL
        }, function(result) {
            kimchi.window.close();
            kimchi.topic('kimchi/storageVolumeAdded').publish();
        }, onError);
    };

    var uploadFile = function() {
        var chunkSize = 8 * 1024 * 1024; // 8MB
        var uploaded = 0;

        var blobFile = $(localFileBox)[0].files[0];

        var createUploadVol = function() {
            kimchi.createVolumeWithCapacity(kimchi.selectedSP, {
                name: blobFile.name,
                format: '',
                capacity: blobFile.size,
                upload: true
            }, function(result) {
                kimchi.window.close();
                trackVolCreation(result.id);
            }, onError);
        };

        var uploadRequest = function(blob) {
            var fd = new FormData();
            fd.append('chunk', blob);
            fd.append('chunk_size', blob.size);

            kimchi.uploadVolumeToSP(kimchi.selectedSP, blobFile.name, {
                formData: fd
            }, function(result) {
                if (uploaded < blobFile.size)
                    setTimeout(doUpload, 500);
            }, onError);

            uploaded += blob.size
        };

        // Check file exists and has read permission
        try {
            var blob = blobFile.slice(0, 20);
            var reader = new FileReader();
            reader.onloadend = function(e) {
                if (e.loaded == 0)
                    kimchi.message.error.code('KCHAPI6008E');
                else
                    createUploadVol();
            };

            reader.readAsBinaryString(blob);
        } catch (err) {
            kimchi.message.error.code('KCHAPI6008E');
            return;
        }

        var doUpload = function() {
            try {
                var blob = blobFile.slice(uploaded, uploaded + chunkSize);
                var reader = new FileReader();
                reader.onloadend = function(e) {
                    if (e.loaded == 0)
                        kimchi.message.error.code('KCHAPI6009E');
                    else
                        uploadRequest(blob);
                };

                reader.readAsBinaryString(blob);
            } catch (err) {
                kimchi.message.error.code('KCHAPI6009E');
                return;
            }
        }

        var trackVolCreation = function(taskid) {
            var onTaskResponse = function(result) {
                var taskStatus = result['status'];
                var taskMsg = result['message'];
                if (taskStatus == 'running') {
                    if (taskMsg != 'ready for upload') {
                        setTimeout(function() {
                            trackVolCreation(taskid);
                        }, 2000);
                    } else {
                        kimchi.topic('kimchi/storageVolumeAdded').publish();
                        doUpload();
                    }
                }
            };
            kimchi.getTask(taskid, onTaskResponse, onError);
        };
    };

    $(addButton).on('click', function(event) {
        $(this).prop('disabled', true);
        $(typeRadios).prop('disabled', true);
        if(type === 'download') {
            fetchRemoteFile();
        }
        else {
            uploadFile();
        }
        event.preventDefault();
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Object.defineProperty(String.prototype, "parseKey", {
    value: function(parsedKey) {
        try {
            if (!Array.isArray(parsedKey)) {
                parsedKey=[];
            }
        }
        catch (err) {
            parsedKey=[];
        }
        var openBracket=this.indexOf("[");
        if (openBracket!=-1) {
            var id=this.slice(0, openBracket);
            parsedKey.push(id);
            var closeBracket=this.lastIndexOf("]");
            if (closeBracket==-1) {
                closeBracket=this.length;
            }
            var tmpName=this.slice(openBracket+1,closeBracket);
            tmpName.parseKey(parsedKey);
        }
        else {
            parsedKey.push(this);
        }
        return(parsedKey);
        }
});
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.substitute = function(templateStr, data, tag) {
    tag = tag || /\{([^\}]+)\}/g;

    var escapeHtml = function(html) {
        return String(html)
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;');
    };

    return templateStr.replace(tag, function(matchResult, express) {
        var propertyArray = express.split('!');
        var defaultValue = propertyArray[1] || '';
        propertyArray = propertyArray[0].split('.');
        var value = data, i = 0, l = propertyArray.length, property;
        for (; i < l; i++) {
            property = propertyArray[i];
            if (value) {
                value = value[property];
            } else {
                break;
            }
        }
        return escapeHtml((value || value === 0) ? value : defaultValue);
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.switchPage = function(fromPageId, toPageId, direction) {
    direction = direction || 'left';
    var toLeftBegin;
    var fromLeftEnd;
    if('left' === direction) {
        toLeftBegin = '100%';
        fromLeftEnd = '-100%';
    } else if('right' === direction) {
        toLeftBegin = '-100%';
        fromLeftEnd = '100%';
    }
    var formPage = $('#'+fromPageId);
    var toPage = $('#'+toPageId);
    toPage.css({
        left: toLeftBegin
    });
    formPage.animate({
        left: fromLeftEnd,
        opacity: 0.1
    }, 400);
    toPage.animate({
        left: '0',
        opacity: 1
    }, 400);
};

kimchi.template_add_main = function() {
    kimchi.deepScanHandler = null;
    // 1-1 local iso
    $('#iso-local').click(function() {
        kimchi.switchPage('iso-type-box', 'iso-local-box');
        initLocalIsoField();
        initIsoFileField();
        kimchi.listIsos(function(isos) {
            if (isos && isos.length) {
                showLocalIsoField(isos);
                $('#iso-more').show();
            } else {
                $('#iso-search').show();
            }
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });

    $('#iso-local-box-back').click(function() {
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
        kimchi.switchPage('iso-local-box', 'iso-type-box', 'right');
    });

    $('#iso-search').click(function() {
        var settings = {
            content : i18n['KCHTMPL6002M']
        };
        kimchi.confirm(settings, function() {
            $('#iso-search').hide();
            $('#iso-search-loading').show();
            deepScan('#iso-search');
        });
    });

    $('#iso-more').click(function() {
        var settings = {
            content : i18n['KCHTMPL6002M']
        };
        kimchi.confirm(settings, function() {
            $('#iso-more').hide();
            $('#iso-more-loading').show();
            deepScan('#iso-more');
        });
    });

    $('#iso-search-loading').click(function() {
        $('#iso-search-loading').hide();
        $('#iso-search').show();
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
    });

    $('#iso-more-loading').click(function() {
        $('#iso-more-loading').hide();
        $('#iso-more').show();
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
    });

    var deepScan = function(button) {
        kimchi.deepScanHandler = kimchi.stepListDeepScanIsos(function(isos, isFinished) {
            if (isos && isos.length) {
                if(button === '#iso-search') {
                    $(button + '-loading').hide();
                    button = '#iso-more';
                    $(button + '-loading').show();
                }
                showLocalIsoField(isos);
            } else {
                if (isFinished) {
                    kimchi.message.warn(i18n['KCHTMPL6001W']);
                }
            }
            if (isFinished) {
                $(button + '-loading').hide();
                $(button).show();
            }
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
            $(button + '-loading').hide();
            $(button).show();
        });
    };

    //1-1-1 local iso list
    var initLocalIsoField = function() {
        kimchi.isoInfo = {};
        $('#local-iso-field').hide();
        $('#select-all-local-iso').prop('checked', false);
        $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        $('#iso-search').hide();
        $('#iso-more').hide();
        $('#iso-search-loading').hide();
        $('#iso-more-loading').hide();
        $('#list-local-iso').empty();
    };

    var showLocalIsoField = function(isos) {
        var html = '';
        var template = $('#tmpl-list-local-iso').html();
        $.each(isos, function(index, volume) {
            var isoId = volume.os_distro + '*' + volume.name + '*' + volume.os_version;
            if (!kimchi.isoInfo[isoId]) {
                volume.isoId = isoId;
                volume.capacity = kimchi.changetoProperUnit(volume.capacity, 1);
                kimchi.isoInfo[isoId] = volume;
                html += kimchi.substitute(template, volume);
            }
        });
        $('#list-local-iso').append(html);
        $('#local-iso-field').show();
    };

    $('#select-all-local-iso').click(function() {
        $('#list-local-iso [type="checkbox"]').prop('checked', $(this).prop('checked'));
        if ($(this).prop('checked')) {
            $('#btn-template-local-iso-create').removeAttr('disabled');
        } else {
            $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#list-local-iso').on('click', '[type="checkbox"]', function() {
        var checkedLength = $('#list-local-iso [type="checkbox"]:checked').length;
        if (checkedLength) {
            $('#btn-template-local-iso-create').removeAttr('disabled');
            var length = $('#list-local-iso [type="checkbox"]').length;
            $('#select-all-local-iso').prop('checked', length == checkedLength);
        } else {
            $('#select-all-local-iso').prop('checked', false);
            $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-local-iso-create').click(function() {
        submitIso('form-local-iso');
    });

    //1-1-2 local iso file
    var initIsoFileField = function() {
        $('#iso-file-check').prop('checked', false);
        $('#iso-file-box').hide();
        $('#iso-file').val('');
        $('#btn-template-file-create').attr('disabled', 'disabled');
    };

    $('#iso-file-check').click(function() {
        if ($(this).prop('checked')) {
            $('#iso-file-box').slideDown();
        } else {
            $('#iso-file-box').slideUp();
        }
    });

    $('#iso-file').on('input propertychange', function() {
        if ($('#iso-file').val()) {
            $('#btn-template-file-create').removeAttr('disabled');
        } else {
            $('#btn-template-file-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-file-create').click(function() {
        var isoFile = $('#iso-file').val();
        if (!kimchi.template_check_path(isoFile)) {
            kimchi.message.error.code('KCHAPI6003E');
            return;
        }
        var data = {
            "cdrom" : isoFile
        };
        addTemplate(data);
    });

    //1-2 remote iso
    $('#iso-remote').css('opacity', 0.3).css('cursor', 'not-allowed');

    var enabledRemoteIso = function() {
        if (kimchi.capabilities == undefined) {
            setTimeout(enabledRemoteIso, 2000);
            return;
        }

        if (kimchi.capabilities.qemu_stream != true) {
            return;
        }

        $('#iso-remote').css('opacity', 1).css('cursor', 'pointer');
        $('#iso-remote').click(function() {
            kimchi.switchPage('iso-type-box', 'iso-remote-box');
            initRemoteIsoField();
            initIsoUrlField();
            kimchi.listDistros(function(isos) {
                showRemoteIsoField(isos);
            }, function() {
            });
        });
    };
    enabledRemoteIso();

    $('#iso-remote-box-back').click(function() {
        kimchi.switchPage('iso-remote-box', 'iso-type-box', 'right');
    });

    //1-2-1 remote iso list
    var initRemoteIsoField = function() {
        $('#load-remote-iso').show();
        $('#remote-iso-field').hide();
        $('#iso-url-field').hide();
        $('#select-all-remote-iso').prop('checked', false);
        $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
    };

    var showRemoteIsoField = function(isos) {
        if (isos && isos.length) {
            kimchi.isoInfo = {};
            var html = '';
            var template = $('#tmpl-list-remote-iso').html();
            $.each(isos, function(index, volume) {
                var isoId = volume.os_distro + '*' + volume.name + '*' + volume.os_version;
                if (!kimchi.isoInfo[isoId]) {
                    volume.isoId = isoId;
                    kimchi.isoInfo[isoId] = volume;
                    html += kimchi.substitute(template, volume);
                }
            });
            $('#list-remote-iso').html(html);
            $('#load-remote-iso').hide()
            $('#remote-iso-field').show();
            $('#iso-url-field').show();
        } else {
            $('#load-remote-iso').hide()
            $('#iso-url-field').show();
            kimchi.message.warn(i18n['KCHTMPL6001W']);
        }
    };

    $('#select-all-remote-iso').click(function() {
        $('#list-remote-iso [type="checkbox"]').prop('checked', $(this).prop('checked'));
        if ($(this).prop('checked')) {
            $('#btn-template-remote-iso-create').removeAttr('disabled');
        } else {
            $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#list-remote-iso').on('click', '[type="checkbox"]', function() {
        var checkedLength = $('#list-remote-iso [type="checkbox"]:checked').length;
        if (checkedLength) {
            $('#btn-template-remote-iso-create').removeAttr('disabled');
            var length = $('#list-remote-iso [type="checkbox"]').length;
            $('#select-all-remote-iso').prop('checked', length == checkedLength);
        } else {
            $('#select-all-remote-iso').prop('checked', false);
            $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-remote-iso-create').click(function() {
        submitIso('form-remote-iso');
    });

    //1-2-2 remote iso url
    var initIsoUrlField = function() {
        $('#iso-url-check').prop('checked', false);
        $('#iso-url-box').hide();
        $('#iso-url').val('');
        $('#btn-template-url-create').attr('disabled', 'disabled');
    }

    $('#iso-url-check').click(function() {
        if ($(this).prop('checked')) {
            $('#iso-url-box').slideDown();
        } else {
            $('#iso-url-box').slideUp();
        }
    });

    $('#iso-url').on('input propertychange', function() {
        if ($('#iso-url').val()) {
            $('#btn-template-url-create').removeAttr('disabled');
        } else {
            $('#btn-template-url-create').attr('disabled', 'disabled');
        }
    });

    $('#vm-image-local').click(function(){
        kimchi.switchPage('iso-type-box', 'vm-image-local-box');
    });
    $('#vm-image-local-box-back').click(function(){
        kimchi.switchPage('vm-image-local-box', 'iso-type-box', 'right');
    });
    $('input', '#vm-image-local-box').on('keyup cut paste', function(){
        setTimeout(function(){
            var isValid = kimchi.template_check_path($('input', '#vm-image-local-box').val());
            $('input', '#vm-image-local-box').toggleClass('invalid-field', !isValid);
            $('button', $('.body', '#vm-image-local-box')).button(isValid ? "enable" : "disable");
        }, 0);
    });
    $('button', $('.body', '#vm-image-local-box')).button({
        disabled: true
    }).click(function(){
        $('input', '#vm-image-local-box').prop('disabled', true);
        $(this).button('option', {
            label: i18n['KCHAPI6008M'],
            disabled: true
        });
        addTemplate({disks:[{base:$('input', '#vm-image-local-box').val()}]}, function(){
            $('input', '#vm-image-local-box').prop('disabled', false);
            $('button', $('.body', '#vm-image-local-box')).button('option', {
                label: i18n['KCHAPI6005M'],
                disabled: false
            });
        });
    });

    $('#btn-template-url-create').click(function() {
        var isoUrl = $('#iso-url').val();
        if (!kimchi.template_check_url(isoUrl)) {
            kimchi.message.error.code('KCHAPI6004E');
            return;
        }
        var data = {
            "cdrom" : isoUrl
        };
        addTemplate(data);
    });

    //do create
    var addTemplate = function(data, callback) {
        kimchi.createTemplate(data, function() {
            if(callback) callback();
            kimchi.doListTemplates();
            kimchi.window.close();
            kimchi.topic('templateCreated').publish();
        }, function(err) {
            if(callback) callback();
            kimchi.message.error(err.responseJSON.reason);
        });
    };

    var submitIso = function(formId) {
        var formData = $('#' + formId).serializeObject();
        if (formData.iso) {
            var length = 0;
            var successNum = 0;
            var addTemplate = function(isoInfo) {
                var data = {
                    "os_distro" : isoInfo.os_distro,
                    "os_version" : isoInfo.os_version,
                    "cdrom" : isoInfo.path
                };
                kimchi.createTemplate(data, function() {
                    successNum++;
                    $('input[value="' + isoInfo.isoId + '"]').prop('checked', false);
                    $('.check-all>input').prop('checked', false);
                    kimchi.doListTemplates();
                    kimchi.topic('templateCreated').publish(data);
                    if (successNum === length) {
                        kimchi.window.close();
                    }
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            };
            if (formData.iso instanceof Array) {
                length = formData.iso.length;
                $.each(formData.iso, function(index, value) {
                    addTemplate(kimchi.isoInfo[value]);
                });
            } else {
                length = 1;
                addTemplate(kimchi.isoInfo[formData.iso]);
            }
        }
    };
};

kimchi.template_check_url = function(url) {
    var reg = /(https|http|ftp|ftps|tftp):\/\//;
    if (url.constructor === String) {
        return reg.test(url);
    }
    return false;
};

kimchi.template_check_path = function(filePath) {
    var reg = /((\/([0-9a-zA-Z-_ \.]+))+)$/;
    if (filePath.constructor === String) {
        return reg.test(filePath);
    }
    return false;
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.template_edit_main = function() {
    var templateEditMain = $('#edit-template-tabs');
    var origDisks;
    var origPool;
    var origNetworks;
    var templateDiskSize;
    $('#template-name', templateEditMain).val(kimchi.selectedTemplate);
    templateEditMain.tabs();

    var initTemplate = function(template) {
        origDisks =  template.disks;
        origPool = template.storagepool;
        origNetworks = template.networks;
        for(var i=0;i<template.disks.length;i++){
            if(template.disks[i].base){
                template["vm-image"] = template.disks[i].base;
                $('.templ-edit-cdrom').addClass('hide');
                $('.templ-edit-vm-image').removeClass('hide');
                break;
            }
        }
        for ( var prop in template) {
            var value = template[prop];
            if (prop == 'graphics') {
               value = value["type"];
            }
            $('input[name="' + prop + '"]', templateEditMain).val(value);
        }

        var vncOpt = [{label: 'VNC', value: 'vnc'}];
        $('#template-edit-graphics').append('<option selected>VNC</option>');
        $('#template-edit-graphics').append('<option>Spice</option>');
        kimchi.select('template-edit-graphics-list', vncOpt);
        var enableSpice = function() {
            if (kimchi.capabilities == undefined) {
                setTimeout(enableSpice, 2000);
                return;
            }
            if (kimchi.capabilities.qemu_spice == true) {
                spiceOpt = [{label: 'Spice', value: 'spice'}]
                kimchi.select('template-edit-graphics-list', spiceOpt);
            }
        };
        enableSpice();
        var initStorage = function(result) {
            var scsipools = {};
            var addStorageItem = function(storageData) {
                var thisName = storageData.storageName;
                var nodeStorage = $.parseHTML(kimchi.substitute($('#template-storage-pool-tmpl').html(), storageData));
                $('.template-tab-body', '#form-template-storage').append(nodeStorage);
                var storageOptions = '';
                var scsiOptions = '';
                $('#selectStorageName').find('option').remove();
                $.each(result, function(index, storageEntities) {
                    if((storageEntities.state === 'active') && (storageEntities.type != 'kimchi-iso')) {
                        if(storageEntities.type === 'iscsi' || storageEntities.type === 'scsi') {
                            kimchi.listStorageVolumes(storageEntities.name, function(currentVolume) {
                                $.each(currentVolume, function(indexSCSI, scsiEntities) {
                                    var tmpPath = storageEntities.name + '/' + scsiEntities.name;
                                    var isSlected = tmpPath === thisName ? ' selected' : '';
                                    scsiOptions += '<option' + isSlected + '>' + tmpPath + '</option>';
                                });
                                $('#selectStorageName').append(scsiOptions);
                            }, function() {});
                        } else {
                            var isSlected = storageEntities.name === thisName ? ' selected' : '';
                            storageOptions += '<option' + isSlected + '>' + storageEntities.name + '</option>';
                        }
                    }
                });
                $('#selectStorageName').append(storageOptions);

                // Set disk format
                $('#diskFormat').val(storageData.storageDiskFormat);
                $('#diskFormat').on('change', function() {
                    $('.template-storage-disk-format').val($(this).val());
                });

                $('#selectStorageName').change(function() {
                    var selectedItem = $(this).parent().parent();
                    var tempStorageNameFull = $(this).val();
                    var tempName = tempStorageNameFull.split('/');
                    var tempStorageName = tempName[0];
                    $('.template-storage-name').val(tempStorageNameFull);
                    kimchi.getStoragePool(tempStorageName, function(info) {
                        tempType = info.type;
                        selectedItem.find('.template-storage-type').val(tempType);
                        if (tempType === 'iscsi' || tempType === 'scsi') {
                            kimchi.getStoragePoolVolume(tempStorageName, tempName[tempName.length-1], function(info) {
                                volSize = info.capacity / Math.pow(1024, 3);
                                $('.template-storage-disk', selectedItem).attr('readonly', true).val(volSize);
                                $('#diskFormat').val('raw');
                                $('#diskFormat').prop('disabled', true).change();
                            });
                        } else if (tempType === 'logical') {
                            $('.template-storage-disk', selectedItem).attr('readonly', false);
                            $('#diskFormat').val('raw');
                            $('#diskFormat').prop('disabled', true).change();
                        } else {
                            $('.template-storage-disk', selectedItem).attr('readonly', false);
                            if ($('#diskFormat').prop('disabled') == true) {
                                $('#diskFormat').val('qcow2');
                                $('#diskFormat').prop('disabled', false).change();
                            }
                        }
                    });
                });
            };

            if ((origDisks && origDisks.length) && (origPool && origPool.length)) {
                splitPool = origPool.split('/');
                var defaultPool = splitPool[splitPool.length-1];
                var defaultType;

                kimchi.getStoragePool(defaultPool, function(info) {
                    defaultType = info.type;
                    $.each(origDisks, function(index, diskEntities) {
                        var storageNodeData = {
                            viewMode : '',
                            editMode : 'hide',
                            storageName : defaultPool,
                            storageType : defaultType,
                            storageDisk : diskEntities.size,
                            storageDiskFormat : diskEntities.format ? diskEntities.format : 'qcow2'
                        }

                        if (diskEntities.volume) {
                            kimchi.getStoragePoolVolume(defaultPool, diskEntities.volume, function(info) {
                                var volSize = info.capacity / Math.pow(1024, 3);
                                var nodeData = storageNodeData
                                nodeData.storageName = defaultPool + '/' + diskEntities.volume;
                                nodeData.storageDisk = volSize;
                                addStorageItem(nodeData);
                                $('.template-storage-disk').attr('readonly', true);
                                $('#diskFormat').val('raw');
                                $('#diskFormat').prop('disabled', true).change();
                            });
                        } else if (defaultType === 'logical') {
                            addStorageItem(storageNodeData);
                            $('#diskFormat').val('raw');
                            $('#diskFormat').prop('disabled', true).change();
                        } else {
                            addStorageItem(storageNodeData);
                        }
                    });
                });
            }

            $('#template-edit-storage-add-button').button({
                icons: {
                    primary: "ui-icon-plusthick"
                },
                text: false,
                disabled: true
            }).click(function(event) {
                event.preventDefault();
                var storageNodeData = {
                    viewMode : 'hide',
                    editMode : '',
                    storageName : 'null',
                    storageType : 'dir',
                    storageDisk : '10'
                }
                addStorageItem(storageNodeData);
            });
        };
        var initInterface = function(result) {
            var networkItemNum = 0;
            var addInterfaceItem = function(networkData) {
                var networkName = networkData.networkV;
                var nodeInterface = $.parseHTML(kimchi.substitute($('#template-interface-tmpl').html(), networkData));
                $('.template-tab-body', '#form-template-interface').append(nodeInterface);
                $('.delete', '#form-template-interface').button({
                    icons : {primary : 'ui-icon-trash'},
                    text : false
                }).click(function(evt) {
                    evt.preventDefault();
                    $(this).parent().parent().remove();
                });
                var networkOptions = '';
                for(var i=0;i<result.length;i++){
                    if(result[i].state === "active") {
                        var isSlected = networkName===result[i].name ? ' selected' : '';
                        networkOptions += '<option' + isSlected + '>' + result[i].name + '</option>';
                    }
                }
                $('select', '#form-template-interface #networkID' + networkItemNum).append(networkOptions);
                networkItemNum += 1;
            };
            if(result && result.length > 0) {
                for(var i=0;i<origNetworks.length;i++) {
                    addInterfaceItem({
                        networkID : 'networkID' + networkItemNum,
                        networkV : origNetworks[i],
                        type : 'network'
                    });
                }
            }
            $('#template-edit-interface-add-button').button({
                icons: {
                    primary: 'ui-icon-plusthick'
                },
                text: false
            }).click(function(evt) {
                evt.preventDefault();
                addInterfaceItem({
                    networkID : 'networkID' + networkItemNum,
                    networkV : 'default',
                    type : 'network'
                });
            });
        };
        var initProcessor = function(){
            var setCPUValue = function(){
                if(!$('#cores').hasClass("invalid-field")&&$('#cores').val()!=""){
                    $("#cpus").val(parseInt($("#cores").val())*parseInt($("#threads").val()));
                }else{
                    $("#cpus").val('');
                }
            };
            $("input:text", "#form-template-processor").on('keyup', function(){
                $(this).toggleClass("invalid-field", !$(this).val().match('^[0-9]*$'));
                if($(this).prop('id')=='cores') setCPUValue();
            });
            $("input:checkbox", "#form-template-processor").click(function(){
                $(".topology", "#form-template-processor").toggleClass("hide", !$(this).prop("checked"));
                $("#cpus").attr("disabled", $(this).prop("checked"));
                setCPUValue();
            });
            $('select', '#form-template-processor').change(function(){
                setCPUValue();
            });
            kimchi.getCPUInfo(function(data){
                var options = "";
                for(var i=0;Math.pow(2,i)<=data.threads_per_core;i++){
                    var lastOne = Math.pow(2,i+1)>data.threads_per_core?" selected":"";
                    options += "<option"+lastOne+">"+Math.pow(2,i)+"</option>";
                }
                $('select', '#form-template-processor').append(options);
                if(template.cpus) $("#cpus").val(template.cpus);
                var topo = template.cpu_info.topology;
                if(topo&&topo.cores) $("#cores").val(topo.cores);
                if(topo&&topo.threads){
                    $('select', '#form-template-processor').val(topo.threads);
                    $("input:checkbox", "#form-template-processor").trigger('click');
                }
            });
        };
        kimchi.listNetworks(initInterface);
        kimchi.listStoragePools(initStorage);
        initProcessor();
    };
    kimchi.retrieveTemplate(kimchi.selectedTemplate, initTemplate);


    $('#tmpl-edit-button-save').on('click', function() {
        var editableFields = [ 'name', 'memory', 'disks', 'graphics'];
        var data = {};
        //Fix me: Only support one storage pool now
        var storages = $('.template-tab-body .item', '#form-template-storage');
        var tempName = $('.template-storage-name', storages).val();
        var tmpItem = $('#form-template-storage .item');
        tempName = tempName.split('/');
        var tempNameHead = tempName[0];
        var tempNameTail = tempNameHead;
        if($('.template-storage-type', tmpItem).val() === 'iscsi' || $('.template-storage-type', tmpItem).val() == 'scsi') {
            tempNameTail = tempName[tempName.length-1];
        }
        tempName = '/storagepools/' + tempNameHead;
        data['storagepool'] = tempName;
        $.each(editableFields, function(i, field) {
            /* Support only 1 disk at this moment */
            if (field == 'disks') {
                if($('.template-storage-type', tmpItem).val() === 'iscsi' || $('.template-storage-type', tmpItem).val() == 'scsi') {
                    origDisks[0]['size'] && delete origDisks[0]['size'];
                    origDisks[0]['volume'] = tempNameTail;
                } else {
                    origDisks[0]['volume'] && delete origDisks[0]['volume'];
                    origDisks[0].size = Number($('.template-storage-disk', tmpItem).val());
                }
                origDisks[0].format = $('.template-storage-disk-format', tmpItem).val();
                data[field] = origDisks;
            }
            else if (field == 'graphics') {
               var type = $('#form-template-general [name="' + field + '"]').val();
               data[field] = {'type': type};
            }
            else {
               data[field] = $('#form-template-general [name="' + field + '"]').val();
            }
        });
        data['memory'] = Number(data['memory']);
        data['cpus']   = parseInt($('#cpus').val());
        if($("input:checkbox", "#form-template-processor").prop("checked")){
            data['cpu_info'] = {
                topology: {
                    sockets: 1,
                    cores: parseInt($("#cores").val()),
                    threads: parseInt($("#threads").val())
                }
            };
        }else{
            data['cpu_info'] = {};
        }
        var networks = $('.template-tab-body .item', '#form-template-interface');
        var networkForUpdate = new Array();
        $.each(networks, function(index, networkEntities) {
            var thisValue = $('select', networkEntities).val();
            networkForUpdate.push(thisValue);
        });
        if (networkForUpdate instanceof Array) {
            data.networks = networkForUpdate;
        } else if (networkForUpdate != null) {
            data.networks = [networkForUpdate];
        } else {
            data.networks = [];
        }

        kimchi.updateTemplate($('#template-name').val(), data, function() {
            kimchi.doListTemplates();
            kimchi.window.close();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.doListTemplates = function() {
    kimchi.listTemplates(function(result) {
        if (result && result.length) {
            $('#noTemplates').hide();
            var listHtml = '';
            var templateHtml = $('#templateTmpl').html();
            $.each(result, function(index, value) {
                var isLocal;
                if(value.cdrom){
                    isLocal = /^\//.test(value['cdrom']);
                }else{
                    for(var i=0;i<value.disks.length;i++){
                        if(value.disks[i].base){
                            isLocal = /^\//.test(value.disks[i].base);
                            break;
                        }
                    }
                }
                if(isLocal){
                    value.location = "images/theme-default/icon-local.png";
                }else{
                    value.location = "images/theme-default/icon-remote.png";
                }
                listHtml += kimchi.substitute(templateHtml, value);
            });
            $('#templateList').html(listHtml);
            kimchi.templateBindClick();
        } else {
            $('#templateList').html('');
            $('#noTemplates').show();
        }
        $('html').removeClass('processing');
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
        $('html').removeClass('processing');
    });
};

kimchi.templateBindClick = function() {
    $('.template-edit').on('click', function(event) {
        var templateName = $(this).data('template');
        kimchi.selectedTemplate = templateName;
        kimchi.window.open("template-edit.html");
    });
    $('.template-clone').on('click', function(event) {
        kimchi.selectedTemplate = $(this).data('template');
        $('html').addClass('processing');
        kimchi.cloneTemplate(kimchi.selectedTemplate, function() {
                kimchi.doListTemplates();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
                kimchi.doListTemplates();
            });
    });
    $('.template-delete').on('click', function(event) {
        var $template = $(this);
        var settings = {
            title : i18n['KCHAPI6001M'],
            content : i18n['KCHTMPL6003M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };
        kimchi.confirm(settings, function() {
            var templateName = $template.data('template');
            kimchi.deleteTemplate(templateName, function() {
                kimchi.doListTemplates();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        }, function() {
        });
    });
}
kimchi.hideTitle = function() {
    $('#tempTitle').hide();
};

kimchi.template_main = function() {
    if(kimchi.tabMode['templates'] === 'admin') {
        $('.tools').attr('style','display');
        $("#template-add").on("click", function(event) {
            kimchi.window.open({
                url: 'template-add.html',
                close: function() {
                    if (kimchi.deepScanHandler) {
                        kimchi.deepScanHandler.stop = true;
                    }
                }
            });
        });
    }

    kimchi.doListTemplates();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * pub/sub
 * Usage:
 *   Publish - kimchi.topic('eventname').publish(params);
 *   Subscribe - kimchi.topic('eventname').subscribe(listener);
 *   Unsubscribe - kimchi.topic('eventname').unsubscribe(listener);
 */
kimchi.topic = (function() {

    var topics = {};

    return function( id ) {
        var callbacks,
            method,
            topic = id && topics[ id ];

        if ( !topic ) {
            callbacks = jQuery.Callbacks();
            topic = {
                publish: callbacks.fire,
                subscribe: callbacks.add,
                unsubscribe: callbacks.remove
            };
            if ( id ) {
                topics[ id ] = topic;
            }
        }
        return topic;
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.user = (function() {
    var getUserName = function() {
        return kimchi.cookie.get('username');
    };

    var setUserName = function(userName) {
        kimchi.cookie.set('username', userName, 365);
    };

    var showUser = function(toShow) {
        if (toShow) {
            var userName = getUserName();
            userName && $('#user-name').text(userName);
            $('#user').removeClass('not-logged-in');
            return;
        }

        $('#user').addClass('not-logged-in');
    };

    return {
        getUserName: getUserName,
        setUserName: setUserName,
        showUser: showUser
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * To check whether a given DOM node is entirely within the viewport
 * of a browser.
 * @param {DOMObject} el The given DOM node to check.
 *
 * @returns {true|false|undefined}
 *     true if the node is within viewport, or
 *     false if the node is not entirely visible, or
 *     undefined if the given parameter is invalid.
 */
kimchi.isElementInViewport = function(el) {
    if (!el || !el.getBoundingClientRect) {
        return undefined;
    }

    var rect = el.getBoundingClientRect();

    return (
        rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
        rect.right <= (window.innerWidth || document.documentElement.clientWidth)
    );
};

/**
 * To change the byte to proper unit.
 * @param number needed to change unit.
 * @param digits after the decimal point.
 * @returns str with unit.
 */
kimchi.changetoProperUnit = function(numOrg, digits, base) {
    if (numOrg === undefined) {
        return "";
    }
    var suffixes = [ 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'B' ];
    var base = base || 1024;
    var numTemp = numOrg;
    var result = numOrg.toFixed(digits) + 'B';
    for ( var i = 0; i < suffixes.length; i++) {
        var numTemp = numTemp / base;
        if (numTemp < 1)
            break;
        result = numTemp.toFixed(digits) + suffixes[i]
    }
    return result;
};

/**
 * kimchi.formatMeasurement util.
 *
 * Refer to "Units of information" (
 *   http://en.wikipedia.org/wiki/Units_of_information
 * ) for more information about measurement units.
 *
 * @param number The number to be normalized.
 * @param settings
 *     * base Measurement base, accepts 2 or 10. defaults to 2.
 *     * unit The unit of the measurement, e.g., B, Bytes/s, bps, etc.
 *     * fixed The number of digits after the decimal point.
 *
 * @returns [object]
 *     * v The number part of the measurement.
 *     * s The suffix part of the measurement including multiple and unit.
 *         e.g., kB/s means 1000B/s, KiB/s for 1024B/s.
 */
(function() {
    var unitBaseMapping = {
        2: [{
            us: 'Ki',
            v: 1024
        }, {
            us: 'Mi',
            v: 1048576
        }, {
            us: 'Gi',
            v: 1073741824
        }, {
            us: 'Ti',
            v: 1099511627776
        }, {
            us: 'Pi',
            v: 1125899906842624
        }],
        10: [{
            us: 'k',
            v: 1000
        }, {
            us: 'M',
            v: 1000000
        }, {
            us: 'G',
            v: 1000000000
        }, {
            us: 'T',
            v: 1000000000000
        }, {
            us: 'P',
            v: 1000000000000000
        }]
    };

    var Formatted = function(value, suffix) {
        this['v'] = value;
        this['s'] = suffix;
    };
    Formatted.prototype.toString = function() {
        return this['v'] + this['s'];
    };

    var format = function(number, settings) {
        if(number === (undefined || null) || isNaN(number)) {
            return number;
        }

        var settings = settings || {};
        var unit = settings['unit'] || 'B';
        var base = settings['base'] || 2;
        if(base !== 2 && base !== 10) {
            return new Formatted(number, unit);
        }

        var fixed = settings['fixed'];

        var unitMapping = unitBaseMapping[base];
        for(var i = unitMapping.length - 1; i >= 0; i--) {
            var mapping = unitMapping[i];
            var suffix = mapping['us'];
            var startingValue = mapping['v'];
            if(number < startingValue) {
                continue;
            }

            var formatted = number / startingValue;
            formatted = fixed ? formatted.toFixed(fixed) : formatted;

            return new Formatted(formatted, suffix + unit);
        }

        return new Formatted(fixed ? number.toFixed(fixed) : number, unit);
    };

    kimchi.formatMeasurement = format;
})();

kimchi.isUnsignedNumeric = function(number) {
    var reg = /^d+(.d+)?$/
    return reg.test(number);
}

kimchi.isServer = function(server) {
    var domain = "([0-9a-z_!~*'()-]+\.)*([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\.[a-z]{2,6}";
    var ip = "(\\d{1,3}\.){3}\\d{1,3}";
    regex = new RegExp('^' + domain + '|' + ip + '$');
    if (!regex.test(server)) {
        return false;
    } else {
        return true;
    }
};

kimchi.escapeStr = function(str) {
    if (str)
        return str.replace(/([ #;?%&,.+*~\\':"!^$[\]()<=>`{|}\/@])/g,'\\$&');

    return str;
};

kimchi.urlSafeB64Decode = function(str) {
    return atob(str.replace(/-/g, '+').replace(/_/g, '/'));
}

kimchi.urlSafeB64Encode = function(str) {
    return btoa(str).replace(/\+/g, '-').replace(/\//g, '_');
}
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013-2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.window = (function() {
    var _windows = [];
    var _listeners = {};
    var open = function(settings) {
        var settings = jQuery.type(settings) === 'object' ? settings : {
            url: settings
        };

        var windowID = settings['id'] || 'window-' + _windows.length;

        if ($('#' + windowID).length) {
            $('#' + windowID).remove();
        }

        _windows.push(windowID);
        _listeners[windowID] = settings['close'];
        var windowNode = $('<div></div>', {
            id: windowID,
            'class': settings['class'] ? settings['class'] + ' bgmask remove-when-logged-off' : 'bgmask remove-when-logged-off'
        });

        $(windowNode).css(settings['style'] || '');

        $(windowNode).appendTo('body').on('click', '.window .close', function() {
            kimchi.window.close();
        });

        if (settings['url']) {
            $(windowNode).load(settings['url']).fadeIn(100);
            return;
        }

        settings['content'] && $(windowNode).html(settings['content']);
    };

    var close = function() {
        var windowID = _windows.pop();
        if(_listeners[windowID]) {
            _listeners[windowID]();
            _listeners[windowID] = null;
        }
        delete _listeners[windowID];

        $('#' + windowID).fadeOut(100, function() {
            $(this).remove();
        });
    };

    return {
        open: open,
        close: close
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.dropdownButton', {

        _create : function() {
            this.actionDiv = this.element;
            this.actionDiv.addClass('btn dropdown popable');
            this.actionDiv.find('div').addClass('popover');
            this.actionDiv.find('button').addClass('button-big');
        },

        destroy : function() {
            this.actionDiv.removeClass('btn dropdown popable');
            this.actionDiv.find('div').removeClass('popover');
            this.actionDiv.find('button').removeClass('button-big');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 (function($) {
    $.widget('kimchi.circleGauge', {

        options : {
            color : '#87C004',
            fillColor : '#87C004',
            lineWidth : 20,
            shadowSize : '2px',
            font : 'bold 13px Geneva, sans-serif',
            textAlign : 'center',
            radius : 35,
            peakRate : 100,
            display : 0,
            circle : 0,
            label : ''
        },

        _create : function() {
            //valuesAttr="{" + this.element.data('value')+ "}";
            //console.info(valuesAttr);
            //values=eval("(" + valuesAttr + ")");
            //$.extend(this.options, values);
            this.options.display=this.element.data('display');
            this.options.percentage=this.element.data('percentage');
            this._fixupPeakRate();
            this._draw();
        },

        setValues : function(values) {
            $.extend(this.options, values);
            this._fixupPeakRate();
            this._draw();
        },

        _fixupPeakRate : function() {
            if (this.options.circle>this.options.peakRate) {
                this.options.peakRate=this.options.circle;
            }
        },

        _draw : function() {
            this.element.empty();
            var canvas = document.createElement('canvas');
            //this.element.append($(canvas));  //I don't quite understand this line so trying the one below...
            this.element.append(canvas);

            var ctx = canvas.getContext('2d');
            var radius = this.options.radius;

            var shadowSize = 2;
            var width = height = radius * 2;
            $(canvas).attr('height', height);
            $(canvas).attr('width', width);

            $(canvas).css({
                'boxShadow' : shadowSize + 'px ' + shadowSize + 'px ' + shadowSize + 'px #fff, -' + shadowSize + 'px -' + shadowSize + 'px ' + shadowSize + 'px #eaeaea',
                borderRadius : radius + 'px'
            });

            ctx.clearRect(0, 0, width, height);
            ctx.fillStyle = this.options.fillColor;
            ctx.font = this.options.font;
            ctx.textAlign = 'center';
            var originPos = radius;
            ctx.textBaseline = 'middle';
            ctx.fillText(this.options.display, originPos, originPos);
            ctx.strokeStyle = this.options.color;
            ctx.lineWidth = this.options.lineWidth;
            ctx.beginPath();
            ctx.arc(originPos, originPos, radius, -.5 * Math.PI, (this.options.percentage / 50 - .5) * Math.PI);
            ctx.stroke();
        },

        destroy : function() {
            this.element.empty();
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));

kimchi.circleGauge = function(selector) {
    $(selector).circleGauge();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.combobox', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    $.each(items, function(index, item) {
                        that.listControl.append(item);
                    })
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (temp > 0 && that.listControl.html() !== '') {
                        that._open();
                    } else {
                        that._close();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.filterselect', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                        that.target.removeClass("invalid-field");
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    if (items.length === 0) {
                        that.listControl.html(i18n['KCHAPI6005E']);
                    } else {
                        $.each(items, function(index, item) {
                            that.listControl.append(item);
                        });
                    }
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (that.listControl.html() === '') {
                        that.target.addClass("invalid-field");
                        that.listControl.html(i18n['KCHAPI6005E']);
                    } else {
                        that.target.removeClass("invalid-field");
                    }
                    if (temp > 0) {
                        that._open();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover')
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2014
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.selectMenu', {

        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.label = this.selectDiv.find('span').first();
            this.selectDiv.addClass('btn dropdown popable');
            this.target.addClass('input');
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function (options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            var item;
            that.listControl.find('li').remove();
            that.label.text("");
            that.target.val("");
            if (options.length > 0) {
                $.each(options, function(index, option) {
                    item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                    item.data('value', option.value);
                    if(option.value === value) {
                        item.addClass(selectedClass);
                        that.label.text(option.label);
                        that.target.val(option.value);
                    }
                    that.listControl.append(item);
                });
                that.listControl.on('click', itemTag, function() {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    that.label.text($(this).text());
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if(oldValue !== newValue) {
                        that.target.change();
                    }
                });
            }
        },

        value : function(data) {
            if (data === undefined) {
                return this.target.val();
            }
            this.target.val(data.value);
            this.label.val(data.label);
        },

        destroy : function() {
            this.selectDiv.removeClass('btn dropdown popable');
            this.target.removeClass('input');
            this.label.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
